#ifndef __ICSTUDIOCS_H_INCLUDED__
#define __ICSTUDIOCS_H_INCLUDED__

#include "csutil/scf.h"

struct iEngine;
struct iMaterialWrapper;
struct iLoader;
struct iSaver;
struct iGraphics3D;
struct iGraphics2D;
struct iKeyboardDriver;
struct iVirtualClock;
struct iEventQueue;
struct iVFS;
struct iEngineSequenceManager;
struct iObjectRegistry;
struct iSector;
struct icssCommandManager;
struct icssPlugin;
struct icssEditModeManager;
struct icssSelection;

class wxGauge;
class wxFrame;



SCF_VERSION(icssRedrawViewCallback , 0, 0, 1);
struct icssRedrawViewCallback : public iBase
{
  virtual void Redraw() = 0;
};

SCF_VERSION(icssActiveSectorCallback, 0, 0, 1);
struct icssActiveSectorCallback : public iBase
{
  virtual void OnActiveSectorChanged() = 0;
};

SCF_VERSION(icssContentCallback, 0, 0, 1);
struct icssContentCallback : public iBase
{
  virtual void OnMeshListChanged() = 0;
  virtual void OnSectorListChanged() = 0;
  virtual void OnMaterialListChanged() = 0;
};

SCF_VERSION(iCStudioCS, 0,0,1);
struct iCStudioCS : public iBase
{
  virtual iObjectRegistry* getObjectRegistry()= 0;
  	
  /// Active Sector
  virtual iSector* getActiveSector()= 0;
  virtual void SetActiveSector(iSector *sector) = 0;
  virtual void RegisterActiveSectorCB(icssActiveSectorCallback *cb) = 0;
  virtual void UnregisterAcitveSectorCB(icssActiveSectorCallback *cb) = 0;

  /// Active Material
  virtual iMaterialWrapper* GetActiveMaterial() = 0;
  virtual void SetActiveMaterial(iMaterialWrapper *mat) = 0;

  /// Plugin
  virtual csRef<icssPlugin> GetPlugin(const char *name) = 0;
  virtual size_t GetPluginCount() = 0;
  virtual csRef<icssPlugin> GetPlugin(size_t idx) = 0;

  /**
  @brief Registers a content callback

  @param cb pointer to callback to add 
  */
  virtual void RegisterContentCB(icssContentCallback *cb) = 0;

  /**
  @brief Unregisters a content changed callback

  @param cb callback to remove 
  */
  virtual void UnregisterContentCB(icssContentCallback *cb) = 0;
  
  
  
  virtual void RegisterRedrawViewCB(icssRedrawViewCallback *cb) = 0;
  virtual void UnregisterRedrawViewCB(icssRedrawViewCallback *cb) = 0;

  /**
  @brief Loads a Map from a file

  @param path  path to the file (vfs changes to that)
  @param file  filename of file containing the map
  @param gauge loadmap will control the gauge during loading  

  @return true on success, false on error
  */
  virtual bool LoadMap(char *path, char* file, wxGauge *gauge = NULL) = 0;

  /**
  @brief Save the current map to a specific file

  @param path  directory of the file (vfs changes to that)
  @param file  name of the file

  @return True/False depending on success
  */
  virtual bool SaveMap(char *path, char* file) = 0;

  /**
  @brief print a message to the console

  @param the string to print
  */
  virtual void Print(const char *msg) = 0;
  virtual void Printf(const char *format, ...) = 0;

  /**
  @brief returns the view3d frame around the wxgl canvas
  */
  virtual wxFrame *GetView3DFrame() = 0;

  /**
  @brief redraws all views registered with RegisterRedrawViewCB()
  */
  virtual void RedrawViews() = 0;  
};


#endif 
