#ifndef __ICSSDRAWER_H_INCLUDED__
#define __ICSSDRAWER_H_INCLUDED__

#include "csutil/scf.h"
#include "csutil/cscolor.h"
#include "csgeom/vector3.h"

struct iMeshWrapper;
struct iLight;

SCF_VERSION (icssDrawer, 0, 0, 1);

struct icssDrawer : public iBase
{
  /*
  * Sets the active color to draw with
  */
  virtual void SetColor(csColor4 &c) = 0;

  /*
  * Get the current active drawing color
  */ 
  virtual csColor4 &GetColor() = 0;
 
  /*
  * Set Line Stipple 
  * factor and pattern same as in GL
  * factor == 0 deactivates line stippling
  */
  virtual void SetLineStipple(int factor, unsigned short pattern) = 0;

  /*
  * Get Line Stipple
  */
  virtual void GetLineStipple(int &ofactor, unsigned short &opatter) = 0;

  /*
  * Set Line Width
  */
  virtual void SetLineWidth (float width) = 0;
  
  /*
  * Get Line Width
  */
  virtual float GetLineWidth () = 0;

  /*
  * Set active Point Size
  */
  virtual void SetPointSize (float size) = 0;
  
  /*
  * Get active point Size
  */
  virtual float GetPointSize () = 0;
  /*
  * Draws a mesh in wireframe mode
  */
  virtual void DrawMesh (iMeshWrapper *m) = 0;

  /*
  * Draw a light
  */
  virtual void DrawLight(iLight *light) = 0;
  
  /*
  * Draw any object the drawer is capable of
  */
  virtual void DrawObject(iBase *object) = 0;
  
  /*
  * Draws a axis aligned bounding box 
  */ 
  virtual void DrawAABB (csVector3 &min, csVector3 &max) = 0;

  /*
  * Draw Point
  */
  virtual void DrawPoint (csVector3 &p) = 0;

  /*
  * Draw Circle
  */  
  virtual void DrawCircle (csVector3 &m, float r, int segcount = 12) = 0;

  /*
  * Draw Line
  */
  virtual void DrawLine (csVector3 &s, csVector3 &e) = 0;
};

#endif
