#ifndef __ICSSSELECTION_H_INCLUDED__
#define __ICSSSELECTION_H_INCLUDED__

#include "csutil/scf.h"
#include "csutil/refarr.h"

struct iSector;
struct iMeshWrapper;

/*
How Selection works.

Main interface will be icssSelection which will 
implemenent multiple specific selections like 
icssMeshSelection, icssPortalSelection etc.

*/



SCF_VERSION(icssMeshSelection, 0, 0, 1);
/* icssMeshSelection takes care of mesh selection

all selected meshes need to be part of the current active
sector

*/
struct icssMeshSelection : public iBase
{
  /// Clear mesh selection
  virtual void Clear () = 0;

  /// Add a iMeshWrapper to the selection
  /// returns the index in the mesh selection
  /// This does a push smart, so double selection is
  /// not possible
  virtual size_t Add (iMeshWrapper *mwrap) = 0;

  /// Remove a iMeshWrapper from the selection
  virtual void Remove (iMeshWrapper *mwrap) = 0;
  virtual void Remove (size_t index) = 0;

  /// Check if the mesh is in the selection
  virtual bool Contains (iMeshWrapper *mwrap) = 0;

  /// Get number of selected meshes
  virtual size_t GetCount () = 0;

  /// Get the Meshwrapper at index..
  /// returns 0 if not in selection
  virtual iMeshWrapper* Get(size_t i) = 0;

	/// Save selection to an iBase array
	virtual void SaveToArray(csRefArray<iBase> &target) = 0;
	/// Load selection from an iBase array
	virtual void LoadFromArray(csRefArray<iBase> &source) = 0;


};

SCF_VERSION(icssSelectionCallback, 0, 0, 1);

struct icssSelectionCallback : public iBase
{
  /// called if selection changed
  virtual void OnSelectionChanged() = 0;
};


SCF_VERSION(icssSelection, 0, 0, 1);
struct icssSelection : public iBase
{
  /// Clear the current selection 
  /// -> nothing will be selected
  virtual void Clear () = 0;

  /// save selection so that it can be loaded
  /// (loaded in same session => memory pointers stay same)
  virtual void Save (const char *name) = 0;

  /// load selection
  virtual void Load (const char *name) = 0;

  /// Callback system
  virtual void RegisterCB(icssSelectionCallback *cb) = 0;
  virtual void UnregisterCB(icssSelectionCallback *cb) = 0;
  
	/// remove from selection, no matter what kind
	virtual void Remove(iBase *obj) = 0;
	/// add to the selection, no matter what kind
	virtual void Add(iBase *obj) = 0;
  /// check wether the item is selected or not
  virtual bool Contains(iBase *obj) = 0;
	/// Save selection to an iBase array
	virtual void SaveToArray(csRefArray<iBase> &target) = 0;
	/// Load selection from an iBase array
	virtual void LoadFromArray(csRefArray<iBase> &source) = 0;

};


#endif

