#include "cssysdef.h"

#include "actionmgr.h"

SCF_IMPLEMENT_IBASE(cssActionManagerPlugin)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssActionManager)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssActionManagerPlugin::cssActionManager)
SCF_IMPLEMENTS_INTERFACE(icssActionManager)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY(cssActionManagerPlugin);

CS_IMPLEMENT_PLUGIN;

cssActionManagerPlugin::cssActionManagerPlugin(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssActionManager);
}

cssActionManagerPlugin::~cssActionManagerPlugin()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssActionManager);
  SCF_DESTRUCT_IBASE();
}

//// icssPlugin implementation
bool cssActionManagerPlugin::Initialize(iCStudioCS *csi)
{
	m_csi = csi;
	return true;
}

const char * cssActionManagerPlugin::GetName()
{
	return "cstudio.plugin.actionmgr";
}

//// icssActionManager implementation
bool cssActionManagerPlugin::Perform(icssAction *toperform)
{
    csString msg;
	bool ret_val = toperform->Do();
	if(ret_val)
	{
		m_done.Push(toperform);
		m_undone.DeleteAll();

		msg<<"Done action: "<<toperform->GetDescription()<<".\n";
		m_csi->Print(msg);
	}
	else
	{
      msg<<"Failed to perform action: "<<toperform->GetDescription()<<".\n";
      m_csi->Print(msg);
    }
	return ret_val;
}

bool cssActionManagerPlugin::Undo()
{
  csString msg;
  csRef<icssAction> a = m_done.Pop();
  bool ret_val = a->Undo();
  if(ret_val)
  {
    m_undone.Push(a);
    msg<<"Undone action: "<<a->GetDescription()<<".\n";
    m_csi->Print(msg);
  }
  else
  {
    msg<<"Failed undoing action: "<<a->GetDescription()<<".\n";
    m_csi->Print(msg);
  }
  return ret_val;
}

bool cssActionManagerPlugin::Redo()
{
  csString msg;
  if(m_undone.Length() > 0)
  {
    csRef<icssAction> a = m_undone.Pop();
	bool ret_val = a->Redo();
	if(ret_val)
	{
      m_done.Push(a);
      msg<<"Redone action: "<<a->GetDescription()<<".\n";
      m_csi->Print(msg);
    }
    else
    {
      msg<<"Failed redoing action: "<<a->GetDescription()<<".\n";
      m_csi->Print(msg);
    }          

	return ret_val;
  }
  return true;
}

void cssActionManagerPlugin::Clear()
{
	m_undone.DeleteAll();
	m_done.DeleteAll();
}

icssAction * cssActionManagerPlugin::GetLastAction()
{
  size_t count = m_done.Length();
	if(count > 0)
	{
		csRef<icssAction> a = m_done.Get(count - 1);
		return a;
	}

	return 0;
}
