/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"

#include "iutil/stringarray.h"
#include "csutil/csstring.h"

#include "app/icsstudio.h"
#include "plugins/icsscommandmanager.h"

#include "console.h"
#include "consolewnd.h"



CS_IMPLEMENT_PLUGIN;

SCF_IMPLEMENT_FACTORY (cssConsolePlugin);

SCF_IMPLEMENT_IBASE(cssConsolePlugin )
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssCommandManager)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssTextOutput)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssCreateView)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssConsolePlugin::cssCommandManager)
SCF_IMPLEMENTS_INTERFACE(icssCommandManager)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssConsolePlugin::eicssTextOutput)
SCF_IMPLEMENTS_INTERFACE(icssTextOutput)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssConsolePlugin::eicssCreateView)
SCF_IMPLEMENTS_INTERFACE(icssCreateView)
SCF_IMPLEMENT_EMBEDDED_IBASE_END


cssConsolePlugin::cssConsolePlugin (iBase *parent) : consolewnd(0)
{
  SCF_CONSTRUCT_IBASE (parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE (scficssCommandManager);
  SCF_CONSTRUCT_EMBEDDED_IBASE (scficssTextOutput);
  SCF_CONSTRUCT_EMBEDDED_IBASE (scficssCreateView);
}
  
cssConsolePlugin::~cssConsolePlugin ()
{
  SCF_DESTRUCT_EMBEDDED_IBASE (scficssCommandManager);
  SCF_DESTRUCT_EMBEDDED_IBASE (scficssTextOutput);
  SCF_DESTRUCT_EMBEDDED_IBASE (scficssCreateView);
  SCF_DESTRUCT_IBASE ();
}

  
bool cssConsolePlugin::Initialize(iCStudioCS *csi)
{
  m_csi = csi;

  LoadCommands();
  return true;
}

void cssConsolePlugin::RegisterCommand(const char *cmd, icssCommand *cb)
{
  m_cmds.Put(cmd, cb);
}

void cssConsolePlugin::UnregisterCommand(const char *cmd)
{
  m_cmds.Delete(cmd, m_cmds.Get(cmd, 0));
}

bool cssConsolePlugin::Execute (const char *cmdline)
{
  csStringArray *strarr = new csStringArray();
  FilterArgs(cmdline, strarr);

  csRef<icssCommand> cmd = m_cmds.Get(strarr->Get(0), 0);
  if(!cmd)
  {
    delete strarr;
    if(consolewnd)
      consolewnd->Print("Error: Command not found.\n");
    return false; // couldn't find command
  }
 

  bool rval = cmd->OnCommand(strarr);
  delete strarr;
  return rval;
}

icssCommand *cssConsolePlugin::GetCommand(const char *cmd)
{
  return m_cmds.Get(cmd, 0);
}


void cssConsolePlugin::LoadCommands()
{
  csRef<iStringArray> cl = iSCF::SCF->QueryClassList("cstudio.cmd.");
  size_t count = cl->Length();
  for(size_t i = 0; i < count; i++)
  {
    if(strcmp(cl->Get(i), "cstudio.cmd.console") == 0) // Ingore our self
      continue; 

    const char *c = cl->Get(i);
    csRef<icssPlugin> cmdplug = m_csi->GetPlugin(c);
    
    if(!cmdplug.IsValid())
      continue;

    csRef<icssCommandList> cmdlist = SCF_QUERY_INTERFACE(cmdplug, icssCommandList);
    if(!cmdlist)
      continue;

    m_cmdlists.Push(cmdlist);

    size_t ncmds = cmdlist->GetCount();
    for(size_t i = 0; i < ncmds; i++)
    {
      csRef<icssCommand> cmd = cmdlist->Get(i);
      RegisterCommand(cmd->GetCmdName(), cmd);
      
    }
  }
}
 
void cssConsolePlugin::FilterArgs (const char *cmdline, csStringArray *args)
{
  char *c;
  for(c = (char*)cmdline; *c == ' '; c++); // remove whitespaces at beginning

  char *l;
  l = c;

  for(c = (char*)cmdline; *c != '\0'; c++)
  {
    if(*c == ' ') 
    {
      for(;*l == ' ' && l < c; l++); // skip spaces
      
      if(l == c)
        continue;

      // we found a new arg

      csString s;
      s.Append(l, c - l);
      args->Push(s);

      l = c;
    }
  }

  for(;*l == ' ' && l; l++); // skip spaces

  if(l < c)
  {
    csString s;
    s.Append(l, c - l);
    args->Push(s);
  }
 
}

