/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __CMDMGR_H_INCLUDED__
#define __CMDMGR_H_INCLUDED__

#include "csutil/hash.h"
#include "csutil/ref.h"
#include "csutil/refarr.h"
#include "csutil/csstring.h"
#include "csutil/stringarray.h"

#include "plugins/icssplugin.h"
#include "plugins/icsscommandmanager.h"
#include "plugins/icsscreateview.h"
#include "plugins/icsstextoutput.h"

#include "consolewnd.h"


struct iCStudioCS;

class cssConsolePlugin : public icssPlugin 
{
  iCStudioCS* m_csi;
  
  csHash<csRef<icssCommand>, csStrKey> m_cmds;

  csRefArray<icssCommandList> m_cmdlists;
public:

  cssConsoleWindow *consolewnd;

  cssConsolePlugin (iBase *parent);
  virtual ~cssConsolePlugin ();

  /// icssPlugin functions
  virtual bool Initialize(iCStudioCS *csi);
	virtual const char *GetName()
	{ return "cstudio.cmd.console"; };

  /// returns argc
  void FilterArgs(const char *cmdline, csStringArray *args);
 
  void LoadCommands();
  SCF_DECLARE_IBASE;
  
  /// icssCommandManager implementation
  void RegisterCommand (const char *cmd, icssCommand *cb);
  void UnregisterCommand (const char *cmd);
  bool Execute (const char *cmdline);
  icssCommand *GetCommand(const char *cmd);
  struct cssCommandManager : public icssCommandManager
  {
    SCF_DECLARE_EMBEDDED_IBASE (cssConsolePlugin);
    virtual void RegisterCommand(const char *cmd, icssCommand *cb)
    { scfParent->RegisterCommand(cmd, cb);    };
    virtual void UnregisterCommand(const char *cmd)
    { scfParent->UnregisterCommand(cmd); };
    virtual bool Execute(const char *cmdline)
    { return scfParent->Execute(cmdline);   };
    virtual icssCommand *GetCommand(const char *cmd)
    { return scfParent->GetCommand(cmd); };
  } scficssCommandManager;
  friend struct cssCommandManager;
  
  struct eicssTextOutput : public icssTextOutput
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssConsolePlugin);
    virtual void Print(const char *text)
    {
      if(scfParent->consolewnd)
        scfParent->consolewnd->Print(text);
    }
    virtual void Clear()
    {
      if(scfParent->consolewnd)
        scfParent->consolewnd->Clear();
    }
  } scficssTextOutput;
  friend struct eicssTextOutput;
  
  /// icssView embedded interface
  struct eicssCreateView : public icssCreateView
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssConsolePlugin);
    virtual wxWindow * Create(wxWindow *parent,
        const wxChar *title, int posx, int posy, int sizex, int sizey)
      { return new cssConsoleWindow(scfParent, parent, posx, posy, sizex, sizey); }     
    virtual bool SaveXML(wxWindow *wnd, iDocumentNode *parent) 
      { return true; }
    virtual bool LoadXML(wxWindow *wnd, iDocumentNode *parent)
      { return true; }
   } scficssCreateView;
  friend struct eicssCreateView;  


};


#endif

