/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __CREATE_LIGHT_H_INCLUDED__
#define __CREATE_LIGHT_H_INCLUDED__

#include "plugins/icsscommandmanager.h"

#include "csutil/stringarray.h"
#include "csgeom/vector3.h"
#include "csutil/cscolor.h"
#include "iengine/engine.h"
#include "iengine/light.h"
#include "iengine/sector.h"
#include "iengine/region.h"
#include "action/icssaction.h"
#include "action/icsscreatelight.h"
#include "plugins/icssactionmanager.h"

class cssCreateLightCommand : public icssCommand
{
  iCStudioCS* m_csi;
  csRef<icssActionManager> m_actionmgr;
public:
  cssCreateLightCommand(iCStudioCS *csi) 
  { 
    SCF_CONSTRUCT_IBASE(0);
    m_csi = csi;
		csRef<icssPlugin> amplug = m_csi->GetPlugin("cstudio.plugin.actionmgr");
    if(amplug)
      m_actionmgr = SCF_QUERY_INTERFACE(amplug, icssActionManager);
  };

  virtual ~cssCreateLightCommand() 
  {
    SCF_DESTRUCT_IBASE ();
  };

  SCF_DECLARE_IBASE;

  virtual const char * GetCmdName()
  {
    return "create.light";
  };

  virtual const char * GetHelpLine()
  {
    return "create a new light";
  };

  virtual const char * GetHelpDesc()
  {
    return "create a new light\ncreate.light \t [name] [<0,0,0>] [10] [(0,0,0)] [0]";
  };

  virtual bool OnCommand (csStringArray *args)
  {
    m_csi->Print("Create Light .... ");

    if (args->Length() < 6)
    {
      m_csi->Print("failed\n");
      return false;
    }

    float x, y, z;
    sscanf(args->Get(2), "<%f,%f,%f>", &x, &y, &z);
    csVector3 pos(x,y,z);

    float radius;
    sscanf(args->Get(3), "%f", &radius);

    sscanf(args->Get(5), "(%f,%f,%f)", &x, &y, &z);
    csColor color(x,y,z);

    int dyntype;
    sscanf(args->Get(5), "%d", &dyntype);
		
    csRef<icssAction> a = SCF_CREATE_INSTANCE("cstudio.action.createlight", icssAction);
    if(!a)
      return false; /// @@@@@@@@@@@@@@@ ERROR MSG ??

    if(!a->Initialize(m_csi))
      return false;

    csRef<icssCreateLightAction> cla = SCF_QUERY_INTERFACE(a, icssCreateLightAction);
    if(!cla)
      return false;

    cla->SetParams(args->Get(1), pos, radius, color);

    m_csi->Print("performing create light action..");
    if(!m_actionmgr->Perform(a))
    {
      m_csi->Print(".....failed!");
      return false;
    }
		
    m_csi->Print(".....done\n");
    return true; // success
  };
};

SCF_IMPLEMENT_IBASE (cssCreateLightCommand)
SCF_IMPLEMENTS_INTERFACE (icssCommand)
SCF_IMPLEMENT_IBASE_END

#endif

