#include "cssysdef.h"

#include "createlightaction.h"
#include "app/icsstudio.h"
#include "csutil/objreg.h"
#include "iengine/engine.h"
#include "iengine/sector.h"

SCF_IMPLEMENT_IBASE(cssCreateLightAction)
SCF_IMPLEMENTS_INTERFACE(icssAction)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssCreateLightAction)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssCreateLightAction::cssCreateLightActionI)
SCF_IMPLEMENTS_INTERFACE(icssCreateLightAction)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

cssCreateLightAction::cssCreateLightAction(iBase *parent)
{
	SCF_CONSTRUCT_IBASE(parent);
	SCF_CONSTRUCT_EMBEDDED_IBASE(scficssCreateLightAction);
}

cssCreateLightAction::~cssCreateLightAction()
{
	SCF_DESTRUCT_EMBEDDED_IBASE(scficssCreateLightAction);
	SCF_DESTRUCT_IBASE();
}

bool cssCreateLightAction::Initialize(iCStudioCS *csi)
{
	m_csi = csi;
	return true;
}

bool cssCreateLightAction::Do()
{
  	csRef<iEngine> engine = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);
  	if(!engine)
    		return false;

  	csRef<iSector> activesector = m_csi->getActiveSector();
  	if(!activesector)
    		return false;

  	light = engine->CreateLight(name, pos, radius, color);
  	if(!light)
    		return false;

  	activesector->GetLights()->Add(light);
  	return true;

}

bool cssCreateLightAction::Undo()
{
  	if(!m_csi)
    		return false;

  	csRef<iEngine> engine = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);
  	if(!engine)
    		return false;

  	csRef<iSector> activesector = m_csi->getActiveSector();
  	if(!activesector)
    		return false;

  	activesector->GetLights()->Remove(light);

  	return true;
}

bool cssCreateLightAction::Redo()
{
  	if(!light)
  	{
    		if(Do())
      			return true;
  	}
 
  	return false;
}

	//// icssCreateLightAction interface
void cssCreateLightAction::SetParams(const char* name, const csVector3& pos,
  float radius, const csColor& color)
{
  	this->name = name;
  	this->pos = pos;
  	this->radius = radius;
  	this->color = color;
}

void cssCreateLightAction::SetName(const char *name)
{
  	this->name = name;
}

void cssCreateLightAction::SetPosition(const csVector3 & pos)
{
  	this->pos = pos;
}

void cssCreateLightAction::SetRadius(float radius)
{
  	this->radius = radius;
}

void cssCreateLightAction::SetColor(const csColor &color)
{
  	this->color = color;
}
