/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __HELP_H_INCLUDED__
#define __HELP_H_INCLUDED__

#include "plugins/icsscommandmanager.h"

#include "csutil/stringarray.h"

class cssHelpCommand : public icssCommand
{
  iCStudioCS* m_csi;
public:
  cssHelpCommand(iCStudioCS *csi) 
  { 
    SCF_CONSTRUCT_IBASE(0);
    m_csi = csi; 
  };
  virtual ~cssHelpCommand() 
  {
    SCF_DESTRUCT_IBASE ();
  };

  SCF_DECLARE_IBASE;

  virtual const char * GetCmdName()
  {
    return "help";
  };

  virtual const char * GetHelpLine()
  {
    return "help <command>\n";
  };

  virtual const char * GetHelpDesc()
  {
    return "help will try to help you with a specific command\n";
  };

  virtual bool OnCommand (csStringArray *args)
  {
    /// need a command name as a param
    if(args->Length() < 2)
    {
      PrintHelp();
      return true;
    }
  
    csRef<icssCommandManager> cmdmgr = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), icssCommandManager);
    if(cmdmgr)
    {
      csRef<icssCommand> cmd = cmdmgr->GetCommand(args->Get(1));

      m_csi->Print(cmd->GetHelpDesc());
      return true; // success
    }

     return false; // some error
  };

  void PrintHelp()
  {
    m_csi->Print("A list of commands...\n");
    /// TBD...
  };
};

SCF_IMPLEMENT_IBASE (cssHelpCommand)
SCF_IMPLEMENTS_INTERFACE (icssCommand)
SCF_IMPLEMENT_IBASE_END

#endif

