/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __RELIGHT_H_INCLUDED__
#define __RELIGHT_H_INCLUDED__

#include "plugins/icsscommandmanager.h"

#include "csutil/stringarray.h"
#include "iengine/engine.h"
#include "iengine/light.h"
#include "iengine/sector.h"
#include "iengine/region.h"

class cssRelightCommand : public icssCommand
{
  iCStudioCS* m_csi;
  csRef<iEngine> m_engine;

public:
  cssRelightCommand(iCStudioCS *csi) 
  { 
    SCF_CONSTRUCT_IBASE(0);
    m_csi = csi;
    m_engine = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);
  };
  virtual ~cssRelightCommand() 
  {
    SCF_DESTRUCT_IBASE ();
  };

  SCF_DECLARE_IBASE;

  virtual const char * GetCmdName()
  {
    return "relight";
  };

  virtual const char * GetHelpLine()
  {
    return "recalculate lights";
  };

  virtual const char * GetHelpDesc()
  {
    return "recalculate lights\nrelight \t [light] [region]";
  };

  virtual bool OnCommand (csStringArray *args)
  {
    m_csi->Print("Relighting .... ");

    if (args->Length() == 1)
    {
      m_engine->ForceRelight();
      m_csi->Print("done\n");
      return true;
    }
    
    iLight* light = m_engine->FindLight(args->Get(1));
    if (light)
    {
      iRegion* region = m_engine->GetRegions()->FindByName(args->Get(2));
      if (region)
      {
        m_engine->ForceRelight(light, region);
        m_csi->Print("done\n");
        return true;
      }
      m_engine->ForceRelight(light);
      m_csi->Print("done\n");
      return true;
    }
    
    iRegion* region = m_engine->GetRegions()->FindByName(args->Get(1));
    if (region)
    {
      m_engine->ForceRelight(region);
      m_csi->Print("done\n");
      return true;
    }
    
    m_csi->Print("failed\n");
    return false;
  };
};

SCF_IMPLEMENT_IBASE (cssRelightCommand)
SCF_IMPLEMENTS_INTERFACE (icssCommand)
SCF_IMPLEMENT_IBASE_END

#endif
