/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __REMOVE_LIGHT_H_INCLUDED__
#define __REMOVE_LIGHT_H_INCLUDED__

#include "plugins/icsscommandmanager.h"

#include "csutil/stringarray.h"
#include "iengine/engine.h"
#include "iengine/light.h"
#include "iengine/sector.h"
#include "iengine/region.h"

class cssRemoveLightCommand : public icssCommand
{
  iCStudioCS* m_csi;
  csRef<iEngine> m_engine;
public:
  cssRemoveLightCommand(iCStudioCS *csi) 
  { 
    SCF_CONSTRUCT_IBASE(0);
    m_csi = csi;
    m_engine = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);
  };
  virtual ~cssRemoveLightCommand() 
  {
    SCF_DESTRUCT_IBASE ();
  };

  SCF_DECLARE_IBASE;

  virtual const char * GetCmdName()
  {
    return "remove.light";
  };

  virtual const char * GetHelpLine()
  {
    return "remove a light source";
  };

  virtual const char * GetHelpDesc()
  {
    return "remove a light source\nremove \t light [name]";
  };

  virtual bool OnCommand (csStringArray *args)
  {
    m_csi->Print("Remove Light .... ");

    if (args->Length() != 2)
    {
      m_csi->Print("failed\n");
      return false;
    }

    iLight* light = m_engine->FindLight(args->Get(1));
    if (!light)
    {
      m_csi->Print("failed\n");
      return false;
    }

    m_engine->RemoveLight(light);

    m_csi->Print("done\n");
    return true; // success
  };
};

SCF_IMPLEMENT_IBASE (cssRemoveLightCommand)
SCF_IMPLEMENTS_INTERFACE (icssCommand)
SCF_IMPLEMENT_IBASE_END

#endif
