/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"

#include "stdcmds.h"

#include "app/icsstudio.h"
#include "about.h"
#include "help.h"
#include "createlight.h"
#include "relight.h"
#include "removelight.h"

CS_IMPLEMENT_PLUGIN;

SCF_IMPLEMENT_IBASE(cssStdCommandsPlugin)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssCommandList)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssStdCommandsPlugin::cssCommandList)
SCF_IMPLEMENTS_INTERFACE(icssCommandList)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY(cssStdCommandsPlugin);

cssStdCommandsPlugin::cssStdCommandsPlugin(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssCommandList);
}
  
cssStdCommandsPlugin::~cssStdCommandsPlugin()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssCommandList);
  SCF_DESTRUCT_IBASE();
}


bool cssStdCommandsPlugin::Initialize(iCStudioCS *csi)
{
  m_csi = csi;

  csRef<icssCommand> tmp;

  tmp = new cssAboutCommand(m_csi);
  m_cmds.Push(tmp);
  tmp = new cssHelpCommand(m_csi);
  m_cmds.Push(tmp);
  tmp = new cssCreateLightCommand(m_csi);
  m_cmds.Push(tmp);
  tmp = new cssRelightCommand(m_csi);
  m_cmds.Push(tmp);
  tmp = new cssRemoveLightCommand(m_csi);
  m_cmds.Push(tmp);

  return true;
}

size_t cssStdCommandsPlugin::GetCount()
{
  return m_cmds.Length();
}

csPtr<icssCommand> cssStdCommandsPlugin::Get(size_t idx)
{
  return m_cmds.Get(idx);
}


