#include "cssysdef.h"

#include "csgeom/vector3.h"
#include "iengine/material.h"
#include "iengine/sector.h"

#include "fromfact.h"

#include "app/icsstudio.h"
#include "plugins/icssview.h"
#include "action/icssaction.h"
#include "action/create/icssfromfactaction.h"
#include "action/create/icsscreatesectoraction.h"
#include "plugins/icssactionmanager.h"
#include "plugins/icssplugin.h"

#include "fromfactdlg.h"

SCF_IMPLEMENT_IBASE (cssFromFactCreator)
SCF_IMPLEMENTS_INTERFACE(icssObjectCreator)
SCF_IMPLEMENT_IBASE_END

CS_IMPLEMENT_PLUGIN;

SCF_IMPLEMENT_FACTORY(cssFromFactCreator);

cssFromFactCreator::cssFromFactCreator(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
}

cssFromFactCreator::~cssFromFactCreator()
{
  SCF_DESTRUCT_IBASE();
}

void cssFromFactCreator::CreateObject (iCStudioCS *csi,
                         csVector3 &bbox_min, csVector3 &bbox_max,
                         icssView *view)
{
  m_csi = csi;
  cssFromFactDlg dlg(m_csi, view->GetWxWindow(), -1, "fromfactdlg test",
                     wxPoint(100, 100), wxSize(600, 600), wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER);
  if(dlg.ShowModal() != wxID_OK)
  {
    printf("user clicked cancel on fromfactdlg!");
    return;
  }
  
  m_csi = csi;

  csRef<icssAction> a = 
       SCF_CREATE_INSTANCE("cstudio.action.create.fromfact", icssAction);

  a->Initialize(m_csi);

  csRef<icssFromFactAction> fla =
       SCF_QUERY_INTERFACE(a, icssFromFactAction);

  csVector3 pos = (bbox_min + bbox_max) * 0.5f;
  
  csRef<iSector> as = csi->getActiveSector();
  if(!as)
    as = CreateEmptySector();
  
  fla->SetSector(as);
  fla->SetPosition(pos);
  fla->SetFactory(dlg.GetFactoryWrapper());
  fla->SetName(dlg.GetName());

  csRef<icssPlugin> amgrplg = csi->GetPlugin("cstudio.plugin.actionmgr");
  csRef<icssActionManager> amgr = 
        SCF_QUERY_INTERFACE(amgrplg, icssActionManager);

  if(!amgr->Perform(a))
    printf("couldn't perform createfromfactaction\n");

}

iSector * cssFromFactCreator::CreateEmptySector()
{
  csRef<icssAction> a =
       SCF_CREATE_INSTANCE("cstudio.action.create.sector", icssAction);
 
  a->Initialize(m_csi);

  csRef<icssPlugin> plg = m_csi->GetPlugin("cstudio.plugin.actionmgr");
 
  csRef<icssActionManager> actionmgr = 
       SCF_QUERY_INTERFACE(plg, icssActionManager);
  

  if(!a) 
  {
    printf("couldn't get actionmanager\n"); 
    return 0;
  }
  csRef<icssCreateSectorAction> csa =
        SCF_QUERY_INTERFACE(a, icssCreateSectorAction);


  csa->SetName("newsector");


  if(!actionmgr->Perform(a))
  {
    printf("couldn't perform create sector action\n");
     return 0;
  }
  /// the action set the new sector to the current active sector
  return m_csi->getActiveSector();
  
}

