#include "cssysdef.h"

#include "fromfactdlg.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif

#include "wx/button.h"
#include "wx/sizer.h"

#include "iengine/mesh.h"
#include "iengine/engine.h"
#include "iutil/object.h"

#include "iutil/objreg.h"
#include "imesh/object.h"

#include "app/icsstudio.h"

enum {
  OK_BUTTON = wxID_HIGHEST + 1,
  CANCEL_BUTTON,
  FACTORY_LIST,
  PREVIEW_PANE,
};

BEGIN_EVENT_TABLE (cssFromFactDlg, wxDialog)
  EVT_LIST_ITEM_SELECTED(FACTORY_LIST, cssFromFactDlg::OnFactListItemSelected) 
END_EVENT_TABLE()
  
cssFromFactDlg::cssFromFactDlg(iCStudioCS *csi, wxWindow* parent, wxWindowID id, 
                  const wxString& title, const wxPoint& pos,
                  const wxSize& size, long style, 
                  const wxString& name)
            :wxDialog(parent, id, title, pos, size, style, name)
{
  m_csi = csi;
  wxBoxSizer *mainsizer = new wxBoxSizer( wxVERTICAL );
  wxBoxSizer *topsizer = new wxBoxSizer( wxHORIZONTAL );
  wxBoxSizer *rightsizer = new wxBoxSizer( wxVERTICAL );
  wxBoxSizer *bottomsizer = new wxBoxSizer( wxHORIZONTAL );
  wxBoxSizer *txtsizer = new wxBoxSizer( wxHORIZONTAL );

  bottomsizer->Add (new wxButton( this, wxID_CANCEL, "Cancel" ),
                      0, wxALIGN_LEFT, 3 );

  bottomsizer->Add(20, 20, 1, wxGROW | wxALL, 3);

  bottomsizer->Add ( new wxButton( this, wxID_OK, "Ok" ),
                      0, wxALIGN_RIGHT, 3 );

  m_previewpanectrl = new wxPanel(this, PREVIEW_PANE);
  m_namectrl = new wxTextCtrl (this, -1, title);
  m_namectrl->SetValue("new mesh name");
  
  txtsizer->Add(new wxStaticText ( this, -1, "Name"), 1, 
                wxALIGN_RIGHT | wxLEFT | wxRIGHT, 3);
  txtsizer->Add(m_namectrl, 1, 0, 3);

  rightsizer->Add(m_previewpanectrl, 2, wxGROW | wxALL, 3);
  rightsizer->Add(txtsizer, 1, wxGROW | wxALL , 3);

  m_factlistctrl = new wxListView ( this, FACTORY_LIST, wxDefaultPosition,
                                    wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL);
  topsizer->Add ( m_factlistctrl, 1, wxGROW | wxALL, 3);
  topsizer->Add ( rightsizer, 1, wxGROW | wxALL, 3);

  mainsizer->Add(topsizer, 1, wxGROW | wxALL, 3);
  mainsizer->Add(bottomsizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout(TRUE);
  SetSizer(mainsizer);

  mainsizer->Fit(this);
  mainsizer->SetSizeHints(this);
  FillDialog();
}

cssFromFactDlg::~cssFromFactDlg()
{
}

const char *cssFromFactDlg::GetName()
{
  return m_namectrl->GetValue();
}

iMeshFactoryWrapper *cssFromFactDlg::GetFactoryWrapper()
{
  return m_meshfactorywrapper;
}

void cssFromFactDlg::FillDialog() 
{
  csRef<iEngine> e = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);

  m_factlistctrl->ClearAll();
  m_factlistctrl->InsertColumn(0, "factory");
  m_factlistctrl->SetColumnWidth(0, 150);
  m_factlistctrl->InsertColumn(1, "mesh type");
  m_factlistctrl->SetColumnWidth(1, 100);

  csRef<iMeshFactoryList> mfl = e->GetMeshFactories();
  int mflcount = mfl->GetCount();
  for(int i = 0; i < mflcount; i++)
  {
    csRef<iMeshFactoryWrapper> mfw = mfl->Get(i);
    InsertFactoryWrapper(i, mfw);
  }
}

void cssFromFactDlg::InsertFactoryWrapper(int idx, iMeshFactoryWrapper *mfw)
{
  m_factlistctrl->InsertItem(idx, mfw->QueryObject()->GetName());

  const char *nametype = DetermineFactoryType(mfw);
  m_factlistctrl->SetItem(idx, 1, &nametype[strlen("crystalspace.mesh.object.")]);
}

const char * cssFromFactDlg::DetermineFactoryType(iMeshFactoryWrapper *mfw)
{
  csRef<iMeshObjectFactory> mf = mfw->GetMeshObjectFactory();
  csRef<iFactory> factory = 
      SCF_QUERY_INTERFACE(mf->GetMeshObjectType(), iFactory);

  return factory->QueryClassID();
}

void cssFromFactDlg::OnFactListItemSelected(wxListEvent &e)
{
  csRef<iEngine> engine = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);

  csRef<iMeshFactoryList> mfl = engine->GetMeshFactories();
  long idx = e.GetIndex();
  
  m_meshfactorywrapper = mfl->Get(idx);
    
}
