#include "cssysdef.h"

#include "csgeom/vector3.h"
#include "iengine/material.h"
#include "iengine/sector.h"

#include "thingbox.h"

#include "app/icsstudio.h"
#include "plugins/icssview.h"
#include "action/icssaction.h"
#include "action/create/icssthingboxaction.h"
#include "action/create/icsscreatesectoraction.h"
#include "plugins/icssactionmanager.h"
#include "plugins/icssplugin.h"

#include "namedlg.h"

SCF_IMPLEMENT_IBASE (cssThingBoxCreator)
SCF_IMPLEMENTS_INTERFACE(icssObjectCreator)
SCF_IMPLEMENT_IBASE_END

CS_IMPLEMENT_PLUGIN;

SCF_IMPLEMENT_FACTORY(cssThingBoxCreator);

cssThingBoxCreator::cssThingBoxCreator(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
}

cssThingBoxCreator::~cssThingBoxCreator()
{
  SCF_DESTRUCT_IBASE();
}

void cssThingBoxCreator::CreateObject (iCStudioCS *csi,
                         csVector3 &bbox_min, csVector3 &bbox_max,
                         icssView *view)
{
  cssNameDialog dlg(view->GetWxWindow(), -1, "Name test");
  if(dlg.ShowModal() == wxID_OK)
  {
    printf("hooorrawww!");
  }
  
  m_csi = csi;

  csRef<icssAction> a = 
       SCF_CREATE_INSTANCE("cstudio.action.create.thingbox", icssAction);

  a->Initialize(m_csi);

  csRef<icssThingBoxAction> tba =
       SCF_QUERY_INTERFACE(a, icssThingBoxAction);

  tba->SetBBox(bbox_min, bbox_max);

  csRef<iMaterialWrapper> mat = csi->GetActiveMaterial();
  tba->SetMaterial(mat);

  csRef<iSector> as = csi->getActiveSector();
  if(!as)
    as = CreateEmptySector();
  
  tba->SetSector(as);

  csRef<icssPlugin> amgrplg = csi->GetPlugin("cstudio.plugin.actionmgr");
  csRef<icssActionManager> amgr = 
        SCF_QUERY_INTERFACE(amgrplg, icssActionManager);

  if(!amgr->Perform(a))
    printf("couldn't perform createthingboxaction\n");

}

iSector * cssThingBoxCreator::CreateEmptySector()
{
  csRef<icssAction> a =
       SCF_CREATE_INSTANCE("cstudio.action.create.sector", icssAction);
 
  a->Initialize(m_csi);

  csRef<icssPlugin> plg = m_csi->GetPlugin("cstudio.plugin.actionmgr");
 
  csRef<icssActionManager> actionmgr = 
       SCF_QUERY_INTERFACE(plg, icssActionManager);
  

  if(!a) 
  {
    printf("couldn't get actionmanager\n"); 
    return 0;
  }
  csRef<icssCreateSectorAction> csa =
        SCF_QUERY_INTERFACE(a, icssCreateSectorAction);


  csa->SetName("newsector");


  if(!actionmgr->Perform(a))
  {
    printf("couldn't perform create sector action\n");
     return 0;
  }
  /// the action set the new sector to the current active sector
  return m_csi->getActiveSector();
  
}

