#include "cssysdef.h"

#include "iengine/engine.h"
#include "iengine/mesh.h"
#include "iengine/material.h"
#include "iengine/movable.h"
#include "iutil/objreg.h"
#include "imesh/thing.h"
#include "imesh/object.h"
#include "iengine/sector.h"

#include "app/icsstudio.h"

#include "thingboxaction.h"

SCF_IMPLEMENT_IBASE(cssThingBoxAction)
SCF_IMPLEMENTS_INTERFACE(icssAction)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssThingBoxAction)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssThingBoxAction::ecssThingBoxAction)
SCF_IMPLEMENTS_INTERFACE(icssThingBoxAction)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY(cssThingBoxAction);

cssThingBoxAction::cssThingBoxAction(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssThingBoxAction);
}

cssThingBoxAction::~cssThingBoxAction()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssThingBoxAction);
  SCF_DESTRUCT_IBASE();
}

bool cssThingBoxAction::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
  return true;
}

bool cssThingBoxAction::Do()
{
  if(!m_sector)
    return false;
  
  if(!m_matwrap)
    return false;


  csRef<iEngine> e = 
    	CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);

  csRef<iMeshWrapper> mwrap = e->CreateThingMesh( m_sector, "newmesh");
  if(!mwrap)	
    return false;

  csRef<iThingState> ts =
        SCF_QUERY_INTERFACE(mwrap->GetMeshObject(), iThingState);
  if (!ts)
    return false;

  csRef<iThingFactoryState> tfs = SCF_QUERY_INTERFACE (
	mwrap->GetMeshObject()->GetFactory(), iThingFactoryState);
  tfs->AddOutsideBox(m_bb_min, m_bb_max); 
  
  tfs->SetPolygonMaterial(CS_POLYRANGE_ALL, m_matwrap);
  tfs->SetPolygonTextureMapping(CS_POLYRANGE_ALL, 3);
  
  mwrap->GetMovable()->UpdateMove();

  m_meshwrap = mwrap;
 
  e->ForceRelight();

  csRef<icssContentCallback> ccb = SCF_QUERY_INTERFACE(m_csi, icssContentCallback);
  ccb->OnMeshListChanged();
  
  return true; 
}

bool cssThingBoxAction::Undo()
{
  csRef<iEngine> e =
        CS_QUERY_REGISTRY (m_csi->getObjectRegistry(), iEngine);

  e->RemoveObject(m_meshwrap);

  m_meshwrap = 0; // remove object completly

  csRef<icssContentCallback> ccb = SCF_QUERY_INTERFACE(m_csi, icssContentCallback);
  ccb->OnMeshListChanged();
  
  return true;
}

bool cssThingBoxAction::Redo()
{
  return Do();
}
  
void cssThingBoxAction::SetBBox(csVector3 &min, csVector3 &max)
{
  m_bb_min = min;
  m_bb_max = max;
}

void cssThingBoxAction::SetMaterial(iMaterialWrapper *mat)
{
  m_matwrap = mat;
}

void cssThingBoxAction::SetSector(iSector *s)
{
  m_sector = s;
}

