/*
    Copyright (C) 2005 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cssysdef.h"
#include "csgeom/vector3.h"
#include "bsptree.h"



cssBspTree::cssBspTree()
{
  front = back = 0;
}


cssBspTree::~cssBspTree()
{
  Clear();
}


void cssBspTree::Clear()
{
  delete front;
  delete back;
  front = back = 0;
}


void cssBspTree::Build(const cssCsgMesh& mesh)
{
  cssPolyArray polys;
  for(size_t p = 0; p < mesh.polys.Length(); p++)
    polys.Push(mesh.polys[p].poly);
  
  Build(polys);
}


void cssBspTree::Build(const cssPolyArray& polys)
{
  Clear();

  pl = polys[0].ComputePlane();

  cssPolyArray polys_front, polys_back;

  for(size_t i = 1; i < polys.Length(); i++)
  {
     csPoly3D poly = polys[i];
     int pos = poly.Classify(pl);

     if(pos == CS_POL_SAME_PLANE)
     { 
       if(poly.ComputeNormal() != pl.Normal())
         //polys_back.Push(poly);
         polys_front.Push(poly);
       continue;
     }
     if(pos == CS_POL_FRONT)
     {
       polys_front.Push(poly);
       continue;
     }
     if(pos == CS_POL_BACK)
     {
       polys_back.Push(poly);
       continue;
     }

     csPoly3D front, back;
     polys[i].SplitWithPlane(front, back, pl);
     polys_front.Push(front);
     polys_back.Push(back);
  }

  if(polys_front.Length())
  {
    front = new cssBspTree();
    front->Build(polys_front);
  }

  if(polys_back.Length())
  {
    back = new cssBspTree();
    back->Build(polys_back);
  }

}


float cssBspTree::Classify(const csVector3& pt)
{
  float ret = pl.Classify(pt);

  if(ret <= 0)
  {
    if(front)
      return front->Classify(pt);
    return ret;
  }

  if(back)
    return back->Classify(pt);
  return ret;
}


bool cssBspTree::PointInside(const csVector3& pt)
{
  if(pl.Classify(pt) < 0)
  {
    if(front)
      return front->PointInside(pt);
    return false;
  }

  if(back)
    return back->PointInside(pt);
  return true;
}
