/*
    Copyright (C) 2005 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cssysdef.h"
#include "csutil/objreg.h"
#include "csutil/csstring.h"

#include "csgplugin.h"
#include "csgaction.h"
#include "csgcmd.h"

#include "plugins/icssactionmanager.h"
#include "plugins/icssselection.h"


#include "iengine/material.h"
#include "iengine/sector.h"
#include "iengine/mesh.h"
#include "iengine/engine.h"
#include "imesh/object.h"
#include "imesh/thing.h"


SCF_IMPLEMENT_IBASE (cssCsgCommand)
SCF_IMPLEMENTS_INTERFACE (icssCommand)
SCF_IMPLEMENT_IBASE_END



cssCsgCommand::cssCsgCommand(iCStudioCS *csi)
{
  SCF_CONSTRUCT_IBASE(0);
  this->csi = csi;
  csRef<icssPlugin> am = csi->GetPlugin("cstudio.plugin.actionmgr");

  if(am)
    actionmgr = SCF_QUERY_INTERFACE(am, icssActionManager);
}


cssCsgCommand::~cssCsgCommand()
{
  SCF_DESTRUCT_IBASE();
}


bool cssCsgCommand::OnCommand(csStringArray* args)
{
  if (args->Length() != 2)
  {
    PrintUsage();
    return false;
  }

  csString arg = args->Get(1);
  int operation = CSS_CSG_ADD;

  if(arg == "add")
    operation = CSS_CSG_ADD;
  else if(arg == "subtract")
    operation = CSS_CSG_SUBTRACT;
  else if(arg == "intersect")
    operation = CSS_CSG_INTERSECT;
  //else if(arg == "test")
    //return CreateTestMeshes();
  else
  {
    PrintUsage();
    return false;
  }
  
  csRef<icssPlugin> sel  (csi->GetPlugin("cstudio.select.defselect"));
  if(!sel)
    return false;

  csRef<icssMeshSelection> meshsel (SCF_QUERY_INTERFACE(sel, icssMeshSelection));
  if(!meshsel)
    return false;

  if(meshsel->GetCount() != 2)
  {
     csi->Print("error: 2 meshes must be selected.\n");
     return false;
  }

  csRef<iEngine> engine (CS_QUERY_REGISTRY(csi->getObjectRegistry(), iEngine));
  
  csRef<cssCsgAction> action = csPtr<cssCsgAction> (new cssCsgAction(this));
  if(!action->Initialize(csi))
    return false;

  action->SetOperation(operation);
  action->SetSourceMeshes(meshsel->Get(0), meshsel->Get(1));
  //action->SetSourceMeshes(mesh1, mesh2);
  
  if(!actionmgr->Perform(action))
  {
    return false;
  }

  return true;
}





#if 0
bool cssCsgCommand::CreateTestMeshes()
{
  printf("creating test meshes...\n");

  csRef<iEngine> engine
    (CS_QUERY_REGISTRY(csi->getObjectRegistry(), iEngine));
    
  engine->SetClearZBuf(true);
  engine->SetClearScreen(true);
  engine->SetLightingCacheMode(0);
  
  iSector* sector = csi->getActiveSector();

  mesh1 = CreateMesh(csBox3(csVector3(-1,-1,-1+0), csVector3(1,1,1+0)), true, "mesh1", sector, engine);
  mesh2 = CreateMesh(csBox3(csVector3(-0.5,-0.5,-0.5+0), csVector3(1.5,1.5,1.5+0)), false, "mesh2", sector, engine);

  printf("preparing engine...\n");
  engine->Prepare();

  return true;
}


csPtr<iMeshWrapper> cssCsgCommand::CreateMesh(csBox3 box, bool outside, const char* name, iSector* sector, iEngine* engine)
{

  printf("creating mesh...\n");
  
  iMaterialWrapper* m = engine->GetMaterialList ()->Get(0);  
  
  csRef<iMeshWrapper> mesh;
  
  mesh = engine->CreateThingMesh (sector, name);
  printf("created thing mesh.\n");


  csRef<iThingFactoryState> thing_fact
  	  (SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iThingFactoryState));

  if(outside)
    thing_fact->AddOutsideBox(box.Min(), box.Max());
  else
    thing_fact->AddInsideBox(box.Min(), box.Max());

  printf("setting material...\n");    
  thing_fact->SetPolygonTextureMapping (CS_POLYRANGE_ALL,  3);
  thing_fact->SetPolygonMaterial (CS_POLYRANGE_ALL,  m);
  
  thing_fact->CompressVertices();
  
  mesh->GetMovable()->UpdateMove();
  
  return csPtr<iMeshWrapper> (mesh);
}
#endif
