/*
    Copyright (C) 2004 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"

#include "csgplugin.h"
#include "csgmesh.h"

#include "app/icsstudio.h"

#include "imesh/object.h"
#include "imesh/thing.h"
#include "iengine/engine.h"
#include "iengine/sector.h"
#include "iengine/mesh.h"

#include "csgeom/poly3d.h"
//#include "csutil/csobject.h"



cssCsgMesh::cssCsgMesh(iEngine* engine, iSector* sector, const char* name, int type)
{
  this->sector = sector;
  this->engine = engine;
  this->name = name;
  this->type = type;
}


cssCsgMesh::cssCsgMesh()
{
  this->type = CSS_MESHTYPE_THING;
}


cssCsgMesh::~cssCsgMesh()
{
}


bool cssCsgMesh::ImportMesh(iMeshWrapper*  mesh)
{
  csRef<iThingFactoryState> thing_fact
  	(SCF_QUERY_INTERFACE (mesh->GetMeshObject (), iThingFactoryState));
  
  if(thing_fact)
  {
    ImportThingFactory(thing_fact);
    return true;
  }
  
  return false;
}


bool cssCsgMesh::ImportThingFactory(iThingFactoryState* thing_fact)
{
  printf("csgmesh: importing thing factory...");
  for(int p = 0; p < thing_fact->GetPolygonCount(); p++)
  {
    csPoly3D poly;
    for(int v = 0; v < thing_fact->GetPolygonVertexCount(p); v++)
       poly.AddVertex(thing_fact->GetPolygonVertex(p,v));
    polys.Push(cssCsgPolygon(poly, thing_fact->GetPolygonMaterial(p)));
  }

  printf("done.\n");
  return true;
}


csPtr<iMeshWrapper> cssCsgMesh::Build()
{
  if(type == CSS_MESHTYPE_THING)
    return BuildThing();
  if(type == CSS_MESHTYPE_GENERAL)
    return BuildGenMesh();
  else
    return csPtr<iMeshWrapper> (0);
}


csPtr<iMeshWrapper> cssCsgMesh::BuildThing()
{
  csRef<iMeshWrapper> meshobj;
  
  meshobj = engine->CreateThingMesh (sector, name);

  if(!meshobj)
    return csPtr<iMeshWrapper> (0);

  csRef<iThingFactoryState> thing_fact
  	  (SCF_QUERY_INTERFACE (meshobj->GetMeshObject (), iThingFactoryState));
   
  for(size_t p = 0; p < polys.Length(); p++)
  {
        csPoly3D poly = polys[p].poly;
        int vertexcount = poly.GetVertexCount();
        csVector3* verts = new csVector3[vertexcount];

        for(int v = 0; v < vertexcount; v++)
          verts[v] = *poly.GetVertex(v); 

        int idx=-1;

        if(vertexcount == 4)
          idx = thing_fact->AddQuad(verts[0],verts[1],verts[2],verts[3]);

        else if(vertexcount == 3)
          idx = thing_fact->AddTriangle(verts[0],verts[1],verts[2]);

        else
          idx = thing_fact->AddPolygon(verts, vertexcount);

        //CS_ASSERT(idx >= 0);  

        thing_fact->SetPolygonTextureMapping (CS_POLYRANGE_SINGLE(idx),  *poly.GetVertex(0),  *poly.GetVertex(1),  3);
        thing_fact->SetPolygonMaterial (CS_POLYRANGE_SINGLE(idx),  polys[p].material);

        delete[] verts;
  }

  thing_fact->CompressVertices();

  return csPtr<iMeshWrapper> (meshobj);
}


csPtr<iMeshWrapper> cssCsgMesh::BuildGenMesh()
{
  csRef<iMeshWrapper> meshobj;

  return csPtr<iMeshWrapper> (meshobj);
}


/*void cssCsgMesh::CopyPolygons(cssPolyArray& output) const
{

}*/
