/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __DEFAULTEM_H_INCLUDED__
#define __DEFAULTEM_H_INCLUDED__

#include "plugins/icsseditmode.h"
#include "plugins/icssplugin.h"
#include "plugins/icssactionmanager.h"
#include "plugins/icssselector.h"
#include "action/icssaction.h"
#include "csutil/csstring.h"

struct iSector;

class cssDefaultEditModePlugin : public icssPlugin
{
  iCStudioCS* m_csi;
  csRef<icssActionManager> m_actionmgr;
  csRef<icssSelector> m_meshselector;
  void GetActionManager();
  void GetMeshSelector();
public:
  cssDefaultEditModePlugin(iBase *parent);
  virtual ~cssDefaultEditModePlugin();

  //// initialisation 
  virtual bool Initialize(iCStudioCS *csi);
  virtual const char *GetName()
  { return "cstudio.em.defaultem"; };

  /// events split up
  bool OnLeftDown(wxMouseEvent *e, icssView *view);
  void ClearSelection();

  //// icssEditMode implementation
  const char *GetDescription();
  void OnActivate(icssEditMode *lastactive);
  void OnDeactivate(icssEditMode *nextactive);
  bool OnEvent(wxEvent &event, icssView *view);
  void OnDraw(icssDrawer *d, icssView *v);
  
  class cssEditMode : public icssEditMode
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssDefaultEditModePlugin);
    virtual icssPlugin *GetPlugin()
    { return scfParent; };
    virtual const char *GetDescription()
    { return scfParent->GetDescription(); };
    virtual void OnActivate(icssEditMode *lastactive)
    { scfParent->OnActivate(lastactive); };
    virtual void OnDeactivate(icssEditMode *nextactive)
    { scfParent->OnDeactivate(nextactive); };
    virtual bool OnEvent(wxEvent &event, icssView *view)
    { return scfParent->OnEvent(event, view); };
    virtual void OnDraw(icssDrawer *d, icssView *v)
    { scfParent->OnDraw(d, v); };
  } scficssEditMode;
  friend class cssEditMode;

  /// Sector Switching
  void SwitchLocalSector(wxKeyEvent &event);
  void SetActiveSector(csString newsector, bool inall);

  SCF_DECLARE_IBASE;
};




#endif
