/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"
#include "csutil/scf.h"
#include "csutil/csstring.h"
#include "ivaria/reporter.h"
#include "ivaria/stdrep.h"
#include "csutil/csstring.h"
#include "iutil/stringarray.h"

#include "app/icsstudio.h"
#include "plugins/icssplugin.h"

#include "csseditmodemanager.h"

CS_IMPLEMENT_PLUGIN

SCF_IMPLEMENT_IBASE(cssEditModeManagerPlugin)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssEditModeManager)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssEditModeManagerPlugin::cssEditModeManager)
SCF_IMPLEMENTS_INTERFACE(icssEditModeManager)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY(cssEditModeManagerPlugin);

cssEditModeManagerPlugin::cssEditModeManagerPlugin(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssEditModeManager);
}

cssEditModeManagerPlugin::~cssEditModeManagerPlugin()
{
 
  m_activeEM = 0;
}

bool cssEditModeManagerPlugin::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
  LoadEditModes();
  return true;
}

const char *cssEditModeManagerPlugin::GetName()
{
  return "cstudio.plugin.emmanager";
}

void cssEditModeManagerPlugin::SetActiveEditMode (icssEditMode *newactive)
{
  size_t count = m_callbacks.Length();

  for(size_t i = 0; i < count; i++)
    m_callbacks.Get(i)->OnEditModeChange(newactive);

  csRef<icssEditMode> oldem = m_activeEM;

  if(m_activeEM)
    m_activeEM->OnDeactivate(newactive);

  m_activeEM = newactive;
  m_activeEM->OnActivate(oldem);
}

icssEditMode* cssEditModeManagerPlugin::GetActiveEditMode ()
{
  return m_activeEM;
}

void cssEditModeManagerPlugin::AddCallBack (icssEditModeManagerCB *toadd)
{
  m_callbacks.Push(toadd);
}

void cssEditModeManagerPlugin::RemoveCallBack (icssEditModeManagerCB *torem)
{
  m_callbacks.Delete(torem);
}

csPtr<icssEditMode> cssEditModeManagerPlugin::GetEM(const char* name)
{
	csRef<icssPlugin> ep = m_emplugs.Get(name, 0);
	if(ep)
	{	//// edit mode plug is already loaded.. return icssEditMode
		csRef<icssEditMode> em = SCF_QUERY_INTERFACE(ep, icssEditMode);
		return csPtr<icssEditMode>(em);
	}

  csString fullpath;
  fullpath = "cstudio.em.";
  fullpath.Append(name);

  csRef<icssPlugin> newemplugin = SCF_CREATE_INSTANCE(fullpath.GetData(), icssPlugin);
  if(newemplugin == 0)
  {
    csString errormsg = "Couldn't load ";
    errormsg.Append(fullpath);
    errormsg.Append(" edit mode plugin!");
    csReport(m_csi->getObjectRegistry(), CS_REPORTER_SEVERITY_ERROR,
      "crystalspace.application.cstudio",
      errormsg);
    return 0;
  }

	if(!newemplugin->Initialize(m_csi))
	{
		csString errormsg = "Couldn't initialize ";
    errormsg.Append(fullpath);
    errormsg.Append(" edit mode plugin!");
    csReport(m_csi->getObjectRegistry(), CS_REPORTER_SEVERITY_ERROR,
      "crystalspace.application.cstudio",
      errormsg);
	}
	
	csRef<icssEditMode> newem = SCF_QUERY_INTERFACE(newemplugin, icssEditMode);
	if(newem == 0)
	{
		csString errormsg = "Edit Mode ";
    errormsg.Append(fullpath);
		errormsg.Append(" doesn't implement icssEditMode!");
    csReport(m_csi->getObjectRegistry(), CS_REPORTER_SEVERITY_ERROR,
      "crystalspace.application.cstudio",
      errormsg);	
	}
	
	m_emplugs.Put(name, newemplugin);

  return csPtr<icssEditMode>(newem);
}

void cssEditModeManagerPlugin::LoadEditModes()
{
	csRef<iStringArray> emstr = iSCF::SCF->QueryClassList("cstudio.em.");
	size_t ncount = emstr->Length();
	for(size_t i = 0; i < ncount; i++)
	{
		const char * str = emstr->Get(i);
		csRef<icssEditMode> em = GetEM(&str[strlen("cstudio.em.")]);
	}
}
