/*
   Copyright (C) 2003 by Philipp Aumayr

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include "cssysdef.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
  #include "wx/wx.h"
#endif

#include "igeom/polymesh.h"
#include "iengine/engine.h"
#include "iengine/mesh.h"
#include "iengine/movable.h"
#include "imesh/object.h"
#include "iengine/sector.h"
#include "igeom/objmodel.h"
#include "csutil/csstring.h"
#include "iutil/objreg.h"
#include "iutil/stringarray.h"

#include "app/icsstudio.h"
#include "plugins/icssview.h"
#include "plugins/icssdrawer.h"
#include "plugins/icssobjectcreator.h"

#include "objcreateem.h"

CS_IMPLEMENT_PLUGIN

// we need a good guess here ;)
enum {
  MORPH = wxID_HIGHEST + 1000,
  MORPH_START,
  MORPH_END = MORPH_START + 100, // max 100 plugins, increase as necessary 
};

BEGIN_EVENT_TABLE(cssObjectCreateEMPlugin::EvHandler, wxEvtHandler)
  EVT_MENU_RANGE (MORPH_START, MORPH_END, cssObjectCreateEMPlugin::EvHandler::OnCreateRange)
END_EVENT_TABLE()

SCF_IMPLEMENT_IBASE(cssObjectCreateEMPlugin)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssEditMode)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssContextMenu)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssObjectCreateEMPlugin::cssEditMode)
SCF_IMPLEMENTS_INTERFACE(icssEditMode)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssObjectCreateEMPlugin::cssContextMenu)
SCF_IMPLEMENTS_INTERFACE(icssContextMenu)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY (cssObjectCreateEMPlugin);

cssObjectCreateEMPlugin::cssObjectCreateEMPlugin(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssEditMode);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssContextMenu);
  last_view = 0;
  evhdler.p = this;
}

cssObjectCreateEMPlugin::~cssObjectCreateEMPlugin()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssContextMenu);
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssEditMode);
  evhdler.p = 0;
  SCF_DESTRUCT_IBASE();
}


//// initialisation 
bool cssObjectCreateEMPlugin::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
  return true;
}

void cssObjectCreateEMPlugin::GetActionManager()
{
  csRef<icssPlugin> amplug = m_csi->GetPlugin("cstudio.plugin.actionmgr");
  m_actionmgr = SCF_QUERY_INTERFACE(amplug, icssActionManager);
}

/*
 * icssEditMode implementation
 */
const char *cssObjectCreateEMPlugin::GetDescription()
{
  return "Object Create";
}


void cssObjectCreateEMPlugin::OnActivate(icssEditMode *lastactive)
{
  GetActionManager();
  bbox_state = BBOX_FIRST_INSERT;
}

void cssObjectCreateEMPlugin::OnDeactivate(icssEditMode *nextactive)
{
  last_view = 0;
}

/// Event catcher, dynamic_cast to check type
bool cssObjectCreateEMPlugin::OnEvent(wxEvent &event, icssView *view)
{
  last_view = view;
/*
# wxEVT_LEFT_DOWN
# wxEVT_LEFT_UP
# wxEVT_LEFT_DCLICK
# wxEVT_MIDDLE_DOWN
# wxEVT_MIDDLE_UP
# wxEVT_MIDDLE_DCLICK
# wxEVT_RIGHT_DOWN
# wxEVT_RIGHT_UP
# wxEVT_RIGHT_DCLICK
# wxEVT_MOTION
# wxEVT_MOUSEWHEEL 
 */
  if(event.GetEventType() == wxEVT_LEFT_DOWN)
  {
    wxMouseEvent *me = wxDynamicCast(&event, wxMouseEvent);
    if(me)
      return OnLeftDown(me, view);
  }
  else if(event.GetEventType() == wxEVT_KEY_DOWN)
  {
    wxKeyEvent *kev = wxDynamicCast(&event, wxKeyEvent);

    if(kev)
      return OnKeyDown(kev, view);
  }
  else if(event.GetEventType() == wxEVT_MOTION)
  {
    wxMouseEvent *me = wxDynamicCast(&event, wxMouseEvent);
    if(me)
      return OnMouseMove(me, view);
  }

  return evhdler.ProcessEvent(event);
  // if we don't do anything we have to skip the event
  event.Skip();
  return false;
}

void cssObjectCreateEMPlugin::OnCreateRange(wxCommandEvent &e)
{
  csRef<iStringArray> cp = iSCF::SCF->QueryClassList("cstudio.creators.");
  cp->Sort();
  
  const char *s = cp->Get(e.GetId() - MORPH_START);

  csRef<icssObjectCreator> objcreator = SCF_CREATE_INSTANCE(s, icssObjectCreator);
  if(objcreator)
  {
    objcreator->CreateObject (m_csi, bbox_min, bbox_max, last_view);
    return;
  }
}

bool cssObjectCreateEMPlugin::OnLeftDown(wxMouseEvent *e, icssView *view)
{
  // get map position
  csVector3 mappos;
  view->GetMapPosfromMouseCoords(e->m_x, e->m_y, mappos);
  if(bbox_state == BBOX_FIRST_INSERT)
  {
    bbox_min = bbox_max = mappos;
    bbox_min += csVector3(-1.0f, -1.0f, -1.0f);
    bbox_max += csVector3(1.0f, 1.0f, 1.0f);  

    CategorizePoint(mappos);
    bbox_state = BBOX_MODIFY;
  }
  else
  if(bbox_state == BBOX_MODIFY)
  {
    CategorizePoint(mappos);
    
  }
  return true;
}


void cssObjectCreateEMPlugin::OnRightDown(wxMouseEvent *e, icssView *v)
{
  wxPoint mouse(e->m_x, e->m_y);
  wxWindow * wnd = wxDynamicCast(e->GetEventObject(), wxWindow);
  if(wnd)
    mouse = wnd->ClientToScreen(mouse);

}

bool cssObjectCreateEMPlugin::OnKeyDown(wxKeyEvent *e, icssView *v)
{
  return true;
}

bool cssObjectCreateEMPlugin::OnMouseMove(wxMouseEvent *e, icssView *v)
{
  csVector3 mappos;
  v->GetMapPosfromMouseCoords(e->m_x, e->m_y, mappos);
  
  if(bbox_state == BBOX_MODIFY && e->m_leftDown)
  { 
    if(mappos.x != 0.0f)
    {
      if(cursor_cat.x == BBOX_GREATER)
      {
        if(bbox_max.x + mappos.x - old_mpos.x > bbox_min.x)
          bbox_max.x += mappos.x - old_mpos.x;
        
      }
      else 
      if(cursor_cat.x == BBOX_SMALLER)
      {
        if(bbox_min.x + mappos.x - old_mpos.x < bbox_max.x)
          bbox_min.x += mappos.x - old_mpos.x;
      }
     
    }
    
    if(mappos.y != 0.0f)
    {
      if(cursor_cat.y == BBOX_GREATER)
      {
        if(bbox_max.y + mappos.y - old_mpos.y > bbox_min.y)
          bbox_max.y += mappos.y - old_mpos.y;
      }
      else
      if(cursor_cat.y == BBOX_SMALLER)
      {
        if(bbox_min.y + mappos.y - old_mpos.y < bbox_max.y)
          bbox_min.y += mappos.y - old_mpos.y;
      }
    }
   
    if(mappos.z != 0.0f)
    {
      if(cursor_cat.z == BBOX_GREATER)
      {
        if(bbox_max.z + mappos.z - old_mpos.z > bbox_min.z)
          bbox_max.z += mappos.z - old_mpos.z;
      }
      else
      if(cursor_cat.z == BBOX_SMALLER)
      {
        if(bbox_min.z + mappos.z - old_mpos.z < bbox_max.z)
          bbox_min.z += mappos.z - old_mpos.z;
      }
    }
  }

  old_mpos = mappos; 
  return true;
}

void cssObjectCreateEMPlugin::OnDraw (icssDrawer *d, icssView *v)
{
  csRef<iSector> as = m_csi->getActiveSector();
  csColor4 c(0.7f, 0.7f, 0.7f);
  if(as)
  {
    d->SetColor(c);

    csRef<iMeshList> ml = as->GetMeshes();
    int mesh_count = ml->GetCount();
    for (int i = 0; i < mesh_count; i++)
    {
      d->DrawMesh(ml->Get(i));
    }
  }

  c = csColor4(0.0, 1.0, 0.0);
  
  d->SetColor(c);
  d->SetLineStipple(1, 0xF0F0);
  d->DrawAABB(bbox_min, bbox_max);

}

void cssObjectCreateEMPlugin::CategorizePoint (csVector3 &p)
{
  if(p.x == 0.0f)
  {
    cursor_cat.x = BBOX_MIDDLE;
   
    if(p.y > bbox_max.y)
      cursor_cat.y = BBOX_GREATER;
    else
    if(p.y > bbox_min.y)
      cursor_cat.y = BBOX_MIDDLE;
    else
      cursor_cat.y = BBOX_SMALLER;

    if(p.z > bbox_max.z)
      cursor_cat.z = BBOX_GREATER;
    else
    if(p.z > bbox_min.z)
      cursor_cat.z = BBOX_MIDDLE;
    else
      cursor_cat.z = BBOX_SMALLER;
  }
  else
  if(p.y == 0.0f)
  {
    cursor_cat.y = BBOX_MIDDLE;
   
    if(p.x > bbox_max.x)
      cursor_cat.x = BBOX_GREATER;
    else
    if(p.x > bbox_min.x)
      cursor_cat.x = BBOX_MIDDLE;
    else
      cursor_cat.x = BBOX_SMALLER;
   
    if(p.z > bbox_max.z)
      cursor_cat.z = BBOX_GREATER;
    else
    if(p.z > bbox_min.z)
      cursor_cat.z = BBOX_MIDDLE;
    else
      cursor_cat.z = BBOX_SMALLER;
  }
  else
  if(p.z == 0.0f)
  {
    cursor_cat.z = BBOX_MIDDLE;
   
    if(p.x > bbox_max.x)
      cursor_cat.x = BBOX_GREATER;
    else
    if(p.x > bbox_min.x)
      cursor_cat.x = BBOX_MIDDLE;
    else
      cursor_cat.x = BBOX_SMALLER;
   
    if(p.y > bbox_max.y)
      cursor_cat.y = BBOX_GREATER;
    else
    if(p.y > bbox_min.y)
      cursor_cat.y = BBOX_MIDDLE;
    else
      cursor_cat.y = BBOX_SMALLER;

  }
}


void cssObjectCreateEMPlugin::AppendContextMenu(wxMenu *parent)
{
  wxMenu *sub = new wxMenu();

  AppendCreatePlugins(sub);

  parent->Append(MORPH, wxT("Create"), sub, wxT("Create a new mesh object"));
}

void cssObjectCreateEMPlugin::AppendCreatePlugins(wxMenu *sub)
{
  csRef<iStringArray> cp = iSCF::SCF->QueryClassList("cstudio.creators.");
  cp->Sort();

  size_t count = cp->Length();
  for(size_t i = 0; i < count; i++)
  {
    sub->Append(MORPH_START + i, cp->Get(i));
  }
}




