/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __CSS_EDITMODEMANAGER_H_INCLUDED__
#define __CSS_EDITMODEMANAGER_H_INCLUDED__

#include "csutil/refarr.h"
#include "csutil/ref.h"
#include "csutil/hash.h"
#include "csutil/hashhandlers.h"

#include "plugins/icssactionmanager.h"
#include "plugins/icssplugin.h"
#include "plugins/icsseditmode.h"
#include "plugins/icsscontextmenu.h"
#include "app/icsstudio.h"
#include "wx/event.h"

#define BBOX_SMALLER       1
#define BBOX_MIDDLE        2
#define BBOX_GREATER        3

enum 
{
  BBOX_FIRST_INSERT = 1,
  BBOX_MODIFY,

};

struct s_cursor_cat
{
  char x, y, z;
};

class cssObjectCreateEMPlugin : public icssPlugin
{
  iCStudioCS* m_csi;
  csRef<icssActionManager> m_actionmgr;
  void GetActionManager();

  csRef<icssView> last_view;

  /// for object creation when dragging a bounding box
  int bbox_state;
  // stores the cursor kategory for each axis
  s_cursor_cat  cursor_cat;

  csVector3 bbox_min, bbox_max;
  csVector3 old_mpos;
public:
  cssObjectCreateEMPlugin(iBase *parent);
  virtual ~cssObjectCreateEMPlugin();

  //// initialisation 
  virtual bool Initialize(iCStudioCS *csi);
  virtual const char *GetName()
  { return "cstudio.em.objcreateem"; };

  /// event splitting
  bool OnLeftDown(wxMouseEvent *e, icssView *view);
  bool OnKeyDown(wxKeyEvent *e, icssView *v);
  bool OnMouseMove(wxMouseEvent *e, icssView *v);
  void OnRightDown(wxMouseEvent *e, icssView *v);
  bool OnCommandEvent(wxCommandEvent *e, icssView *view);
  
  
  // classifies a point compared to the bbox
  void CategorizePoint (csVector3 &p);
 
  //// icssEditMode implementation
  const char *GetDescription();
  void OnActivate(icssEditMode *lastactive);
  void OnDeactivate(icssEditMode *nextactive);
  bool OnEvent(wxEvent &event, icssView *view);
  void OnDraw (icssDrawer *d, icssView *v);
  void OnCreateRange( wxCommandEvent & event);

  class EvHandler : public wxEvtHandler
  {
  public:
    csRef<cssObjectCreateEMPlugin> p;
    void OnCreateRange( wxCommandEvent & event)
    { p->OnCreateRange(event); };
    DECLARE_EVENT_TABLE();
  } evhdler;

  class cssEditMode : public icssEditMode
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssObjectCreateEMPlugin);
    virtual icssPlugin *GetPlugin()
    { return scfParent; };
    virtual const char *GetDescription()
    { return scfParent->GetDescription(); };
    virtual void OnActivate(icssEditMode *lastactive)
    { scfParent->OnActivate(lastactive); };
    virtual void OnDeactivate(icssEditMode *nextactive)
    { scfParent->OnDeactivate(nextactive); };
    virtual bool OnEvent(wxEvent &event, icssView *view)
    { return scfParent->OnEvent(event, view); };
    virtual void OnDraw (icssDrawer *d, icssView *v)
    { scfParent->OnDraw(d, v); };

  } scficssEditMode;
  friend class cssEditMode;

  void AppendContextMenu(wxMenu *parent);
  void AppendCreatePlugins(wxMenu *sub);

  class cssContextMenu : public icssContextMenu
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssObjectCreateEMPlugin);
    virtual void AppendContextMenu(wxMenu *parent)
    { scfParent->AppendContextMenu(parent); };
  } scficssContextMenu;
  friend class cssContextMenu;

  SCF_DECLARE_IBASE;

};

#endif

