#include "cssysdef.h"

#include "meshobjmove.h"

#include "iengine/mesh.h"
#include "iengine/movable.h"
#include "imesh/object.h"

#include "app/icsstudio.h"
#include "plugins/icssselection.h"
#include "plugins/icssthingselection.h"

SCF_IMPLEMENT_IBASE(cssSelMoveAction)
SCF_IMPLEMENTS_INTERFACE(icssAction)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssMoveAction)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssSelMoveAction::cssMoveAction)
SCF_IMPLEMENTS_INTERFACE(icssMoveAction)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY(cssSelMoveAction)

cssSelMoveAction::cssSelMoveAction(iBase* parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssMoveAction);
  m_move.Set(0.0f, 0.0f, 0.0f);
}

cssSelMoveAction::~cssSelMoveAction()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssMoveAction);
  SCF_DESTRUCT_IBASE();
}

//// icssAction interface
bool cssSelMoveAction::Initialize(iCStudioCS *csi)
{
  m_selplug = csi->GetPlugin("cstudio.select.defselect");
  return true;
}

bool cssSelMoveAction::Do()
{
  MoveAll(m_lastmove);
  m_move = m_lastmove;
  m_lastmove.Set(0.0f);
  
  UpdateMoveAll();
  return true;
}

bool cssSelMoveAction::Undo()
{
  MoveAll(-m_move);
  UpdateMoveAll();
  return true;
}

bool cssSelMoveAction::Redo()
{
  m_lastmove = m_move;
  Do();
  return true;
}

//// icssMoveAction methods
void cssSelMoveAction::AddMove(csVector3 dir)
{
  m_move += m_lastmove;
  m_lastmove = dir;
}


void cssSelMoveAction::MoveAll(csVector3 dir)
{
  /// move selected meshes
  csRef<icssMeshSelection> meshsel = SCF_QUERY_INTERFACE(m_selplug, icssMeshSelection);
  if(meshsel)
  {
    size_t meshcount = meshsel->GetCount();
    for(size_t i = 0; i < meshcount; i++)
    {
      csRef<iMeshWrapper> mwrap = meshsel->Get(i);
      if(mwrap)
      {
        mwrap->GetMovable()->MovePosition(dir);
      }
    }
  }
	/// move selected thingpartsel
	csRef<icssThingSelection> thingsel = SCF_QUERY_INTERFACE(m_selplug, icssThingSelection);
	if(thingsel)
	{
		size_t thingcount = thingsel->GetCount();
		for(size_t i = 0; i < thingcount; i++)
		{
			csRef<icssThingPart> tps = thingsel->Get(i);
			csRef<iMeshWrapper> cm = tps->GetMesh();
			csRef<iThingState> ts = SCF_QUERY_INTERFACE(cm->GetMeshObject(), iThingState);
			csRef<iThingFactoryState> tfs = SCF_QUERY_INTERFACE (
				cm->GetMeshObject()->GetFactory(), iThingFactoryState);
			/// move vertices
			int vcount = tps->GetVertexCount();
			csReversibleTransform fulltrans = cm->GetMovable()->GetFullTransform();
			for(int j = 0; j < vcount; j++)
			{
				// get the vertex in object space
				csVector3 v = tfs->GetVertex(j);
				// transform vertex to world space
				v = fulltrans.This2Other(v);
				// move the vertex in world space
				v += dir;
				// transform it back to object space
				v = fulltrans.Other2This(v);
				// set the vertex in ojbect itself
				tfs->SetVertex(j, v);
			}

			/// move polygons
			int pcount = tps->GetPolygonCount();
			for(int j = 0; j < pcount; j++)
			{
				int poly = tps->GetPolygonIndex(j);
				MovePolygon(poly, cm, dir);
			}
		}
	}
}

void cssSelMoveAction::MovePolygon(int polyidx, iMeshWrapper *cm, const csVector3 &dir)
{
	csRef<iThingState> ts = SCF_QUERY_INTERFACE(cm->GetMeshObject(), iThingState);
	csRef<iThingFactoryState> tfs = (ts != 0) ? SCF_QUERY_INTERFACE (
		cm->GetMeshObject()->GetFactory(), iThingFactoryState) : 0;  

	if(tfs)
	{
		csReversibleTransform fulltrans = cm->GetMovable()->GetFullTransform();
		int pvcount = tfs->GetPolygonVertexCount(polyidx);
		// index table for the polygon (contains indices to vertex table of object)
		int *idxtbl = tfs->GetPolygonVertexIndices(polyidx);
		if(!idxtbl)
			return; 

		for(int i = 0; i < pvcount; i++)
		{
			// get the index to vertex table
			int realidx = idxtbl[i];
			// get vertex in object space
			csVector3 v = tfs->GetVertex(realidx);
			// transform it to world space
			v = fulltrans.This2Other(v);
			// move it in worldspace
			v += dir;
			// transform it back to objectspace
			v = fulltrans.Other2This(v);
			// set the vertex
			tfs->SetVertex(realidx, v);
		}
	}
  ts->Unprepare();
  ts->Prepare();
}

void cssSelMoveAction::UpdateMoveAll()
{
	// meshes
  csRef<icssMeshSelection> meshsel = SCF_QUERY_INTERFACE(m_selplug, icssMeshSelection);
  if(meshsel)
  {
    size_t meshcount = meshsel->GetCount();
    for(size_t i = 0; i < meshcount; i++)
    {
      csRef<iMeshWrapper> mwrap = meshsel->Get(i);
      if(mwrap)
      {
        mwrap->GetMovable()->UpdateMove();
      }
    }
  }
	// thingpartsel
	csRef<icssThingSelection> thingsel = SCF_QUERY_INTERFACE(m_selplug, icssThingSelection);
	if(thingsel)
	{
		size_t thingcount = thingsel->GetCount();
		for(size_t i = 0; i < thingcount; i++)
		{
			csRef<icssThingPart> tps = thingsel->Get(i);
			csRef<iMeshWrapper> mwrap = tps->GetMesh();
			if(mwrap)
				mwrap->GetMovable()->UpdateMove();
		}
	}


}




