/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif


#include "objplaceem.h"

#include "igeom/polymesh.h"
#include "iengine/engine.h"
#include "iengine/mesh.h"
#include "iengine/movable.h"
#include "imesh/object.h"
#include "igeom/objmodel.h"
#include "iengine/sector.h"
#include "iengine/portalcontainer.h"
#include "iengine/portal.h"
#include "csutil/csstring.h"
#include "iutil/objreg.h"
#include "app/icsstudio.h"
#include "plugins/icssselection.h"
#include "plugins/icssview.h"
#include "plugins/icssdrawer.h"
#include "action/icssaction.h"
#include "action/icssmoveaction.h"
#include "action/icssaddselection.h"
#include "iutil/object.h"
#include "ivaria/reporter.h"


#include "sectordialog.h"

CS_IMPLEMENT_PLUGIN


SCF_IMPLEMENT_IBASE(cssObjectPlaceEMPlugin)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssEditMode)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssObjectPlaceEMPlugin::cssEditMode)
SCF_IMPLEMENTS_INTERFACE(icssEditMode)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY (cssObjectPlaceEMPlugin)

cssObjectPlaceEMPlugin::cssObjectPlaceEMPlugin(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
	SCF_CONSTRUCT_EMBEDDED_IBASE(scficssEditMode);
}

cssObjectPlaceEMPlugin::~cssObjectPlaceEMPlugin()
{
	SCF_DESTRUCT_EMBEDDED_IBASE(scficssEditMode);
  SCF_DESTRUCT_IBASE();
}


//// initialisation 
bool cssObjectPlaceEMPlugin::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
  
  return true;
}

void cssObjectPlaceEMPlugin::GetActionManager()
{
  csRef<icssPlugin> amplug = m_csi->GetPlugin("cstudio.plugin.actionmgr");
  m_actionmgr = SCF_QUERY_INTERFACE(amplug, icssActionManager);
}

void cssObjectPlaceEMPlugin::GetMeshSelector()
{
  csRef<icssPlugin> selectorplug = m_csi->GetPlugin("cstudio.select.object.mesh");
  if(selectorplug == 0) 
  {
    csReport(m_csi->getObjectRegistry(), CS_REPORTER_SEVERITY_ERROR,
		    "crystalspace.application.cstudio",
		    "Couldn't retrieve mesh selector plugin! (cstudio.select.object.mesh");
    return;
  }
  m_meshselector = SCF_QUERY_INTERFACE(selectorplug, icssSelector);
}

void cssObjectPlaceEMPlugin::GetSelectionPlugin()
{
  m_selplug = m_csi->GetPlugin("cstudio.select.defselect");
  m_sel = SCF_QUERY_INTERFACE(m_selplug, icssSelection);
}

/*
* icssEditMode implementation
*/
const char *cssObjectPlaceEMPlugin::GetDescription()
{
  return "Object Place";
}

void cssObjectPlaceEMPlugin::OnActivate(icssEditMode *lastactive)
{
  GetActionManager();
  GetMeshSelector();
  GetSelectionPlugin();
}

void cssObjectPlaceEMPlugin::OnDeactivate(icssEditMode *nextactive)
{
}

/// Event catcher, dynamic_cast to check type
bool cssObjectPlaceEMPlugin::OnEvent(wxEvent &event, icssView *view)
{
  /*
  # wxEVT_LEFT_DOWN
  # wxEVT_LEFT_UP
  # wxEVT_LEFT_DCLICK
  # wxEVT_MIDDLE_DOWN
  # wxEVT_MIDDLE_UP
  # wxEVT_MIDDLE_DCLICK
  # wxEVT_RIGHT_DOWN
  # wxEVT_RIGHT_UP
  # wxEVT_RIGHT_DCLICK
  # wxEVT_MOTION
  # wxEVT_MOUSEWHEEL 
  */
  if(event.GetEventType() == wxEVT_MOTION)
  {
    return OnMouseMove(event, view);
  }
  else
  if(event.GetEventType() == wxEVT_LEFT_DOWN)
  {
    wxMouseEvent *me = wxDynamicCast(&event, wxMouseEvent);
    if(me)
      return OnLeftDown(me, view);
  }
  else  
  if(event.GetEventType() == wxEVT_KEY_DOWN)
  {
    wxKeyEvent *kev = wxDynamicCast(&event, wxKeyEvent);
    
    switch(kev->GetKeyCode())
    {
    case WXK_F3:
      SwitchLocalSector(*kev);
      return true;
      break;
    case WXK_ESCAPE:
      ClearSelection();
      return true;
      break;
    }
  }
  // if we don't do anything we have to skip the event
  event.Skip();
  return false;
}

void cssObjectPlaceEMPlugin::SwitchLocalSector(wxKeyEvent &event)
{
  /// opens a little dialog with a list of adjacent sectors
  /// of the current active sector

  wxPoint mouse(event.m_x, event.m_y);
  wxWindow * wnd = wxDynamicCast(event.GetEventObject(), wxWindow);
  if(wnd)
    mouse = wnd->ClientToScreen(mouse);

  cssSectorDialog dlg(m_csi, NULL, -1, "Select sector", 
    mouse, wxDefaultSize);
  dlg.FillSectors(m_csi->getActiveSector());
  if(dlg.ShowModal() == wxID_OK)
  {
    SetActiveSector((char *)dlg.m_selected_sector.GetData(), true);
    wnd->Refresh();
  }
}

void cssObjectPlaceEMPlugin::SetActiveSector(csString newsector, bool inall)
{
  // inall decides wether to search in current active sector only
  // or if it's one of all sectors

  csRef<iSector> curactive = m_csi->getActiveSector();
  if(inall == false && curactive.IsValid())
  {
    csSet<csPtrKey<iMeshWrapper> >::GlobalIterator it(curactive->GetPortalMeshes().GetIterator());
    while(it.HasNext())
    {
      iMeshWrapper* mwrap = it.Next();
      csRef<iPortalContainer> pc = mwrap->GetPortalContainer();
      
      int nportals = pc->GetPortalCount();

      for(int i = 0; i < nportals; i++)
      {
        csRef<iPortal> p = pc->GetPortal(i);

        csRef<iSector> s = p->GetSector();
 
        if(s.IsValid() && strcmp(s->QueryObject()->GetName(), newsector) == 0)
        {
          m_csi->SetActiveSector(s);
          return;
        }
        
      }
    }
  }
  else
  {
    csRef<iEngine> ie = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);
    if(ie)
    {
      csRef<iSectorList> sl = ie->GetSectors();
      if(sl)
      {
        int count = sl->GetCount();
        for (int i = 0; i < count; i++)
        {
          csRef<iSector> s = sl->Get(i);
          if(s)
          {
            if(strcmp(s->QueryObject()->GetName(), newsector.GetData()) == 0)
              m_csi->SetActiveSector(s);
          }
        }
      }
    }
  }
}

bool cssObjectPlaceEMPlugin::OnMouseMove(wxEvent &e, icssView *view)
{
  wxMouseEvent *mev = wxDynamicCast(&e, wxMouseEvent);
  if(!mev)
    return false;
  
  if(mev->LeftIsDown() && mev->ControlDown() == false)
  {
    csVector3 curpos;
    view->GetMapPosfromMouseCoords(mev->m_x, mev->m_y, curpos);

    csVector3 lastpos;
    view->GetMapPosfromMouseCoords(m_lastmpos.x, m_lastmpos.y, lastpos);

    csVector3 move = curpos - lastpos;

    MoveSelection(move);
    view->Redraw();
    m_lastmpos = mev->GetPosition();
    return true;
  }

  m_lastmpos = mev->GetPosition();
  return false;
}

void cssObjectPlaceEMPlugin::MoveSelection(csVector3 &move)
{
  csRef<icssAction> a = m_actionmgr->GetLastAction();
  csRef<icssMoveAction> ma;
  if(a)
    ma = SCF_QUERY_INTERFACE(a, icssMoveAction);
	
  if(!ma.IsValid())
  {
    /// last action is not a move action -> create a new move action and move the selection
    a = SCF_CREATE_INSTANCE("cstudio.action.selmove", icssAction);
    a->Initialize(m_csi);
    ma = SCF_QUERY_INTERFACE(a, icssMoveAction);
    ma->AddMove(move);
    m_actionmgr->Perform(a);
  } else {
    ma->AddMove(move);
    a->Do();
  }
}

bool cssObjectPlaceEMPlugin::OnLeftDown(wxMouseEvent *e, icssView *view)
{
  if(e->ControlDown())
  {
    csSegment3 ray = view->GetSegmentAt(e->m_x, e->m_y);

    csRefArray<iBase> newsel;

    // fill newsel with results from the meshselector
    int hitcount = m_meshselector->HitBeam(ray.Start(), ray.End(), 0.0, newsel);
    /// 
    /// if shift is not pressed, we have to select and not add
    /// -> clear the selection first
    if(hitcount > 0)
    {
      if(!e->m_shiftDown)
      {
        csRef<icssAction> clearaction = SCF_CREATE_INSTANCE("cstudio.action.clearselection", icssAction);
        clearaction->Initialize(m_csi);
        m_actionmgr->Perform(clearaction);
    		
      }
      csRef<icssAction> addaction = SCF_CREATE_INSTANCE("cstudio.action.addselection", icssAction);
      addaction->Initialize(m_csi);
      csRef<icssAddSelectionAction> iaddsel = SCF_QUERY_INTERFACE(addaction, icssAddSelectionAction);

      for(size_t i = 0; i < newsel.Length(); i++)
        iaddsel->Add(newsel.Get(i));
    	
      m_actionmgr->Perform(addaction);
      view->Redraw();
    }
  }
  return true;
}

void cssObjectPlaceEMPlugin::ClearSelection()
{
  csRef<icssAction> clearaction = SCF_CREATE_INSTANCE("cstudio.action.clearselection", icssAction);
  clearaction->Initialize(m_csi);
  m_actionmgr->Perform(clearaction);
}


void cssObjectPlaceEMPlugin::OnDraw (icssDrawer *d, icssView *v)
{
  csColor4 c(0.7f, 0.7f, 0.7f);
  d->SetColor(c);
  csRef<iSector> s = m_csi->getActiveSector();
  if(s)
  {
    csRef<iMeshList> ml = s->GetMeshes();
    int meshcount = ml->GetCount();
    for(int i = 0; i < meshcount; i++)
    {
      csRef<iMeshWrapper> mwrap = ml->Get(i);
      if(!m_sel->Contains(mwrap))
        d->DrawMesh(mwrap);
    }
  }

  csRef<icssMeshSelection> m_meshsel = SCF_QUERY_INTERFACE(m_selplug, icssMeshSelection);
  if(!m_meshsel)
    return;

  c = csColor4(1.0, 0.0, 0.0);
 
  d->SetColor(c);
  size_t meshcount = m_meshsel->GetCount();

  for (size_t i = 0; i < meshcount; i++)
  {
    d->DrawMesh(m_meshsel->Get(i));
  }
}
