/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif

#include <wx/listbox.h>

#include "sectordialog.h"

#include "igeom/polymesh.h"
#include "iengine/engine.h"
#include "iengine/mesh.h"
#include "iengine/movable.h"
#include "imesh/object.h"
#include "igeom/objmodel.h"
#include "iengine/sector.h"
#include "iengine/portalcontainer.h"
#include "iengine/portal.h"
#include "iutil/objreg.h"
#include "csutil/csstring.h"
#include "iutil/object.h"
#include "app/icsstudio.h"

#define SECTORDIALOG_LISTBOX    wxID_HIGHEST + 1

BEGIN_EVENT_TABLE(cssSectorDialog, wxDialog)
EVT_LISTBOX_DCLICK(SECTORDIALOG_LISTBOX, cssSectorDialog::OnDClickItem)
END_EVENT_TABLE()





cssSectorDialog::cssSectorDialog
     ( iCStudioCS *csi,
       wxWindow *parent,
       wxWindowID id,
       const wxString &title,
       const wxPoint& pos,
       const wxSize& size,
       long style) : wxDialog(parent, id, title, pos, size, style)
{
  m_csi = csi;
  listbox = new wxListBox(this, 
    SECTORDIALOG_LISTBOX, wxDefaultPosition, 
    wxDefaultSize, 0, NULL, 
    wxLB_SINGLE | wxLB_NEEDED_SB | wxLB_SORT);

  wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

  // create text ctrl with minimal size 100x60
  topsizer->Add(
    listbox,
    1,            // make vertically stretchable
    wxEXPAND |    // make horizontally stretchable
    wxALL,        //   and make border all around
    10 );         // set border width to 10


  SetSizer( topsizer );      // use the sizer for layout
  topsizer->SetSizeHints( this );   // set size hints to honour minimum size

}

     
void cssSectorDialog::OnDClickItem( wxCommandEvent &event )
{
  m_selected_sector = event.GetString();
  EndModal(wxID_OK);
}

void cssSectorDialog::FillSectors(iSector *curactive)
{
  if(curactive)
  {
    csSet<csPtrKey<iMeshWrapper> >::GlobalIterator it(curactive->GetPortalMeshes().GetIterator());

    while(it.HasNext())
    {
      iMeshWrapper* mwrap = it.Next();
      csRef<iPortalContainer> pc = mwrap->GetPortalContainer();
      
      int nportals = pc->GetPortalCount();

      for(int i = 0; i < nportals; i++)
      {
        csRef<iPortal> p = pc->GetPortal(i);

        iSector *s = p->GetSector();
        if(s)
          listbox->Append(s->QueryObject()->GetName());
      }
    }
  }
  else
  {
    csRef<iEngine> ie = CS_QUERY_REGISTRY(m_csi->getObjectRegistry(), iEngine);;
    if(!ie)
      return;

    csRef<iSectorList> slist = ie->GetSectors();
    int nsecs = slist->GetCount();
    for(int i = 0; i < nsecs; i++)
    {
      csRef<iSector> s = slist->Get(i);
      if(s)
        listbox->Append(s->QueryObject()->GetName());
    }
  }
}

