/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif


#include "thingem.h"
#include "app/icsstudio.h"
#include "plugins/icssselection.h"
#include "plugins/icssview.h"
#include "plugins/icssthingselection.h"
#include "action/icssaction.h"
#include "action/icssmoveaction.h"
#include "action/icssaddselection.h"
#include "plugins/icssactionmanager.h"
#include "plugins/icssselector.h"

#include "igeom/polymesh.h"
#include "iengine/engine.h"
#include "iengine/mesh.h"
#include "iengine/movable.h"
#include "imesh/object.h"
#include "igeom/objmodel.h"
#include "iengine/sector.h"
#include "iengine/portalcontainer.h"
#include "iengine/portal.h"
#include "csutil/csstring.h"

CS_IMPLEMENT_PLUGIN


SCF_IMPLEMENT_IBASE(cssThingEditModePlugin)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssEditMode)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssThingEditModePlugin::cssEditMode)
SCF_IMPLEMENTS_INTERFACE(icssEditMode)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY (cssThingEditModePlugin)

cssThingEditModePlugin::cssThingEditModePlugin(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
	SCF_CONSTRUCT_EMBEDDED_IBASE(scficssEditMode);
}

cssThingEditModePlugin::~cssThingEditModePlugin()
{
	SCF_DESTRUCT_EMBEDDED_IBASE(scficssEditMode);
  SCF_DESTRUCT_IBASE();
}


//// initialisation 
bool cssThingEditModePlugin::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
  return true;
}

void cssThingEditModePlugin::GetActionManager()
{
  csRef<icssPlugin> amplug = m_csi->GetPlugin("cstudio.plugin.actionmgr");
  m_actionmgr = SCF_QUERY_INTERFACE(amplug, icssActionManager);
}

void cssThingEditModePlugin::GetThingSelector()
{
  csRef<icssPlugin> thingselplug = m_csi->GetPlugin("cstudio.select.thingselector");
  m_thingselector = SCF_QUERY_INTERFACE(thingselplug, icssSelector);
}

/*
* icssEditMode implementation
*/
const char *cssThingEditModePlugin::GetDescription()
{
  return "Thing";
}

void cssThingEditModePlugin::OnActivate(icssEditMode *lastactive)
{
	GetActionManager();
  GetThingSelector();
}

void cssThingEditModePlugin::OnDeactivate(icssEditMode *nextactive)
{
}

/// Event catcher, dynamic_cast to check type
bool cssThingEditModePlugin::OnEvent(wxEvent &event, icssView *view)
{
  /*
  # wxEVT_LEFT_DOWN
  # wxEVT_LEFT_UP
  # wxEVT_LEFT_DCLICK
  # wxEVT_MIDDLE_DOWN
  # wxEVT_MIDDLE_UP
  # wxEVT_MIDDLE_DCLICK
  # wxEVT_RIGHT_DOWN
  # wxEVT_RIGHT_UP
  # wxEVT_RIGHT_DCLICK
  # wxEVT_MOTION
  # wxEVT_MOUSEWHEEL 
  */

	if(event.GetEventType() == wxEVT_LEFT_DOWN)
	{
		wxMouseEvent *me = wxDynamicCast(&event, wxMouseEvent);
		if(me)
			return OnLeftDown(me, view);
	}
	else
	if(event.GetEventType() == wxEVT_MOTION)
	{
		wxMouseEvent *me = wxDynamicCast(&event, wxMouseEvent);
		if(me)
			return OnMotion(me, view);
	}
	else
	if(event.GetEventType() == wxEVT_KEY_DOWN)
	{
		wxKeyEvent *ke = wxDynamicCast(&event, wxKeyEvent);
		if(ke)
			return OnKeyDown(ke, view);
	}

  // if we don't do anything we have to skip the event
  event.Skip();
  return false;
}

bool cssThingEditModePlugin::OnKeyDown(wxKeyEvent *event, icssView* view)
{
	return false; /// we don't do anything yet
}

bool cssThingEditModePlugin::OnMotion(wxMouseEvent *event, icssView* view)
{
	if(event->LeftIsDown() && event->ControlDown() == false)
	{
		csVector3 curpos;
    view->GetMapPosfromMouseCoords(event->m_x, event->m_y, curpos);

    csVector3 lastpos;
    view->GetMapPosfromMouseCoords(m_lastmpos.x, m_lastmpos.y, lastpos);

    csVector3 move = curpos - lastpos;

    MoveSelection(move);
    view->Redraw();
		m_lastmpos = event->GetPosition();
		return true;
	}

	m_lastmpos = event->GetPosition();
	return false; /// we don't do anything yet
}

bool cssThingEditModePlugin::OnLeftDown(wxMouseEvent *event, icssView* view)
{
  if(event->ControlDown())
  {
    /// hit test
    csSegment3 ray = view->GetSegmentAt(event->m_x, event->m_y);
    csRefArray<iBase> hits;
    int hitcount = m_thingselector->HitBeam(ray.Start(), ray.End(), 1.0, hits);
    if(hitcount > 0)
    {
      if(!event->ShiftDown())
	    {
		    csRef<icssAction> clearaction = SCF_CREATE_INSTANCE("cstudio.action.clearselection", icssAction);
        clearaction->Initialize(m_csi);
		    m_actionmgr->Perform(clearaction);
    		
	    }


	    csRef<icssAction> addaction = SCF_CREATE_INSTANCE("cstudio.action.addselection", icssAction);
      addaction->Initialize(m_csi);
	    csRef<icssAddSelectionAction> iaddsel = SCF_QUERY_INTERFACE(addaction, icssAddSelectionAction);

      for(size_t i = 0; i < hits.Length(); i++)
        iaddsel->Add(hits.Get(i));
    	
	    m_actionmgr->Perform(addaction);
	    view->Redraw();
    }
  }
	return true;
}

void cssThingEditModePlugin::MoveSelection(csVector3 &v)
{
	csRef<icssAction> a = m_actionmgr->GetLastAction();
	csRef<icssMoveAction> ma;
	if(a)
		ma = SCF_QUERY_INTERFACE(a, icssMoveAction);
	
	if(!ma.IsValid())
	{
		/// last action is not a move action -> create a new move action and move the selection
		a = SCF_CREATE_INSTANCE("cstudio.action.selmove", icssAction);
		a->Initialize(m_csi);
		ma = SCF_QUERY_INTERFACE(a, icssMoveAction);
		ma->AddMove(v);
		m_actionmgr->Perform(a);
	}
	else
	{
		ma->AddMove(v);
    a->Do();
	}
}

void cssThingEditModePlugin::OnDraw(icssDrawer *d, icssView *v)
{
}
