/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __THINGEM_H_INCLUDED__
#define __THINGEM_H_INCLUDED__

#include "plugins/icsseditmode.h"
#include "plugins/icssplugin.h"
#include "csutil/csstring.h"


struct icssActionManager;
struct iSector;
struct icssSelector;
struct icssPlugin;
class csVector3;

class cssThingEditModePlugin : public icssPlugin
{
  iCStudioCS* m_csi;
  wxPoint m_lastmpos;
  csRef<icssActionManager> m_actionmgr;
  csRef<icssSelector> m_thingselector;

public:
  cssThingEditModePlugin(iBase *parent);
  virtual ~cssThingEditModePlugin();

  void GetActionManager();
  void GetThingSelector();
  //// initialisation 
  virtual bool Initialize(iCStudioCS *csi);
  virtual const char *GetName()
  { return "cstudio.em.thingem"; };

  //// Events
  bool OnKeyDown(wxKeyEvent *event, icssView* view);
  bool OnMotion(wxMouseEvent *event, icssView* view);
  bool OnLeftDown(wxMouseEvent *event, icssView* view);

  //// moves the selected thingpartsels
  void MoveSelection(csVector3 &v);

  //// icssEditMode implementation
  const char *GetDescription();
  void OnActivate(icssEditMode *lastactive);
  void OnDeactivate(icssEditMode *nextactive);
  bool OnEvent(wxEvent &event, icssView *view);
  void OnDraw(icssDrawer *d, icssView *v);

  class cssEditMode : public icssEditMode
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssThingEditModePlugin);
    virtual icssPlugin *GetPlugin()
    { return (icssPlugin*) scfParent; };
    virtual const char *GetDescription()
    { return scfParent->GetDescription(); };
    virtual void OnActivate(icssEditMode *lastactive)
    { scfParent->OnActivate(lastactive); };
    virtual void OnDeactivate(icssEditMode *nextactive)
    { scfParent->OnDeactivate(nextactive); };
    virtual bool OnEvent(wxEvent &event, icssView *view)
    { return scfParent->OnEvent(event, view); };
    virtual void OnDraw (icssDrawer *d, icssView *v)
    { scfParent->OnDraw(d, v); };
  } scficssEditMode;
  friend class cssEditMode;

  SCF_DECLARE_IBASE;
};




#endif
