#include "cssysdef.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "imesh/thing.h"
#include "imesh/genmesh.h"
#include "iengine/mesh.h"
#include "iengine/light.h"
#include "iengine/sector.h"
#include "imesh/object.h"
#include "iutil/object.h"
#include "app/icsstudio.h"
#include "plugins/icssplugin.h"
#include "plugins/icssselection.h"
#include "plugins/icssactionmanager.h"
#include "action/icssaction.h"
#include "action/icssaddselection.h"

#include <wx/image.h>
#include <wx/imaglist.h>

#include "mapbrowserpanel.h"
#include "mapbrowser.h"

#define DEFAULT_IMAGE_SIZE 16

#include "images/selected.xpm"
#include "images/notselected.xpm"

class cssRefTreeItemData : public wxTreeItemData
{
public:
  csRef<iBase> m_obj;
  cssRefTreeItemData(iBase *obj)
  { m_obj = obj;  };
  virtual ~cssRefTreeItemData()
  { };
};

enum
{
 MAP_TREE = wxID_HIGHEST + 1,
};

enum
{
	MAPBROWSER_SELECT = wxID_HIGHEST + 1,
	MAPBROWSER_ADD_TO_SELECTION
};

BEGIN_EVENT_TABLE(cssMapBrowserPanel, wxPanel)
EVT_TREE_ITEM_RIGHT_CLICK(MAP_TREE, cssMapBrowserPanel::OnRightClickItem)
EVT_MENU(MAPBROWSER_SELECT, cssMapBrowserPanel::OnSelectItem)
EVT_MENU(MAPBROWSER_ADD_TO_SELECTION, cssMapBrowserPanel::OnAddToSelection)
END_EVENT_TABLE()




cssMapBrowserPanel::cssMapBrowserPanel (cssMapBrowserPlugin *mapbrowser, 
                                        wxNotebook *parent)
                                        : wxPanel(parent) 
{
  this->mapbrowser = mapbrowser;
  m_csi = mapbrowser->m_csi;
  m_selplug = m_csi->GetPlugin("cstudio.select.defselect");
  m_sel = SCF_QUERY_INTERFACE(m_selplug, icssSelection);

  imageList = CreateImageList();

  m_tree = new wxTreeCtrl( this, MAP_TREE, 
                      wxDefaultPosition, wxSize(160,400), 
                      wxTR_HAS_BUTTONS|wxTR_LINES_AT_ROOT|wxSUNKEN_BORDER );

  m_tree->SetImageList( imageList );

  wxBoxSizer *top = new wxBoxSizer(wxVERTICAL);
  top->Add( m_tree, 1, wxGROW|wxALIGN_CENTER_HORIZONTAL|wxALL, 5 );
  SetSizer(top);
  top->SetSizeHints(this);
	GetActionManager();
}

cssMapBrowserPanel::~cssMapBrowserPanel()
{
  mapbrowser->m_mbpanel = 0;
}

void cssMapBrowserPanel::GetActionManager()
{
  csRef<icssPlugin> amplug = m_csi->GetPlugin("cstudio.plugin.actionmgr");
  m_actionmgr = SCF_QUERY_INTERFACE(amplug, icssActionManager);
}

void cssMapBrowserPanel::FillTree()
{
  /// Clear the tree first (might not be empty!)
  m_tree->DeleteAllItems();

  rootid = m_tree->AddRoot("world");
  thingsid = m_tree->AppendItem(rootid, "things");
  genmeshid = m_tree->AppendItem(rootid, "genmeshes");
  lightid = m_tree->AppendItem(rootid, "lights");

  csRef<iSector> as = m_csi->getActiveSector();
  csRef<iMeshList> ms = as->GetMeshes();

  int mc = ms->GetCount();
  for(int i = 0; i < mc; i++)
  {
    csRef<iMeshWrapper> mwrap = ms->Get(i);
    csRef<iMeshObject> mo = mwrap->GetMeshObject();

    /// Support for thing meshes
    csRef<iThingState> ts = SCF_QUERY_INTERFACE(mo, iThingState);
    if(ts.IsValid())
    {
      AddThingMesh(mwrap);
      continue;
    }
    
    /// support for genmesh meshes
    csRef<iGeneralMeshState> gm = SCF_QUERY_INTERFACE(mo, iGeneralMeshState);
    if(gm.IsValid())
    {
      AddGenMesh(mwrap);
      continue;
    }
  }

  csRef<iLightList> ls = as->GetLights();

  int lc = ls->GetCount();
  for(int ii = 0; ii < lc; ii++)
  {
    csRef<iLight> light = ls->Get(ii);
    if(light.IsValid())
    {
      AddLight(light);
      continue;
    }
  }
}

void cssMapBrowserPanel::AddThingMesh(iMeshWrapper *mwrap)
{
  if(this->m_sel->Contains((iBase*)mwrap))
    m_tree->AppendItem(thingsid, mwrap->QueryObject()->GetName(), 1, 1, new cssRefTreeItemData(mwrap));
  else
    m_tree->AppendItem(thingsid, mwrap->QueryObject()->GetName(), 0, 0, new cssRefTreeItemData(mwrap));
}

void cssMapBrowserPanel::AddGenMesh(iMeshWrapper *mwrap)
{
  if(this->m_sel->Contains((iBase*)mwrap))
    m_tree->AppendItem(genmeshid, mwrap->QueryObject()->GetName(), 1, 1, new cssRefTreeItemData(mwrap));
  else
    m_tree->AppendItem(genmeshid, mwrap->QueryObject()->GetName(), 0, 0, new cssRefTreeItemData(mwrap));
}

void cssMapBrowserPanel::AddLight(iLight *light)
{
  if(this->m_sel->Contains((iBase*)light))
    m_tree->AppendItem(lightid, light->QueryObject()->GetName(), 1, 1, new cssRefTreeItemData(light));
  else
    m_tree->AppendItem(lightid, light->QueryObject()->GetName(), 0, 0, new cssRefTreeItemData(light));
}

void cssMapBrowserPanel::OnRightClickItem (wxTreeEvent &e)
{
  m_tree->SelectItem(e.GetItem());


  wxMenu popup;
	popup.Append(MAPBROWSER_SELECT, "Select", "Select this item (clears old selection");
  popup.Append(MAPBROWSER_ADD_TO_SELECTION, "Add to Selection", 
              "Add the item (+children) from the selection");

  PopupMenu(&popup, e.GetPoint());
}

void cssMapBrowserPanel::OnSelectItem (wxCommandEvent &e)
{
	csRef<icssPlugin> selplug = m_csi->GetPlugin("cstudio.select.defselect");
	
	if(!selplug)
		return; /// @@@ error msg here ?

	csRef<icssSelection> sel = SCF_QUERY_INTERFACE(selplug, icssSelection);
	if(!sel)
		return; /// @@@ error msg here ?


	wxTreeItemId id = m_tree->GetSelection();
  if(id == rootid) // select everything
  {
    sel->Clear();
  }
  else if(id == thingsid) // select all things
  {
		csRef<icssAction> cleara = SCF_CREATE_INSTANCE("cstudio.action.clearselection", icssAction);
		cleara->Initialize(m_csi);
		m_actionmgr->Perform(cleara);

		csRef<icssAction> a = SCF_CREATE_INSTANCE("cstudio.action.addselection", icssAction);
		a->Initialize(m_csi);
		csRef<icssAddSelectionAction> adda = SCF_QUERY_INTERFACE(a, icssAddSelectionAction);
		

		long cookie;
		wxTreeItemId cur = m_tree->GetFirstChild(thingsid, cookie);
		cssRefTreeItemData * itemdata;
    do
		{
			itemdata = (cssRefTreeItemData*)m_tree->GetItemData(cur);
			if(itemdata)
			{
        adda->Add(itemdata->m_obj);
				
				/// todo: set the image of the item to selected...
			}
      cur = m_tree->GetNextChild(cur, cookie);
		} while (itemdata);

		m_actionmgr->Perform(a);
  }
  else
  {
		csRef<icssAction> cleara = SCF_CREATE_INSTANCE("cstudio.action.clearselection", icssAction);
		cleara->Initialize(m_csi);
		m_actionmgr->Perform(cleara);

		wxTreeItemId cur = m_tree->GetSelection();
    cssRefTreeItemData *itemdata = (cssRefTreeItemData*)m_tree->GetItemData(cur);
    if(itemdata)
    {
			csRef<icssAction> a = SCF_CREATE_INSTANCE("cstudio.action.addselection", icssAction);
			a->Initialize(m_csi);
			csRef<icssAddSelectionAction> adda = SCF_QUERY_INTERFACE(a, icssAddSelectionAction);
      adda->Add(itemdata->m_obj);
			m_actionmgr->Perform(a);
      /// todo: set the image of the item to selected...
    }
  }
		
}

void cssMapBrowserPanel::OnAddToSelection (wxCommandEvent &e)
{
  wxTreeItemId id = m_tree->GetSelection();
  if(id == rootid) // select everything
  {
  }
  else if(id == thingsid) // select all things
  {
		long cookie;
		wxTreeItemId cur = m_tree->GetFirstChild(thingsid, cookie);
		cssRefTreeItemData * itemdata;

		csRef<icssAction> cleara = SCF_CREATE_INSTANCE("cstudio.action.clearselection", icssAction);
		cleara->Initialize(m_csi);
		m_actionmgr->Perform(cleara);

		csRef<icssAction> a = SCF_CREATE_INSTANCE("cstudio.action.addselection", icssAction);
		a->Initialize(m_csi);
		csRef<icssAddSelectionAction> adda = SCF_QUERY_INTERFACE(a, icssAddSelectionAction);

    do
		{
			itemdata = (cssRefTreeItemData*)m_tree->GetItemData(cur);
			if(itemdata)
			{
        adda->Add(itemdata->m_obj);
				/// todo: set the image of the item to selected...
			}
      cur = m_tree->GetNextChild(cur, cookie);
		} while (itemdata);

		m_actionmgr->Perform(a);
  }
  else
  {
    wxTreeItemId cur = m_tree->GetSelection();
    cssRefTreeItemData *itemdata = (cssRefTreeItemData*)m_tree->GetItemData(cur);
    if(itemdata)
    {
			csRef<icssAction> a = SCF_CREATE_INSTANCE("cstudio.action.addselection", icssAction);
			a->Initialize(m_csi);
			csRef<icssAddSelectionAction> adda = SCF_QUERY_INTERFACE(a, icssAddSelectionAction);

      adda->Add(itemdata->m_obj);
      /// todo: set the image of the item to selected...
			m_actionmgr->Perform(a);
    }
  }
}

wxImageList* cssMapBrowserPanel :: CreateImageList()
{
	wxImageList* list = new wxImageList( DEFAULT_IMAGE_SIZE,
	  DEFAULT_IMAGE_SIZE,
	  true,
	  IMAGE_COUNT);
	wxImage image(DEFAULT_IMAGE_SIZE, DEFAULT_IMAGE_SIZE);

	imageIdList[IMAGE_NOT_SELECTED] = list->Add(wxBitmap (notselected_xpm));
	imageIdList[IMAGE_SELECTED] = list->Add(wxBitmap (selected_xpm));

	return list;
}
