#include "cssysdef.h"

#include "defselect.h"

#include "iengine/mesh.h"
#include "iengine/sector.h"

#include "app/icsstudio.h"

#include "selpanel.h"


CS_IMPLEMENT_PLUGIN;

SCF_IMPLEMENT_IBASE(cssDefaultSelection)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssSelection)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssMeshSelection)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssSidePanelTabs)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssSelectionCallback)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssThingSelection)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssDefaultSelection::cssSelection)
SCF_IMPLEMENTS_INTERFACE(icssSelection)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssDefaultSelection::cssMeshSelection)
SCF_IMPLEMENTS_INTERFACE(icssMeshSelection)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssDefaultSelection::cssSidePanelTabs)
SCF_IMPLEMENTS_INTERFACE(icssSidePanelTabs)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssDefaultSelection::cssSelectionCallback)
SCF_IMPLEMENTS_INTERFACE(icssSelectionCallback)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssDefaultSelection::cssThingSelection)
SCF_IMPLEMENTS_INTERFACE(icssThingSelection)
SCF_IMPLEMENT_EMBEDDED_IBASE_END

SCF_IMPLEMENT_FACTORY(cssDefaultSelection);

cssDefaultSelection::cssDefaultSelection(iBase *parent)
{
   SCF_CONSTRUCT_IBASE(parent);
   SCF_CONSTRUCT_EMBEDDED_IBASE(scficssSelection);
   SCF_CONSTRUCT_EMBEDDED_IBASE(scficssMeshSelection);
   SCF_CONSTRUCT_EMBEDDED_IBASE(scficssSidePanelTabs);
   SCF_CONSTRUCT_EMBEDDED_IBASE(scficssSelectionCallback);
   SCF_CONSTRUCT_EMBEDDED_IBASE(scficssThingSelection);
}

cssDefaultSelection::~cssDefaultSelection()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssThingSelection);
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssSelectionCallback);
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssSidePanelTabs);
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssMeshSelection);
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssSelection);
  SCF_DESTRUCT_IBASE();
}



//// icssPlugin implementation

bool cssDefaultSelection::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
  m_selpanel = 0; 
  return true;
}
//// icssSelection implementation

void cssDefaultSelection::Clear ()
{
  /// clear all implemented selections
  scficssMeshSelection.Clear();
  scficssThingSelection.Clear();
}

void cssDefaultSelection::Save (const char *name)
{
  /// TBD
}

void cssDefaultSelection::Load (const char *name)
{
  /// TBD
}

void cssDefaultSelection::RegisterCB( icssSelectionCallback *cb )
{
  m_callbacks.PushSmart(cb);
}
 
void cssDefaultSelection::UnregisterCB( icssSelectionCallback *cb )
{
  m_callbacks.Delete(cb);
}

void cssDefaultSelection::Remove(iBase *obj)
{
  /// check if obj is a mesh
  csRef<iMeshWrapper> mwrap = SCF_QUERY_INTERFACE(obj, iMeshWrapper);
  if(mwrap)
  {
    RemoveMesh(mwrap);
    return;
  }

  // check if obj is a thingpart
  csRef<icssThingPart> tp = SCF_QUERY_INTERFACE(obj, icssThingPart);
  if(tp)
  {
    RemoveThingPart(tp);
    return;
  }
}

void cssDefaultSelection::Add(iBase *obj)
{
  /// check if obj is a mesh
  csRef<iMeshWrapper> mwrap = SCF_QUERY_INTERFACE(obj, iMeshWrapper);
  if(mwrap)
  {
    AddMesh(mwrap);
    return;
  }

  /// check if obj is a thingpart
  csRef<icssThingPart> tp = SCF_QUERY_INTERFACE(obj, icssThingPart);
  if(tp)
  {
    AddThingPart(tp);
    return;
  }

  //// ToDo: Lights, Materials
}

bool cssDefaultSelection::Contains(iBase *obj)
{
  // check if obj is a mesh
  csRef<iMeshWrapper> mwrap = SCF_QUERY_INTERFACE(obj, iMeshWrapper);
  if(mwrap)
    return scficssMeshSelection.Contains(mwrap);

  // check if obj is a thingpart
  csRef<icssThingPart> tp = SCF_QUERY_INTERFACE(obj, icssThingPart);
  if(tp)
    return scficssThingSelection.Contains(tp);

  //// ToDo: Lights, Materials, ThingPart

  return false;
}

void cssDefaultSelection::SaveToArray(csRefArray<iBase> &target)
{
  // copy meshes
  SaveMeshesToArray(target);

  // copy thingparts
  SaveThingPartsToArray(target);
  ///// @@@TBD: Lights, Materials,  etc.
}

void cssDefaultSelection::LoadFromArray(csRefArray<iBase> &source)
{
  // load
  size_t count = source.Length();
  for(size_t i = 0; i < count; i++)
    Add(source.Get(i));
}


/// icssMeshSelection implementation
void cssDefaultSelection::ClearMeshes ()
{
  m_meshes.DeleteAll();
  OnSelectionChanged();
}

size_t cssDefaultSelection::AddMesh (iMeshWrapper *mwrap)
{
  size_t idx = m_meshes.PushSmart(mwrap);

  // trigger callbacks
  OnSelectionChanged();
  return idx;
}

void cssDefaultSelection::RemoveMesh (iMeshWrapper *mwrap)
{
  csRef<iMeshWrapper> mw = mwrap;
  m_meshes.Delete(mwrap);
  // trigger callbacks
  OnSelectionChanged();
}

void cssDefaultSelection::RemoveMesh (size_t index)
{
  csRef<iMeshWrapper> mwrap = m_meshes.Get(index);
  m_meshes.DeleteIndex(index);
  // trigger callbacks
  OnSelectionChanged();
}

bool cssDefaultSelection::ContainsMesh (iMeshWrapper *mwrap)
{
  return (m_meshes.Find(mwrap) != csArrayItemNotFound); 
}

size_t cssDefaultSelection::GetCountMesh ()
{
  return m_meshes.Length();
}

iMeshWrapper* cssDefaultSelection::GetMesh(size_t i)
{
  return m_meshes.Get(i);
}

void cssDefaultSelection::SaveMeshesToArray(csRefArray<iBase> &target)
{
  size_t count = m_meshes.Length();
  for(size_t i = 0; i < count; i++)
  {
    csRef<iMeshWrapper> mwrap = m_meshes.Get(i);
    target.PushSmart((iBase*) mwrap);
  }
}

void cssDefaultSelection::SaveThingPartsToArray(csRefArray<iBase> &target)
{
  size_t count = m_thingpart.Length();
  for(size_t i = 0; i < count; i++)
  {
    csRef<icssThingPart> tp = m_thingpart.Get(i);
    target.PushSmart((iBase *) tp);
  }
}

void cssDefaultSelection::LoadMeshesFromArray(csRefArray<iBase> &source)
{
}


//// icssSidePanelTabs implementation

int cssDefaultSelection::GetPanelCount()
{
  return 1;
}
 
wxPanel *cssDefaultSelection::GetPanel(int idx, wxNotebook *parent)
{
  switch(idx)
  {
    case 0:
      if(!m_selpanel)
        m_selpanel = new cssSelectionPanel(this, parent);
      else if(m_selpanel->GetParent() != parent)
        return 0;
      return m_selpanel;
      break;
  }
  return 0;
}

const char * cssDefaultSelection::GetPanelName(int idx)
{
  switch(idx)
  {
    case 0:
      return "Selection";
  }
  return 0;
}


//// icssSelectionCallback

void cssDefaultSelection::OnSelectionChanged()
{
  size_t n = m_callbacks.Length();
  for(size_t i = 0; i < n; i++)
  {
    csRef<icssSelectionCallback> cb = m_callbacks.Get(i);
    if(cb)
      cb->OnSelectionChanged();
  }
}

//// icssThingPartSelection implementation

void cssDefaultSelection::ClearThingPart()
{
  m_thingpart.DeleteAll();
}

size_t cssDefaultSelection::AddThingPart(icssThingPart *toadd)
{
  return m_thingpart.PushSmart(toadd);
}

size_t cssDefaultSelection::GetCountThingPart()
{
  return m_thingpart.Length();
}

void cssDefaultSelection::RemoveThingPart(icssThingPart *toremove)
{
  m_thingpart.Delete(toremove);	
}

bool cssDefaultSelection::ContainsThingPart(icssThingPart *tp)
{
  return (m_thingpart.Find(tp) != csArrayItemNotFound);
}

icssThingPart *cssDefaultSelection::GetThingPart(size_t idx)
{
  csRef<icssThingPart> thingpart = m_thingpart.Get(idx);	
  return thingpart;
}

