#ifndef __DEFSELECT_H_INCLUDED__
#define __DEFSELECT_H_INCLUDED__

// interfaces
#include "plugins/icssplugin.h"
#include "csutil/refarr.h"

#include "iengine/mesh.h"

#include "plugins/icssselection.h"
#include "plugins/icsssidepaneltabs.h"
#include "plugins/icssthingselection.h"

// implementation
#include "thingpartsel.h"

class cssSelectionPanel;

class cssDefaultSelection : public icssPlugin
{
  iCStudioCS* m_csi;
public:

  cssSelectionPanel *m_selpanel;

  cssDefaultSelection(iBase *parent);
  virtual ~cssDefaultSelection();

  SCF_DECLARE_IBASE;

  virtual bool Initialize(iCStudioCS *csi);
  virtual const char *GetName()
  { return "cstudio.select.defselect"; };

  /// cssSelection implementation
  csRefArray<icssSelectionCallback> m_callbacks;
  void Clear();
  void Save(const char *name);
  void Load(const char *name);
  void RegisterCB( icssSelectionCallback *cb );
  void UnregisterCB( icssSelectionCallback *cb );
  void Remove(iBase *obj);
  void Add(iBase *obj);
  bool Contains(iBase *obj);
  void SaveToArray(csRefArray<iBase> &target);
  void LoadFromArray(csRefArray<iBase> &source);
 
  class cssSelection : public icssSelection
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssDefaultSelection);
    virtual void Clear ()
    { scfParent->Clear(); };
    virtual void Save (const char *name)
    { scfParent->Save(name); };
    virtual void Load (const char *name)
    { scfParent->Load(name); };
    virtual void RegisterCB( icssSelectionCallback *cb)
    { scfParent->RegisterCB(cb); };
    virtual void UnregisterCB( icssSelectionCallback *cb)
    { scfParent->UnregisterCB(cb); };
    virtual void Remove(iBase *obj)
    { scfParent->Remove(obj);};
    virtual void Add(iBase *obj)
    { scfParent->Add(obj); };
    virtual bool Contains(iBase *obj)
    { return scfParent->Contains(obj); };
    virtual void SaveToArray(csRefArray<iBase> &target)
    { scfParent->SaveToArray(target); };
    virtual void LoadFromArray(csRefArray<iBase> &source)
    { scfParent->LoadFromArray(source); };
  } scficssSelection;
  friend class cssSelection;

  /// cssMeshSelection implemenation
  csRefArray<iMeshWrapper> m_meshes;
  void ClearMeshes();
  size_t AddMesh(iMeshWrapper *mwrap);
  void RemoveMesh(iMeshWrapper *mwrap);
  void RemoveMesh(size_t index);
  bool ContainsMesh(iMeshWrapper *mwrap);
  size_t GetCountMesh();
  iMeshWrapper* GetMesh(size_t i);
  void SaveMeshesToArray(csRefArray<iBase> &target);
  void LoadMeshesFromArray(csRefArray<iBase> &source);

  class cssMeshSelection : public icssMeshSelection
  {
  public:
    SCF_DECLARE_EMBEDDED_IBASE(cssDefaultSelection);

    /// Clear mesh selection
    virtual void Clear ()
    { scfParent->ClearMeshes(); };
    virtual size_t Add (iMeshWrapper *mwrap)
    { return scfParent->AddMesh(mwrap); };
    virtual void Remove (iMeshWrapper *mwrap)
    { scfParent->RemoveMesh (mwrap); };
    virtual void Remove (size_t index)
    { scfParent->RemoveMesh (index);  };
    virtual bool Contains (iMeshWrapper *mwrap)
    { return scfParent->ContainsMesh(mwrap); };
    virtual size_t GetCount ()
    { return scfParent->GetCountMesh(); };
    virtual iMeshWrapper* Get(size_t i)
    { return scfParent->GetMesh(i); };
    virtual void SaveToArray(csRefArray<iBase> &target)
    { scfParent->SaveMeshesToArray(target); };
    virtual void LoadFromArray(csRefArray<iBase> &source)
    { scfParent->LoadMeshesFromArray(source); };

  } scficssMeshSelection;
  friend class cssMeshSelection;

  //// icssSidePanelTabs implementation
  int GetPanelCount();
  wxPanel *GetPanel(int idx, wxNotebook *parent);
  const char * GetPanelName(int idx);

  class cssSidePanelTabs : public icssSidePanelTabs
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssDefaultSelection);
    virtual int Count()
    { return scfParent->GetPanelCount(); };
    virtual wxPanel * GetPanel(int idx, wxNotebook *parent)
    { return scfParent->GetPanel(idx, parent); };
    virtual const char * GetPanelName(int idx)
    { return scfParent->GetPanelName(idx);	};
  } scficssSidePanelTabs;
  friend class cssSidePanelTabs;

  //// icssSelectionCallback implementation
  //// this triggers all the callbacks
  void OnSelectionChanged();
  class cssSelectionCallback : public icssSelectionCallback
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssDefaultSelection);
    virtual void OnSelectionChanged()
    { scfParent->OnSelectionChanged(); };
  } scficssSelectionCallback;
  friend class cssSelectionCallback;

	//// icssThingSelection implementation
  csRefArray<icssThingPart> m_thingpart;
  void ClearThingPart();
  size_t AddThingPart(icssThingPart *toadd);
  size_t GetCountThingPart();
  void RemoveThingPart(icssThingPart *toremove);
  bool ContainsThingPart(icssThingPart *tp);
  icssThingPart *GetThingPart(size_t idx);
  void SaveThingPartsToArray(csRefArray<iBase> &target);

  class cssThingSelection : public icssThingSelection
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssDefaultSelection);
    virtual void Clear()
    { scfParent->ClearThingPart(); };
    virtual size_t Add(icssThingPart *toadd)
    { return scfParent->AddThingPart(toadd); };
    virtual size_t GetCount()
    { return scfParent->GetCountThingPart(); };
    virtual void Remove(icssThingPart * toremove)
    { scfParent->RemoveThingPart(toremove); };
    virtual bool Contains(icssThingPart *thingpart)
    { return scfParent->ContainsThingPart(thingpart); };
    virtual icssThingPart * Get(size_t idx)
    { return scfParent->GetThingPart(idx); };
  } scficssThingSelection;
  friend class cssThingSelection;


};

#endif

