#include "cssysdef.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#include "iutil/object.h"
#include "iengine/sector.h"

#include "defselect.h"
#include "selpanel.h"
#include "plugins/icssselection.h"

//// Treeitem data wrapper
class cssRefTreeItemData : public wxTreeItemData
{
  
public:
	csRef<iBase> m_obj;

  cssRefTreeItemData(iBase *obj)
  { m_obj = obj;  };
  virtual ~cssRefTreeItemData()
  { };
};

/// selection changed callback
class cssSelectionChangeCallback : public icssSelectionCallback
{
  cssSelectionPanel *panel;
public:
  SCF_DECLARE_IBASE;
  cssSelectionChangeCallback(cssSelectionPanel *panel)
  { 
    SCF_CONSTRUCT_IBASE(0);
    this->panel = panel; 
  };
  virtual ~cssSelectionChangeCallback()
  { };
  virtual void OnSelectionChanged()
  { panel->FillSelection();  };
};

SCF_IMPLEMENT_IBASE(cssSelectionChangeCallback);
SCF_IMPLEMENTS_INTERFACE(icssSelectionCallback);
SCF_IMPLEMENT_IBASE_END

enum
{
 SELECTION_TREE = wxID_HIGHEST + 1,
};

enum
{
  REMOVE_FROM_SELECTION = 1,
};

BEGIN_EVENT_TABLE(cssSelectionPanel, wxPanel)
  EVT_TREE_ITEM_RIGHT_CLICK(SELECTION_TREE, cssSelectionPanel::OnRightClickItem)
  EVT_MENU(REMOVE_FROM_SELECTION, cssSelectionPanel::OnRemoveSelected)
END_EVENT_TABLE()

cssSelectionPanel::cssSelectionPanel (cssDefaultSelection *sel, 
                                      wxNotebook *parent)
                                      : wxPanel(parent)  
{
  m_defsel = sel;
  /// Register callback;
  csRef<cssSelectionChangeCallback> selchangecb;
  selchangecb.AttachNew(csPtr<cssSelectionChangeCallback>(new cssSelectionChangeCallback(this)));
  csRef<icssSelectionCallback> iselchangecb = SCF_QUERY_INTERFACE(selchangecb, icssSelectionCallback);
  m_defsel->RegisterCB(iselchangecb);

  m_seltree = new wxTreeCtrl( this, SELECTION_TREE, 
                      wxDefaultPosition, wxSize(160,400), 
                      wxTR_HAS_BUTTONS|wxTR_LINES_AT_ROOT|wxSUNKEN_BORDER );

  wxBoxSizer *top = new wxBoxSizer(wxVERTICAL);
  top->Add( m_seltree, 1, wxGROW|wxALIGN_CENTER_HORIZONTAL|wxALL, 5 );
  SetSizer(top);
  top->SetSizeHints(this);
  FillSelection();
}

cssSelectionPanel::~cssSelectionPanel()
{
  m_defsel->m_selpanel = 0;
}

void cssSelectionPanel::FillSelection()
{
  /// Clear tree first
  m_seltree->DeleteAllItems();

  rootid = m_seltree->AddRoot("selection", -1, -1, new cssRefTreeItemData((iBase*)m_defsel) );

  /// meshes
  csRef<icssMeshSelection> meshsel = SCF_QUERY_INTERFACE(m_defsel, icssMeshSelection);
  if(meshsel)
  {   
    size_t nmeshes = meshsel->GetCount();
    if(nmeshes > 0)
    {
      meshesid  = m_seltree->AppendItem(rootid, "meshes", -1, -1, new cssRefTreeItemData((iBase*)meshsel));
      for(size_t i = 0; i < nmeshes; i++)
      {
        csRef<iMeshWrapper> meshwrap = meshsel->Get(i);
        if(meshwrap)
        {
          csRef<iObject> meshobj = meshwrap->QueryObject();
          m_seltree->AppendItem(meshesid, meshobj->GetName(), -1, -1, new cssRefTreeItemData((iBase*)meshwrap));
        }
      }
    }
  }
}

void cssSelectionPanel::OnRightClickItem (wxTreeEvent &e)
{
  m_seltree->SelectItem(e.GetItem());

  wxMenu popup;
  popup.Append(REMOVE_FROM_SELECTION, "Remove", 
              "Remove the item (+children) from the selection");

  PopupMenu(&popup, e.GetPoint());
}

void cssSelectionPanel::OnRemoveSelected (wxCommandEvent &e)
{
  wxTreeItemId id = m_seltree->GetSelection();
  if(id == rootid) // deselect everything
  {
    if(m_defsel)
      m_defsel->Clear();
  }
  else if(id == meshesid) // deselect all meshes
  {
    csRef<icssMeshSelection> meshsel = SCF_QUERY_INTERFACE(m_defsel, icssMeshSelection);
    if(meshsel)
      meshsel->Clear();
  }
	else // remove the single item from the list
	{
		cssRefTreeItemData * itemdata = (cssRefTreeItemData*)m_seltree->GetItemData(id);
		m_defsel->Remove(itemdata->m_obj);
	}
}
