#include "cssysdef.h"

#include "iengine/sector.h"
#include "iengine/light.h"
#include "iengine/movable.h"
#include "csutil/refarr.h"
#include "csgeom/math3d.h"

#include "app/icsstudio.h"
#include "lightselector.h"

CS_IMPLEMENT_PLUGIN

SCF_IMPLEMENT_FACTORY(cssLightSelector);

SCF_IMPLEMENT_IBASE(cssLightSelector)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssSelector)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssLightSelector::eicssMeshSelector)
SCF_IMPLEMENTS_INTERFACE(icssSelector)
SCF_IMPLEMENT_IBASE_END

cssLightSelector::cssLightSelector(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssSelector);
}


cssLightSelector::~cssLightSelector()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssSelector);
  SCF_DESTRUCT_IBASE();
}

bool cssLightSelector::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
  return true;
}

const char *cssLightSelector::GetName()
{
  return "cstudio.select.object.light";
}

int cssLightSelector::HitBeam(const csVector3 &begin, 
  const csVector3 &end, 
  float radius, 
  csRefArray<iBase> &result)
{
  iSector *s = m_csi->getActiveSector();
  if(!s)
    return 0;
  
  iLightList *lights = s->GetLights();
  
  int hits = 0;
  
  int count = lights->GetCount();
  for(int i = 0; i < count; i++)
  {
    iLight *light = lights->Get(i);
    csVector3 center = light->GetCenter();
    
    csBox3 box(center - csVector3(.1, .1, .1), center + csVector3(.1, .1, .1));
        
    csVector3 isect;
    if(csIntersect3::BoxSegment(box, csSegment3(begin, end), isect) != -1)
    {
      result.PushSmart(light);
      hits++;
    }
  }
  
  return hits;
}

int cssLightSelector::InBBox(csBox3 &bbox, csRefArray<iBase> &result)
{
  /// TBD
  return 0;
}
