#include "cssysdef.h"

#include "objectselector.h"

#include "iutil/stringarray.h"
#include "iengine/sector.h"
#include "iengine/mesh.h"
#include "csutil/refarr.h"


#include "app/icsstudio.h"


CS_IMPLEMENT_PLUGIN

SCF_IMPLEMENT_FACTORY(cssObjectSelector);

SCF_IMPLEMENT_IBASE(cssObjectSelector)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssSelector)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssObjectSelector::eicssSelector)
SCF_IMPLEMENTS_INTERFACE(icssSelector)
SCF_IMPLEMENT_IBASE_END


cssObjectSelector::cssObjectSelector(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssSelector);
}

cssObjectSelector::~cssObjectSelector()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssSelector);
  SCF_DESTRUCT_IBASE();
}

bool cssObjectSelector::Initialize(iCStudioCS *csi)
{
  m_csi = csi;
 
  csRef<iStringArray> selectors = iSCF::SCF->QueryClassList("cstudio.select.object.");
 
  printf("selectors->Length = %d\n", selectors->Length());
 
  for(size_t i = 0; i < selectors->Length(); i++)
  {
    printf("selector: %s\n", selectors->Get(i));
    
    csRef<icssPlugin> plg = csi->GetPlugin(selectors->Get(i));
    csRef<icssSelector> selector = SCF_QUERY_INTERFACE(plg, icssSelector);
    if(selector)
      this->selectors.Push(selector);
  }
 
  return true;
}

const char *cssObjectSelector::GetName()
{
	return "cstudio.select.object";
}

int cssObjectSelector::HitBeam(const csVector3 &begin, 
	 const csVector3 &end, 
	 float radius, 
	 csRefArray<iBase> &result)
{
  int hits = 0;
  
  size_t length = selectors.Length();
  for(size_t i = 0; i < length; i++)
    hits += selectors[i]->HitBeam(begin, end, radius, result);
  
  return hits;
}

int cssObjectSelector::InBBox(csBox3 &bbox, csRefArray<iBase> &result)
{
  int hits = 0;
  
  size_t length = selectors.Length();
  for(size_t i = 0; i < length; i++)
    hits += selectors[i]->InBBox(bbox, result);
  
  return hits;
}
