#include "cssysdef.h"

#include "thingselector.h"

#include "iengine/sector.h"
#include "iengine/mesh.h"
#include "csutil/refarr.h"
#include "imesh/object.h"

#include "plugins/icssthingselection.h"

#include "app/icsstudio.h"

CS_IMPLEMENT_PLUGIN

SCF_IMPLEMENT_FACTORY(cssThingSelectorPlugin);

SCF_IMPLEMENT_IBASE(cssThingSelectorPlugin)
SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssSelector)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssThingSelectorPlugin::cssThingSelector)
SCF_IMPLEMENTS_INTERFACE(icssSelector)
SCF_IMPLEMENT_IBASE_END

cssThingSelectorPlugin::cssThingSelectorPlugin(iBase *parent)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssSelector);
}


cssThingSelectorPlugin::~cssThingSelectorPlugin()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssSelector);
  SCF_DESTRUCT_IBASE();
}

bool cssThingSelectorPlugin::Initialize(iCStudioCS *csi)
{
	m_csi = csi;
	return true;
}

const char *cssThingSelectorPlugin::GetName()
{
	return "cstudio.select.thingselector";
}

int cssThingSelectorPlugin::HitBeam(const csVector3 &begin, 
																	 const csVector3 &end, 
																	 float radius, 
																	 csRefArray<iBase> &result)
{
  csRef<iSector> actives = m_csi->getActiveSector();
  if(!actives)
    return 0;

  csRef<iThingState> ts;
  csRef<iMeshWrapper> hit;
  int hitpolyidx = -1;
  csVector3 hitpoint;

  hit = actives->HitBeam(begin, end, hitpoint, &hitpolyidx, true);

  // hitpolyidx != -1 indicates hitpolyidx was modified -> thing MO hit
  if(hit)
    ts = SCF_QUERY_INTERFACE(hit->GetMeshObject(), iThingState);
  if(hit && hitpolyidx != -1 && ts.IsValid()) 
  {
    csRef<icssThingPart> tps = SCF_CREATE_INSTANCE("cstudio.select.thingpart", icssThingPart);
    tps->SetMesh(hit);
    tps->AddPolygon(hitpolyidx);
    result.Push(tps);
    return 1;
  }

	return 0;
}

int cssThingSelectorPlugin::InBBox(csBox3 &bbox, csRefArray<iBase> &result)
{
	/// TBD
	return 0;
}
