/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cssysdef.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif

#include <wx/listbox.h>

#include "csutil/ref.h"
#include "iutil/stringarray.h"

#include "choosedlg.h"



#define CHOOSEDIALOG_LISTBOX    wxID_HIGHEST + 1

BEGIN_EVENT_TABLE(cssChooseDialog, wxDialog)
EVT_LISTBOX_DCLICK(CHOOSEDIALOG_LISTBOX, cssChooseDialog::OnDClickItem)
END_EVENT_TABLE()





cssChooseDialog::cssChooseDialog
     ( wxWindow *parent,
       wxWindowID id,
       const wxString &title,
       const wxPoint& pos,
       const wxSize& size,
       long style) : wxDialog(parent, id, title, pos, size, style)
{
  listbox = new wxListBox(this, 
    CHOOSEDIALOG_LISTBOX, wxDefaultPosition, 
    wxDefaultSize, 0, NULL, 
    wxLB_SINGLE | wxLB_NEEDED_SB | wxLB_SORT);

  wxBoxSizer *topsizer = new wxBoxSizer( wxVERTICAL );

  // create text ctrl with minimal size 100x60
  topsizer->Add(
    listbox,
    1,            // make vertically stretchable
    wxEXPAND |    // make horizontally stretchable
    wxALL,        //   and make border all around
    10 );         // set border width to 10


  SetSizer( topsizer );      // use the sizer for layout
  topsizer->SetSizeHints( this );   // set size hints to honour minimum size
  
  FillViews();
}

     
void cssChooseDialog::OnDClickItem( wxCommandEvent &event )
{
  m_selected_view = event.GetString();
  EndModal(wxID_OK);
}

void cssChooseDialog::FillViews()
{
  csRef<iStringArray> vp = iSCF::SCF->QueryClassList("cstudio.view.");

  for(size_t i = 0; i < vp->Length(); i++)
  {
    listbox->Append(vp->Get(i));
  }
}

