/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cst-config.h"
#include "cssysdef.h"
#include "wx/wxprec.h"


#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif



#define ID_CSSSPLITVIEW_LEFT			10001
#define ID_CSSSPLITVIEW_RIGHT			10002

#include "app/icsstudio.h"
#include "plugins/icssplugin.h"
#include "plugins/icssview.h"
#include "plugins/icsscreateview.h"

#include "splitviewwnd.h"
#include "clickwindow.h"
#include "choosedlg.h"

BEGIN_EVENT_TABLE(cssSplitViewWnd, wxSplitterWindow)
	EVT_RIGHT_DCLICK(cssSplitViewWnd::OnLeftDClick)
  EVT_ERASE_BACKGROUND(cssSplitViewWnd::OnEraseBackground)
END_EVENT_TABLE()

cssSplitViewWnd::cssSplitViewWnd(iCStudioCS *csi, wxWindow* parent, 
									 wxWindowID id, 
									 const wxPoint& point, 
									 const wxSize& size,
									 long style, 
									 const wxString& name)
									 : wxSplitterWindow(parent, id, point, size, style, name)
{
	m_csi = csi;
	left = new cssClickWindow(this, ID_CSSSPLITVIEW_LEFT);
	right = new cssClickWindow(this, ID_CSSSPLITVIEW_RIGHT);										
	SplitVertically(left, right);
}
	
cssSplitViewWnd::~cssSplitViewWnd()
{
}

void cssSplitViewWnd::OnLeftDClick(wxMouseEvent &ev)
{
  if(ev.GetEventObject() == this)
    return;

  cssChooseDialog dlg(this, -1, "Choose new view",
                  wxPoint(ev.m_x, ev.m_y), wxSize(300, 200));
  if(dlg.ShowModal() != wxID_OK)
    return;

  csRef<icssPlugin> tmp = SCF_CREATE_INSTANCE(dlg.m_selected_view.GetData(), icssPlugin);
  if(!tmp)
    return;

  if(!tmp->Initialize(m_csi))
    return;

  csRef<icssCreateView> cv = SCF_QUERY_INTERFACE(tmp, icssCreateView);
  if(!cv)
    return;
  
	if(ev.m_id == ID_CSSSPLITVIEW_LEFT)
	{
    wxSize sz = left->GetSize();
    plugleft = tmp;
    left->Close();
    left->Destroy();
    left = cv->Create(this, "left", 0,0, sz.GetWidth(), sz.GetHeight());
    SplitVertically(left, right);
  } else 
	if(ev.m_id == ID_CSSSPLITVIEW_RIGHT)
	{
    wxSize sz = right->GetSize();
    plugright = tmp;
    right->Close();
    right->Destroy();
    right = cv->Create(this, "right", 0,0, sz.GetWidth(), sz.GetHeight());
    SplitVertically(left, right);
  }
	ev.Skip();
}

void cssSplitViewWnd::OnEraseBackground(wxEraseEvent& event)
{
  // do nothing to avoid flashing
}
