/*
    Copyright (C) 2005 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __CSS_SPLITVIEW4WND_INCLUDED__
#define __CSS_SPLITVIEW4WND_INCLUDED__

#include <wx/wx.h>

#include "csgeom/vector2.h"

struct iDocumentNode;
struct iCStudioCS;
struct icssCreateView;
class csRect;


enum
{
  CSS_VIEW1_START = 10,
  CSS_VIEW2_START = CSS_VIEW1_START + 50,
  CSS_VIEW3_START = CSS_VIEW2_START + 50,
  CSS_VIEW4_START = CSS_VIEW3_START + 50,
  CSS_VIEW_END =  CSS_VIEW4_START + 50
};


class cssSplitView4Wnd : public wxWindow
{
  iCStudioCS *csi;
  wxMenu *menu;

  struct View
  {
    wxWindow *wnd;
    csRef<icssCreateView> view;
  } views[4];
  
  int old_x, old_y;
  
  wxSize clientsize;

  csVector2 splitter_pos;
  csVector2 splitter_pos_abs;
  csVector2 splitter_pos_dragging;
  
  bool dragging_x, dragging_y;
  
  bool update;


  void InvertDraggingPos();  
  void EndDragging();
  void GetSplitterPosAbs();
  void SetSplitterPosAbs(const csVector2 &p);
  void OverSplitter(const csVector2 &p, bool *over_x, bool *over_y);
  
  void LayoutView(View *view, csRect &rect);
  
  wxMenu *CreatePluginMenu(int startid);
  
  void SetView(int view, const char* plugin);
  void SetView(int view, int plugin);
  
public:
  cssSplitView4Wnd(wxWindow *parent, iCStudioCS *csi, wxPoint pos, wxSize size);
  
  bool Load(iDocumentNode *parent);
  bool Save(iDocumentNode *parent);

  virtual bool Layout();
  virtual void OnMouse(wxMouseEvent &event);
  virtual void OnSize(wxSizeEvent &event);  
  virtual void OnPaint(wxPaintEvent &event);
  virtual void OnErase(wxEraseEvent &event);
  virtual void OnCaptureChanged(wxMouseCaptureChangedEvent &event);
  void OnMenu(wxCommandEvent &event);
  
  DECLARE_EVENT_TABLE()
};

#endif
