/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSSVIEW2DTREE_H_INCLUDED__
#define __CSSVIEW2DTREE_H_INCLUDED__

#include "csgeom/csrect.h"
#include "csgeom/transfrm.h"
#include "csgeom/box.h"
#include "csgeom/vector2.h"
#include "csutil/refarr.h"


#include "view2dgrid.h"
#include "drawer.h"

struct iCStudioCS;
struct iPolygonMesh;
struct iSector;
struct iMeshWrapper;
struct iPortal;
struct iLight;
struct iCamera;
struct icssMeshSelection;
struct icssThingSelection;
struct icssThingPart;


enum
{
  V2D_TOP = 0,
  V2D_FRONT,
  V2D_SIDE,
  V2D_PERSPECTIVE
};


enum
{
  CSS_PLANE_X = 0,
  CSS_PLANE_Y,
  CSS_PLANE_Z
};


class cssView2DDrawer
{
  iCStudioCS* m_csi;
  cssView2DWnd *m_wnd;

  csRect viewarea;  
  
public:
  csRefArray<iSector> m_sectorstr;
  
  cssView2DDrawer(csRect rect, int vt = V2D_TOP);
  ~cssView2DDrawer();

  bool Initialize(iCStudioCS *csi, cssView2DWnd *wnd);
  void Draw();
/* 
 void DrawMesh(iMeshWrapper* mesh);
  void DrawThingPart(icssThingPart *tp);
  void DrawSelection();
  void DrawMeshSelection(icssMeshSelection *ms);
  void DrawThingSelection(icssThingSelection *ts);
*/
  
  void Move(csVector3 &dir);
  void Zoom(float dir);

  void GetMapPosfromMouseCoords(int x, int y, csVector3 &omappos);
  
  void MapToView(const csVector3 &mappos, csVector2 &viewpos);
  
  bool MovePoint(const csVector3 &pos, const csVector2 &newpos2d, 
    csVector3 &newpos3d, bool y_direction = false) ;
    
  void CreateBeam(int x, int y, csSegment3 &beam);

  void SetViewArea(int width, int height);
  void SetFOV(float angle);

  cssDrawer drawer;
  cssView2DGrid grid;

  // for 2d view
  csVector2 pos;  // pos is relative to the center of the 2d view
  csVector2 last_pos;  
  float zoom;
  
  // for 3d view
  float fov;
  int ref_plane; // the plane that is used in GetMapPosfromMouseCoords()
  csRef<iCamera> camera3d;
  
  /// for testing
  csVector3 last3dpos;

  int viewtype;

  bool object_view; 
};


#endif

