/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "wx/glcanvas.h"

#include "cssysdef.h"
#include <math.h>
#include "view2dgrid.h"


cssView2DGrid::cssView2DGrid()
{
  minx = maxx = miny = maxy = 0.0f; 
}

cssView2DGrid::~cssView2DGrid()
{
}

void cssView2DGrid::GenerateGridLines(float startx, float endx, float stepx,
                                      float starty, float endy, float stepy)
{
  minx = startx;
  maxx = endx;
  miny = starty;
  maxy = endy;
  // align grid to stepx so that well pass through (0,0)

  startx = floor((startx / stepx)) * stepx;
  starty = floor((starty / stepy)) * stepy;

  // +4 because if possible rounding problems
  int nreqverts = (int)(((endx - startx) / stepx) + 
    ((endy - starty) / stepy)) + 4;
  // * 2 because of line pairs
  nreqverts *= 2;
  griddata.SetLength(nreqverts);

  // generate vertical lines
  int i = 0;
  for(float curx = startx; curx < endx; curx += stepx, i += 2)
  {
    griddata[i].x = curx;
    griddata[i].y = starty;
    griddata[i + 1].x = curx;
    griddata[i + 1].y = endy;
  }

  // generate horizontal lines
  for(float cury = starty; cury < endy; cury += stepy, i += 2)
  {
    griddata[i].x = startx;
    griddata[i].y = cury;
    griddata[i + 1].x = endx;
    griddata[i + 1].y = cury;
  }
  griddata.SetLength(i + 1);
}

void cssView2DGrid::DrawGrid(csColor rgb)
{
  glColor3f(rgb.red, rgb.green, rgb.blue);
  glEnableClientState(GL_VERTEX_ARRAY);
  glVertexPointer(2, GL_FLOAT, 0, griddata.GetArray());
  glDrawArrays(GL_LINES, 0, (GLsizei)griddata.Length());
  glDisableClientState(GL_VERTEX_ARRAY);

}

void cssView2DGrid::DrawOrigin(csColor rgb)
{
  glColor3f(rgb.red, rgb.green, rgb.blue);
  glBegin(GL_LINES);
    glVertex2f(minx, 0.0f);
    glVertex2f(maxx, 0.0f);
    glVertex2f(0.0f, miny);
    glVertex2f(0.0f, maxy);
  glEnd();
}

