/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "cst-config.h"
#include "cssysdef.h"
#include "iengine/camera.h"
#include "iengine/sector.h"
#include "iengine/mesh.h"
#include "iutil/objreg.h"
#include "imap/services.h"
#include "imesh/thing.h"
#include "imesh/object.h"
#include "csgeom/segment.h"
#include "csutil/xmltiny.h"

#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif

#include "wx/menu.h"

#include "view2dwnd.h"

#include "app/icsstudio.h"
#include "app/icsseditmodemanager.h"
#include "plugins/icsseditmode.h"
#include "plugins/icssplugin.h"
#include "plugins/icsscontextmenu.h"

enum
{
  VIEW = 10,
  VIEW_TOP,
  VIEW_SIDE,
  VIEW_FRONT,
  VIEW_PERSPECTIVE,
  VIEW_FREE,
  VIEW_WORLD,
  VIEW_OBJECT
};

BEGIN_EVENT_TABLE(cssView2DWnd, wxGLCanvas)
  EVT_SIZE(cssView2DWnd::OnSize)
  EVT_PAINT(cssView2DWnd::OnPaint)
  EVT_ERASE_BACKGROUND(cssView2DWnd::OnEraseBackground)
  EVT_ENTER_WINDOW( cssView2DWnd::OnEnterWindow )
  EVT_LEAVE_WINDOW( cssView2DWnd::OnLeaveWindow )
  EVT_LEFT_DOWN( cssView2DWnd::OnLeftDown)
  EVT_LEFT_UP( cssView2DWnd::OnLeftUp)  
  EVT_RIGHT_DOWN( cssView2DWnd::OnRightDown)
  EVT_RIGHT_UP( cssView2DWnd::OnRightUp)  
  EVT_MOTION( cssView2DWnd::OnMouseMotion )
  EVT_KEY_DOWN( cssView2DWnd::OnKeyDown)
  EVT_MENU( VIEW_TOP, cssView2DWnd::OnViewTop)
  EVT_MENU( VIEW_SIDE, cssView2DWnd::OnViewSide)
  EVT_MENU( VIEW_FRONT, cssView2DWnd::OnViewFront)
  EVT_MENU( VIEW_PERSPECTIVE, cssView2DWnd::OnViewPerspective)
  EVT_MENU( VIEW_WORLD, cssView2DWnd::OnViewWorld)
  EVT_MENU( VIEW_OBJECT, cssView2DWnd::OnViewObject)  
END_EVENT_TABLE()

SCF_IMPLEMENT_IBASE(cssView2DWnd)
  SCF_IMPLEMENTS_INTERFACE(icssView)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssView2DWnd::eicssRedrawViewCallback)
  SCF_IMPLEMENTS_INTERFACE(icssRedrawViewCallback)
SCF_IMPLEMENT_EMBEDDED_IBASE_END


cssView2DWnd::cssView2DWnd(wxWindow *parent, const wxWindowID id, 
			const wxPoint& pos, const wxSize& size,
			long style, const wxString& name, iCStudioCS *csi) :
  wxGLCanvas(parent, (wxGLCanvas*) NULL, id, pos, size, style, name ),
		drawer(csRect(0, 0, size.x, size.y), V2D_TOP)
{

  SCF_CONSTRUCT_IBASE(0);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssRedrawViewCallback);

  this->csi = (iCStudioCS*) csi;
  drawer.Initialize(csi, this);
  
  csi->RegisterRedrawViewCB(&scficssRedrawViewCallback);
}

cssView2DWnd::~cssView2DWnd()
{
  csi->UnregisterRedrawViewCB(&scficssRedrawViewCallback);
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssRedrawViewCallback);
  SCF_DESTRUCT_IBASE();
}

void cssView2DWnd::Render()
{

#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

  SetCurrent();
	
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  drawer.Draw();

  glFlush();
  SwapBuffers();
}

void cssView2DWnd::OnEnterWindow( wxMouseEvent& event )
{
  SetFocus();
}

void cssView2DWnd::OnPaint( wxPaintEvent& event )
{
  wxPaintDC dc(this);
  Render();
}

void cssView2DWnd::OnSize(wxSizeEvent& event)
{
  // this is also necessary to update the context on some platforms
  wxGLCanvas::OnSize(event);

  // set GL viewport (not called by wxGLCanvas::OnSize on all platforms...)
  int w, h;
  GetClientSize(&w, &h);
#ifndef __WXMOTIF__
  if (GetContext())
#endif
  {
    SetCurrent();
    //drawer.viewarea.Set(0,0,event.m_size.x, event.m_size.y);
    drawer.SetViewArea(event.m_size.x, event.m_size.y);

    glViewport(0, 0, (GLint) w, (GLint) h);
  }
}

void cssView2DWnd::OnEraseBackground(wxEraseEvent& event)
{
  // Do nothing, to avoid flashing.
}

void cssView2DWnd::OnMouseMotion(wxMouseEvent &event)
{
  if(event.m_controlDown && GetCapture() == this)
  {
    if(drawer.viewtype == V2D_PERSPECTIVE)
      OnMove3D(event);   
    else
      OnMove2D(event);
  }

  event.Skip();
}

void cssView2DWnd::OnMove3D(wxMouseEvent &event)
{
  int w, h;
  GetClientSize(&w, &h);

  int offset_x = event.m_x - (w/2);
  int offset_y = event.m_y - (h/2);      
  

  if(event.m_rightDown && event.m_shiftDown)  // set fov
    drawer.SetFOV(drawer.fov + (float)offset_y / 5.0f);
  
  else if(event.m_leftDown && event.m_rightDown) // move camera in y direction
    drawer.camera3d->Move(CS_VEC_DOWN  *  (float)offset_y / 5.0f);
  
  else if(event.m_leftDown) // move camera int x and z direction
  {
    drawer.camera3d->Move(CS_VEC_RIGHT *  (float)offset_x / 5.0f);
    drawer.camera3d->Move(CS_VEC_BACKWARD * (float)offset_y / 5.0f);
  }

  else if(event.m_rightDown) // rotate camera
  {
    float x = (float)offset_y / 200.0f;
    float y = (float)offset_x / 200.0f;

    csMatrix3 mat = drawer.camera3d->GetTransform().GetO2T();
    
    if(x)
      mat = csXRotMatrix3(-x) * mat;
    if(y)
      mat *= csYRotMatrix3(y);

    drawer.camera3d->GetTransform().SetO2T(mat);         
  }
  
  Render();
  
  if(offset_x != 0 || offset_y != 0)
    WarpPointer (w / 2, h / 2);
}

void cssView2DWnd::OnMove2D(wxMouseEvent &event)
{
  int w, h;
  GetClientSize(&w, &h);

  wxPoint curpos = event.GetPosition();
  wxPoint center;
  center.x = w / 2;
  center.y = h / 2;

  if(event.m_leftDown)
  {
    int offset_y = curpos.y - center.y;
    drawer.Zoom((float)offset_y / 40.0f);
    if(offset_y != 0)
      WarpPointer(w / 2, h / 2);
  }
  else if(event.m_rightDown)
  {
    csVector3 curmappos, lastmappos;
  
    drawer.GetMapPosfromMouseCoords(curpos.x, curpos.y, curmappos);
    drawer.GetMapPosfromMouseCoords(center.x, center.y, lastmappos);

    csVector3 move = curmappos - lastmappos;

    drawer.Move(move);

    wxPoint curmove = curpos - center;
	
    if(abs(curmove.x * curmove.x + curmove.y * curmove.y) > 1)
      WarpPointer(w / 2, h / 2);
  }
  
  Render();
}

void cssView2DWnd::OnLeaveWindow(wxMouseEvent& event )
{
  event.Skip();
}

void cssView2DWnd::OnLeftDown(wxMouseEvent& event)
{
  if(GetCapture() !=this)
    CaptureMouse();

  if(event.m_controlDown)
  {

    SetCursor(wxCursor(wxCURSOR_BLANK)); 
    int w, h;
    GetClientSize(&w, &h);
    WarpPointer(w / 2, h / 2);
  }
  else
    event.Skip();
}

void cssView2DWnd::OnLeftUp(wxMouseEvent& event)
{
  if(HasCapture() && !event.m_rightDown)
  {
    SetCursor(wxCursor(wxCURSOR_ARROW));
    ReleaseMouse();
  }

  if(!event.m_controlDown)
  {

  }
  else
    event.Skip();
}

void cssView2DWnd::OnRightDown(wxMouseEvent& event)
{
  if(event.m_controlDown)
  {
    if(!HasCapture())
      CaptureMouse();
  
    SetCursor(wxCursor(wxCURSOR_BLANK));    
    int w, h;
    GetClientSize(&w, &h);
    WarpPointer(w / 2, h / 2);
  }       
  else // do a nice little pop-up menu
    DoPopup(wxPoint(event.m_x, event.m_y));
}

void cssView2DWnd::OnRightUp(wxMouseEvent& event)
{
  if(HasCapture() && !event.m_leftDown)
  {
    SetCursor(wxCursor(wxCURSOR_ARROW));
    ReleaseMouse();
  }
}

void cssView2DWnd::OnKeyDown(wxKeyEvent& event)
{
  switch(event.GetKeyCode())
  {
    case 'Y':
      drawer.Zoom(1);
      Render();
      break;
    case 'A':
      drawer.Zoom(-1);
      Render();
      break;
    case 'P':
      drawer.ref_plane++;
      if(drawer.ref_plane > CSS_PLANE_Z)
        drawer.ref_plane = CSS_PLANE_X;
      break;       
  }

  event.Skip();
}

bool cssView2DWnd::ProcessEvent(wxEvent &event)
{
  if(wxGLCanvas::ProcessEvent(event))
    return true;

  // the window itself doesn't handle the event
  // -> pass it to the current active EM

  csRef<icssPlugin> emplug = csi->GetPlugin("cstudio.plugin.emmanager");
  csRef<icssEditModeManager> emmgr = SCF_QUERY_INTERFACE(emplug, icssEditModeManager);
  csRef<icssEditMode> em = emmgr->GetActiveEditMode();
  if(em)
    return em->OnEvent(event, this);

  // else... there's no active Edit Mode
  return false;
}

void cssView2DWnd::DoPopup(wxPoint pos)
{
  wxMenu menu;

  wxMenu *viewsm = new wxMenu();
  
  viewsm->AppendRadioItem(VIEW_WORLD, "World View");  
  viewsm->AppendRadioItem(VIEW_OBJECT, "Object View");    
  
  viewsm->Check(VIEW_OBJECT, drawer.object_view);
  
  viewsm->AppendSeparator();
  
  viewsm->AppendRadioItem(VIEW_TOP, "Top", "Set this view to view from the top");
  viewsm->Check(VIEW_TOP, drawer.viewtype == V2D_TOP);
  viewsm->AppendRadioItem(VIEW_SIDE, "Side", "Set this view to view from the side");
  viewsm->Check(VIEW_SIDE, drawer.viewtype == V2D_SIDE);  
  viewsm->AppendRadioItem(VIEW_FRONT, "Front", "Set this view to view from the front");
  viewsm->Check(VIEW_FRONT, drawer.viewtype == V2D_FRONT);  
  viewsm->AppendRadioItem(VIEW_PERSPECTIVE, "Perspective", "Set this view to view a perspective");
  viewsm->Check(VIEW_PERSPECTIVE, drawer.viewtype == V2D_PERSPECTIVE);
  
  menu.Append(VIEW, "&View", viewsm);
  
  csRef<icssPlugin> emplug = csi->GetPlugin("cstudio.plugin.emmanager");
  csRef<icssEditModeManager> emmgr = SCF_QUERY_INTERFACE(emplug, icssEditModeManager);
  csRef<icssEditMode> em = emmgr->GetActiveEditMode();

  if(em) 
  {
    csRef<icssContextMenu> cm = SCF_QUERY_INTERFACE(em->GetPlugin(), icssContextMenu);
    if(cm) // if there's a valid context menu for current plugin, add it here
     cm->AppendContextMenu(&menu);
  }
  
  PopupMenu(&menu, pos);
}

void cssView2DWnd::OnViewTop(wxCommandEvent & event)
{
  drawer.viewtype = V2D_TOP;
  Render();
}

void cssView2DWnd::OnViewSide(wxCommandEvent & event)
{
  drawer.viewtype = V2D_SIDE;
  Render();
}

void cssView2DWnd::OnViewFront(wxCommandEvent & event)
{
  drawer.viewtype = V2D_FRONT;
  Render();
}

void cssView2DWnd::OnViewPerspective(wxCommandEvent & event)
{
  drawer.viewtype = V2D_PERSPECTIVE;
  Render();
}

void cssView2DWnd::OnViewWorld(wxCommandEvent & event)
{
  drawer.object_view = false;
  Render();
}
void cssView2DWnd::OnViewObject(wxCommandEvent & event)
{
  drawer.object_view = true;
  Render();
}

//// icssView implementation
void cssView2DWnd::GetMapPosfromMouseCoords(int x, int y, csVector3 &omappos)
{
  drawer.GetMapPosfromMouseCoords(x, y, omappos);
};

void cssView2DWnd::MapToView(const csVector3 &mappos, csVector2 &viewpos)
{
  drawer.MapToView(mappos, viewpos);
}

csSegment3 cssView2DWnd::GetSegmentAt(int x, int y)
{
  /// Hit Meshes first
  csVector3 start, end, intersect;
  drawer.GetMapPosfromMouseCoords(x, y, start);
  end = start;

  if(drawer.viewtype == V2D_SIDE)
  {
    	start.x = -100.0f;
    	end.x = 100.0f;
  } 
  else if(drawer.viewtype == V2D_FRONT)
  {
    	start.y = -100.0f;
    	end.y = 100.0f;
  } 
  else if(drawer.viewtype == V2D_TOP)
  {
    	start.z = -100.0f;
  end.z = 100.0f;
  }

  return csSegment3(start, end);
}

csRef<iDocumentNode> cssView2DWnd::CreateChildNode(
  iDocumentNode* parent, const char* name)
{
  csRef<iDocumentNode> child = parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
  child->SetValue (name);
  return child;
}

bool cssView2DWnd::SaveXML(iDocumentNode *parent)
{
  // TBD
  
  parent->SetAttributeAsInt("type", drawer.viewtype);
  
  return true;
}

bool cssView2DWnd::LoadXML(iDocumentNode *parent)
{
  // TBD
  
  drawer.viewtype = parent->GetAttributeValueAsInt("type");
  
  return true;
}
