/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifndef __VIEW2DWND_H_INCLUDED__
#define __VIEW2DWND_H_INCLUDED__

#include "wx/glcanvas.h"
#include "csgeom/vector3.h"
#include "csgeom/segment.h"
#include "app/icsstudio.h"
#include "plugins/icssview.h"
#include "plugins/icssactionmanager.h"
#include "view2ddrawer.h"

struct iDocumentNode;

class cssView2DWnd : public wxGLCanvas, public icssView
{
public:
  SCF_DECLARE_IBASE;

  cssView2DGrid grid; // used for drawing the grid
  iCStudioCS* csi;
  
  cssView2DDrawer drawer;
  cssView2DWnd(wxWindow *parent, const wxWindowID id = -1, 
  		const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
  		long style = 0, const wxString& name = "View2DWnd", iCStudioCS *csi = 0);
  	   
  ~cssView2DWnd();

  void OnPaint(wxPaintEvent& event);
  void OnSize(wxSizeEvent& event);
  void OnEraseBackground(wxEraseEvent& event);
  void OnEnterWindow( wxMouseEvent& event );
  void OnLeaveWindow( wxMouseEvent& event );
  void OnMouseMotion( wxMouseEvent& event );
  void OnLeftDown(wxMouseEvent& event);
  void OnLeftUp(wxMouseEvent& event);  
  void OnRightDown(wxMouseEvent& event);
  void OnRightUp(wxMouseEvent& event);  
  void OnMove2D(wxMouseEvent &event);
  void OnMove3D(wxMouseEvent &event);  
  void OnKeyDown(wxKeyEvent &event);
  void OnViewTop(wxCommandEvent & event);
  void OnViewSide(wxCommandEvent & event);
  void OnViewFront(wxCommandEvent & event);
  void OnViewPerspective(wxCommandEvent & event);
  void OnViewWorld(wxCommandEvent & event);  
  void OnViewObject(wxCommandEvent & event);    

  bool SaveXML(iDocumentNode *parent);
  bool LoadXML(iDocumentNode *parent);
  csRef<iDocumentNode> CreateChildNode(iDocumentNode* parent, const char* name);
  
  virtual bool ProcessEvent(wxEvent &event);

  /// icssView implementation
  virtual void CreateBeam(int x, int y, csSegment3 &beam)
  {  drawer.CreateBeam(x, y, beam); }
  virtual bool MovePoint(const csVector3 &pos, const csVector2 &newpos2d, 
    csVector3 &newpos3d, bool y_direction)
  {  return drawer.MovePoint(pos, newpos2d, newpos3d, y_direction); }
  virtual void GetMapPosfromMouseCoords(int x, int y, csVector3 &omappos);
  virtual void MapToView(const csVector3 &mappos, csVector2 &viewpos);
  virtual void Redraw()
  {  Render(); }
  virtual csSegment3 GetSegmentAt(int x, int y);
  virtual wxWindow * GetWxWindow()
  { return (wxWindow*) this; };
  void DoPopup(wxPoint pos);
  
  struct eicssRedrawViewCallback : public icssRedrawViewCallback
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssView2DWnd);
    virtual void Redraw()
    { scfParent->Render(); }
  } scficssRedrawViewCallback;

  void Render();

  DECLARE_EVENT_TABLE();
};

#endif
