#include "cssysdef.h"
#include "csswmbase.h"
#include "app/icsstudio.h"

#include "csutil/xmltiny.h"
#include "iutil/objreg.h"
#include "iutil/vfs.h"
#include "iutil/stringarray.h"

#include <wx/frame.h>
#include <wx/menu.h>
#include <wx/log.h>



SCF_IMPLEMENT_IBASE(cssWindowManagerBase)
  SCF_IMPLEMENTS_INTERFACE(icssWindowManager)
  SCF_IMPLEMENTS_EMBEDDED_INTERFACE(icssPlugin)
SCF_IMPLEMENT_IBASE_END

SCF_IMPLEMENT_EMBEDDED_IBASE(cssWindowManagerBase::eicssPlugin)
  SCF_IMPLEMENTS_INTERFACE(icssPlugin)
SCF_IMPLEMENT_EMBEDDED_IBASE_END


BEGIN_EVENT_TABLE( cssWindowManagerBaseEvtHandler, wxEvtHandler )
  EVT_MENU( CSS_WINDOW_CLOSE_VIEWS, cssWindowManagerBaseEvtHandler::OnCloseViews )
  EVT_MENU( CSS_WINDOW_SHOW_VIEW3D, cssWindowManagerBaseEvtHandler::OnShowView3D )
  EVT_MENU( CSS_WINDOW_SHOW_CONSOLE, cssWindowManagerBaseEvtHandler::OnShowConsole )
  EVT_MENU( CSS_WINDOW_SHOW_PANELS, cssWindowManagerBaseEvtHandler::OnShowPanels )
  EVT_MENU_RANGE( CSS_WINDOW_NEW_VIEW_START, CSS_WINDOW_NEW_VIEW_END, cssWindowManagerBaseEvtHandler::OnNewViewRange )
END_EVENT_TABLE( )

cssWindowManagerBase::cssWindowManagerBase(iBase *parent) :
   mbevh(0), csi(0), mainframe(0), menu(0)
{
  SCF_CONSTRUCT_IBASE(parent);
  SCF_CONSTRUCT_EMBEDDED_IBASE(scficssPlugin);
}

cssWindowManagerBase::~cssWindowManagerBase()
{
  SCF_DESTRUCT_EMBEDDED_IBASE(scficssPlugin);
  SCF_DESTRUCT_IBASE();
  delete mbevh;
}

wxEvtHandler *cssWindowManagerBase::CreateEventHandler()
{
  return new cssWindowManagerBaseEvtHandler( this );
}

bool cssWindowManagerBase::Initialize(iCStudioCS *csi)
{
  this->csi = csi;
  mbevh = CreateEventHandler();
  return true;
}

csRef<iDocumentNode> cssWindowManagerBase::CreateChildNode(
  iDocumentNode* parent, const char* name)
{
  csRef<iDocumentNode> child = parent->CreateNodeBefore(CS_NODE_ELEMENT, 0);
  child->SetValue (name);
  return child;
}

csRef<iDocumentNode> cssWindowManagerBase::GetOurNode(iDocumentNode *root)
{
  csRef<iDocumentNodeIterator> it = root->GetNodes("windowmanager");

  while(it->HasNext())
  {
    csRef<iDocumentNode> node = it->Next();
    const char *name = node->GetAttributeValue("name");
    if(strcmp(name, GetName()) == 0)
      return node;
  }

  return 0;
}

csRef<iDocumentNode> cssWindowManagerBase::CreateOurNode(iDocumentNode *root)
{
  csRef<iDocumentNode> node = CreateChildNode(root, "windowmanager");
  node->SetAttribute("name", GetName());
  
  return node;
}

void cssWindowManagerBase::SaveSettings(const char *vfspath)
{
  csRef<iVFS> vfs = CS_QUERY_REGISTRY(csi->getObjectRegistry(), iVFS);
  csRef<iFile> f = vfs->Open(vfspath, VFS_FILE_READ);

  csRef<iDocumentSystem> docsys;
  docsys.AttachNew(csPtr<iDocumentSystem>(new csTinyDocumentSystem()));

  csRef<iDocument> doc = docsys->CreateDocument();
  
  bool parsed = false;
  if(f.IsValid())  
  {
    if(doc->Parse(f) == 0)
      parsed = true;
  }
  if(!parsed)
    doc->CreateRoot();

  csRef<iDocumentNode> node = GetOurNode(doc->GetRoot());
  if(!node)
    node = CreateOurNode(doc->GetRoot());

  node->RemoveNodes();

  SaveSettings(node);

  f = vfs->Open(vfspath, VFS_FILE_WRITE);
  doc->Write(f);
}

void cssWindowManagerBase::SaveSettings(iDocumentNode *parent)
{
  // write mainframe
  csRef<iDocumentNode> mainfrmnode = CreateChildNode(parent, "mainframe");
  SaveCoordinates(mainframe, mainfrmnode);

  // write view3d
  csRef<iDocumentNode> v3dnode = CreateChildNode(parent, "view3d");
  wxFrame *view3dframe = csi->GetView3DFrame();
  SaveCoordinates(view3dframe, v3dnode);
  v3dnode->SetAttributeAsInt("show", view3dframe->IsShown());
}

void cssWindowManagerBase::LoadSettings(const char *vfspath)
{
  csRef<iDocumentSystem> docsys;
  docsys.AttachNew(csPtr<iDocumentSystem>(new csTinyDocumentSystem()));

  csRef<iVFS> vfs = CS_QUERY_REGISTRY(csi->getObjectRegistry(), iVFS);
  csRef<iFile> f = vfs->Open(vfspath, VFS_FILE_READ);
  
  if(f.IsValid())
  {
    csRef<iDocument> doc = docsys->CreateDocument();
    if(doc->Parse(f) == 0)
    {
      csRef<iDocumentNode> node = GetOurNode(doc->GetRoot());
      if(node)
        LoadSettings(node);
    }       
  }
}

void cssWindowManagerBase::LoadSettings(iDocumentNode *parent)
{
  GetMainFrame(parent);
  GetView3d(parent);
}

void cssWindowManagerBase::SaveCoordinates(wxFrame *frame, iDocumentNode *parent)
{
  wxPoint p = frame->GetPosition();
  wxSize sz = frame->GetSize();
  bool maximized = frame->IsMaximized();

  if(frame->IsIconized() || maximized)
  {
    p.x = 0; p.y =0;
    sz.Set(400,400);
  }

  parent->SetAttributeAsInt("maximized", maximized);
  parent->SetAttributeAsInt("x", p.x);
  parent->SetAttributeAsInt("y", p.y);
  parent->SetAttributeAsInt("w", sz.GetWidth());
  parent->SetAttributeAsInt("h", sz.GetHeight());
}

void cssWindowManagerBase::GetCoordinates(wxFrame *frame, iDocumentNode *parent)
{
  int x,y,w,h;
  x = parent->GetAttributeValueAsInt("x");
  y = parent->GetAttributeValueAsInt("y");
  w = parent->GetAttributeValueAsInt("w");
  h = parent->GetAttributeValueAsInt("h");

  frame->Maximize(parent->GetAttributeValueAsInt("maximized"));
  if(!frame->IsMaximized() && !frame->IsIconized())
    frame->SetSize(x,y,w,h);  
}

void cssWindowManagerBase::GetMainFrame(iDocumentNode *parent)
{
  csRef<iDocumentNode> mainfrmnode = parent->GetNode("mainframe");
  if(mainfrmnode)
    GetCoordinates(mainframe, mainfrmnode);
  mainframe->Show(true);
}

void cssWindowManagerBase::GetView3d(iDocumentNode *parent)
{
  csRef<iDocumentNode> view3dnode = parent->GetNode("view3d");
  if(view3dnode)
  {
    wxFrame *view3dframe = csi->GetView3DFrame();
    GetCoordinates(view3dframe, view3dnode);
    view3dframe->Show(view3dnode->GetAttributeValueAsInt("show"));
  }
}

void cssWindowManagerBase::AppendMenuItems()
{
  menu->Append(CSS_WINDOW_SHOW_VIEW3D, "3D View");
  menu->AppendCheckItem(CSS_WINDOW_SHOW_PANELS, "Panels");
  menu->AppendCheckItem(CSS_WINDOW_SHOW_CONSOLE, "Console");
  
  //menu->Append(CSS_WINDOW_CLOSE_VIEWS, "Close All Views");
  
  //AppendViewPluginMenu();
}

wxMenu *cssWindowManagerBase::CreateViewPluginMenu(int startid)
{
  wxMenu *m = new wxMenu();
  
  csRef<iStringArray> vp = iSCF::SCF->QueryClassList("cstudio.view.");
  vp->Sort();

  size_t count = vp->Length();
  for(size_t i = 0; i < count; i++)
  {
    m->Append(startid + i, vp->Get(i));
  }

  return m;
}

void cssWindowManagerBase::ShowView3D(bool show)
{
  csi->GetView3DFrame()->Show(show);
}

void cssWindowManagerBase::Activate(wxFrame *mainframe)
{
  this->mainframe = mainframe;
  this->menu = new wxMenu();
  mainframe->GetMenuBar()->Insert(CSS_WINDOW_MENU_POS, menu, "&Window");
  mainframe->PushEventHandler( mbevh );
}

void cssWindowManagerBase::Deactivate()
{
  mainframe->GetMenuBar()->Remove(CSS_WINDOW_MENU_POS);
  delete menu;
  mainframe->RemoveEventHandler( mbevh );
  mainframe = 0;
  menu = 0;
}

void cssWindowManagerBase::OnNewViewRange(wxCommandEvent &event)
{
  int id = event.GetId() - CSS_WINDOW_NEW_VIEW_START;
    
  csRef<iStringArray> vp = iSCF::SCF->QueryClassList("cstudio.view.");
  vp->Sort();

  CreateView(vp->Get(id));
}

void cssWindowManagerBase::OnShowConsole(wxCommandEvent &event)
{
  ShowConsole(menu->IsChecked(event.GetId()));
}

void cssWindowManagerBase::OnShowPanels(wxCommandEvent &event)
{
  ShowPanels(menu->IsChecked(event.GetId()));
}

void cssWindowManagerBase::OnShowView3D(wxCommandEvent &event)
{
  ShowView3D( true );
}
