#ifndef __CSSWMBASE_H_INCLUDED__
#define __CSSWMBASE_H_INCLUDED__


#include "plugins/icsswindowmanager.h"
#include "plugins/icssplugin.h"
#include <wx/event.h>

struct iDocumentNode;
struct iCStudioCS;
class wxFrame;
class wxMenu;
struct wxCommandEvent;


#define CSS_WINDOW_SETTINGS_PATH "/cstudio/config/cstudio_windowsettings.xml"
#define CSS_WINDOW_MENU_POS 2
#define CSS_MAX_VIEWPLUGIN_COUNT 20

enum
{
  CSS_WINDOW_SHOW_CONSOLE = 200, //= CSS_WINDOW_START,
  CSS_WINDOW_SHOW_PANELS,
  CSS_WINDOW_SHOW_VIEW3D,
  CSS_WINDOW_CLOSE_VIEWS,
  CSS_WINDOW_NEW_VIEW,
  CSS_WINDOW_NEW_VIEW_START,
  CSS_WINDOW_NEW_VIEW_END = CSS_WINDOW_NEW_VIEW_START + CSS_MAX_VIEWPLUGIN_COUNT,
  CSS_WINDOW_LOAD,
  CSS_WINDOW_SAVE,
  CSS_WINDOW_COMMON_END
};

class cssWindowManagerBaseEvtHandler;

class cssWindowManagerBase : public icssWindowManager
{
protected:
  wxEvtHandler *mbevh;
  iCStudioCS *csi;
  wxFrame *mainframe;
  wxMenu *menu;
  
  virtual wxEvtHandler *CreateEventHandler();
  
  virtual void AppendMenuItems();
  wxMenu *CreateViewPluginMenu(int startid);
  
  virtual void CreateView(const char *classid) {}
  virtual void CloseViews() {}
  
  virtual void ShowPanels(bool show) {}
  virtual void ShowConsole(bool show) {}
  virtual void ShowView3D(bool show);

  void LoadSettings()
  { LoadSettings(CSS_WINDOW_SETTINGS_PATH); }
  void SaveSettings()
  { SaveSettings(CSS_WINDOW_SETTINGS_PATH); }
  void SaveSettings(const char *vfspath);
  void LoadSettings(const char *vfspath);
  virtual void LoadSettings(iDocumentNode *parent);
  virtual void SaveSettings(iDocumentNode *parent);    

  void SaveCoordinates(wxFrame *frame, iDocumentNode *parent);
  void GetCoordinates(wxFrame *frame, iDocumentNode *parent);
  void GetMainFrame(iDocumentNode *parent);
  void GetView3d(iDocumentNode *parent);  
  
  csRef<iDocumentNode> GetOurNode(iDocumentNode *root);
  csRef<iDocumentNode> CreateOurNode(iDocumentNode *root);
  csRef<iDocumentNode> CreateChildNode(iDocumentNode* parent, const char* name);

public:
  SCF_DECLARE_IBASE;
  
  void OnCloseViews(wxCommandEvent &event)
  { CloseViews( ); }
  
  void OnNewViewRange(wxCommandEvent &event);
  void OnShowConsole(wxCommandEvent &event);
  void OnShowPanels(wxCommandEvent &event);
  void OnShowView3D(wxCommandEvent &event);

  cssWindowManagerBase (iBase* parent);
  virtual ~cssWindowManagerBase ();
  
  /// icssPlugin implementation
  struct eicssPlugin : public icssPlugin
  {
    SCF_DECLARE_EMBEDDED_IBASE(cssWindowManagerBase);
    virtual bool Initialize(iCStudioCS *csi)
    { return scfParent->Initialize(csi); }
    virtual const char *GetName()
    { return scfParent->GetName(); }
  } scficssPlugin;
  friend struct eicssPlugin;

  bool Initialize(iCStudioCS *csi);
  virtual const char *GetName() = 0;
  
  /// icssWindowManager implementation
  virtual void Activate(wxFrame* mainframe);
  virtual void Deactivate();
  
  wxMenu* GetMenu()
  { return menu; }
};


class cssWindowManagerBaseEvtHandler : public wxEvtHandler
{
protected:
  cssWindowManagerBase *mb;
  void OnCloseViews( wxCommandEvent &event ) { mb->OnCloseViews( event ); }
  void OnNewViewRange( wxCommandEvent &event ) { mb->OnNewViewRange( event ); }
  void OnShowConsole( wxCommandEvent &event ) { mb->OnShowConsole( event ); }
  void OnShowPanels( wxCommandEvent &event ) { mb->OnShowPanels( event ); }
  void OnShowView3D( wxCommandEvent &event ) { mb->OnShowView3D( event ); }

public:
  cssWindowManagerBaseEvtHandler( cssWindowManagerBase *mb ) { this->mb = mb; }

  DECLARE_EVENT_TABLE( )
};

#endif
