#include "cssysdef.h"

#include "app/icsstudio.h"
#include "plugins/icssplugin.h"
#include "plugins/icsscreateview.h"
#include "iutil/document.h"
#include "iutil/vfs.h"

#include "csswindowmanager.h"
#include "cssmbframe.h"
#include "csschildframe.h"

#include <wx/menu.h>

CS_IMPLEMENT_PLUGIN;

SCF_IMPLEMENT_FACTORY(cssWindowManagerMultiframe);

cssWindowManagerMultiframe::cssWindowManagerMultiframe(iBase *parent) :
   cssWindowManagerBase(parent), mbframe(0), consolewnd(0)
{
}

cssWindowManagerMultiframe::~cssWindowManagerMultiframe()
{
}

void cssWindowManagerMultiframe::Add(cssChildFrame *cf)
{
  frames.Push(cf);
}

void cssWindowManagerMultiframe::Remove(cssChildFrame *cf)
{
  frames.Delete(cf);
}

void cssWindowManagerMultiframe::SaveSettings(iDocumentNode *parent)
{
  cssWindowManagerBase::SaveSettings(parent);

  // write Map Browser
  csRef<iDocumentNode> mbnode = CreateChildNode(parent, "mapbrowser");
  SaveCoordinates(mbframe, mbnode);
  mbnode->SetAttributeAsInt("show", mbframe->IsShown());
  
  // write csschildframes
  csRef<iDocumentNode> framenodes = CreateChildNode(parent, "frames");
  for(unsigned int i = 0; i < frames.Length(); i++)
  {
    csRef<iDocumentNode> cfn = CreateChildNode(framenodes, "frame");
    WriteSingleFrame(cfn, frames.Get(i));
  }
}

void cssWindowManagerMultiframe::WriteSingleFrame(iDocumentNode *parent, cssChildFrame *cf)
{
  SaveCoordinates(cf, parent);
  
  csRef<iFactory> f = SCF_QUERY_INTERFACE(cf->GetPlugin(), iFactory);
  const char *classid = f->QueryClassID();
  parent->SetAttribute("classid", classid);
}

void cssWindowManagerMultiframe::LoadSettings(iDocumentNode *parent)
{
  cssWindowManagerBase::LoadSettings(parent);

  GetMapBrowser(parent);
  GetFrames(parent);
}

void cssWindowManagerMultiframe::GetMapBrowser(iDocumentNode *root)
{
  csRef<iDocumentNode> mbnode = root->GetNode("mapbrowser");
  if(mbnode)
  {
    GetCoordinates(mbframe, mbnode);
    mbframe->Show(mbnode->GetAttributeValueAsInt("show"));
  }
}

void cssWindowManagerMultiframe::GetFrames(iDocumentNode *root)
{
  CloseViews();
  csRef<iDocumentNode> framesnode = root->GetNode("frames");
  for(csRef<iDocumentNodeIterator> it = framesnode->GetNodes(); it->HasNext(); )
  {
    csRef<iDocumentNode> frmnode = it->Next();
    GetSingleFrame(frmnode);
  }
}

void cssWindowManagerMultiframe::GetSingleFrame(iDocumentNode *framenode)
{
  const char *classid = framenode->GetAttributeValue("classid");
  cssChildFrame *cf = CreateChildFrame(classid);
 
  GetCoordinates(cf, framenode);
  
  cf->Show(true);
}

void cssWindowManagerMultiframe::CreateView(const char *classid)
{ 
  cssChildFrame *cf = CreateChildFrame(classid);
  cf->Show();
}

cssChildFrame* cssWindowManagerMultiframe::CreateChildFrame(const char *classid)
{
  cssChildFrame *newframe;
  
  newframe = new cssChildFrame(mainframe, "child", wxPoint(0, 0), wxSize(400, 400), 
                              wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE, this);

  if(!newframe->CreateCanvas((char*)classid, csi))
  {
    newframe->Destroy();
    return 0;
  }	
  
  Add(newframe);
  return newframe;
}

void cssWindowManagerMultiframe::CloseViews()
{
  for(unsigned int i = 0; i < frames.Length(); i++)
  {
    cssChildFrame *cf = frames.Get(i);
    cf->DestroyCanvas();
    cf->Destroy();
  }
  frames.DeleteAll();
}

void cssWindowManagerMultiframe::Activate(wxFrame *mainframe)
{
  cssWindowManagerBase::Activate(mainframe);

  mbframe = new cssMapBrowserFrame(csi, this, mainframe, -1, "Map Browser");
  mbframe->ReloadPluginPages();
  
  cssWindowManagerBase::LoadSettings();

  csRef<icssPlugin> cmd = csi->GetPlugin("cstudio.cmd.console");
  
  if(cmd)
  {
    csRef<icssCreateView> view = SCF_QUERY_INTERFACE(cmd, icssCreateView);
  
    int w, h;
    mainframe->GetClientSize(&w, &h);
    consolewnd = view->Create(mainframe, "", 0,0,0,0);
    consolewnd->SetSize(0,0,w,h);
  }
  
  AppendMenuItems();
  
}

void cssWindowManagerMultiframe::Deactivate()
{
  cssWindowManagerBase::SaveSettings();
  CloseViews();

  mbframe->UnloadPluginPages();
  mbframe->Destroy();
  
  if(consolewnd)
    consolewnd->Destroy();
  
  consolewnd = 0;
  mbframe = 0;

  cssWindowManagerBase::Deactivate();
}

void cssWindowManagerMultiframe::AppendMenuItems()
{
  cssWindowManagerBase::AppendMenuItems();
  
  menu->AppendSeparator();
  
  menu->Append(CSS_WINDOW_CLOSE_VIEWS, "Close All Views");
  menu->Append(CSS_WINDOW_NEW_VIEW, "New View", CreateViewPluginMenu(CSS_WINDOW_NEW_VIEW_START));
  
  menu->Enable(CSS_WINDOW_SHOW_CONSOLE, false);
  menu->Check(CSS_WINDOW_SHOW_PANELS, mbframe->IsShown());
}

