/*
    Copyright (C) 2005 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cssysdef.h"
#include "iutil/document.h"
#include "iutil/stringarray.h"

#include "app/icsstudio.h"
#include "plugins/icssplugin.h"
#include "plugins/icsscreateview.h"

#include "../common/cssmbnotebook.h"
#include "csssingleframe.h"


#include <wx/wx.h>
#include <wx/laywin.h>


CS_IMPLEMENT_PLUGIN;

SCF_IMPLEMENT_FACTORY(cssSingleFrame);



BEGIN_EVENT_TABLE(cssSingleFrameEvtHandler, cssWindowManagerBaseEvtHandler)
  EVT_MENU(CSS_WINDOW_CLOSE_CURRENT, cssSingleFrameEvtHandler::OnCloseCurrent)
  EVT_MENU(CSS_WINDOW_CLOSE_OTHERS, cssSingleFrameEvtHandler::OnCloseOthers)
  EVT_MENU_RANGE(CSS_WINDOW_SET_CURRENT_START, CSS_WINDOW_SET_CURRENT_END, 
    cssSingleFrameEvtHandler::OnSetCurrent)
  EVT_SIZE(cssSingleFrameEvtHandler::OnSize)
  EVT_ERASE_BACKGROUND(cssSingleFrameEvtHandler::OnErase)
  EVT_SASH_DRAGGED_RANGE(CSS_SASHWINDOW_PANELS, CSS_SASHWINDOW_CONSOLE, 
    cssSingleFrameEvtHandler::OnSashDragged)
END_EVENT_TABLE()



cssSingleFrame::cssSingleFrame(iBase *parent) 
  : cssWindowManagerBase(parent), 
    tabwnd(0)
{
}


cssSingleFrame::~cssSingleFrame()
{
}

wxEvtHandler *cssSingleFrame::CreateEventHandler()
{
  return new cssSingleFrameEvtHandler(this);
}


void cssSingleFrame::Activate(wxFrame* mainframe)
{
  cssWindowManagerBase::Activate(mainframe);
  
  sashwnd_console = new wxSashLayoutWindow(
      mainframe, CSS_SASHWINDOW_CONSOLE,
      wxDefaultPosition, wxSize(200, 30), wxCLIP_CHILDREN);

  sashwnd_console->SetDefaultSize(wxSize(100, 100));
  sashwnd_console->SetOrientation(wxLAYOUT_HORIZONTAL);
  sashwnd_console->SetAlignment(wxLAYOUT_BOTTOM);
  sashwnd_console->SetSashVisible(wxSASH_TOP, TRUE);
 
  
  sashwnd_panels = new wxSashLayoutWindow(
      mainframe, CSS_SASHWINDOW_PANELS,
      wxDefaultPosition, wxSize(200, 30), wxCLIP_CHILDREN);

  sashwnd_panels->SetDefaultSize(wxSize(200, 100));
  sashwnd_panels->SetOrientation(wxLAYOUT_VERTICAL);
  sashwnd_panels->SetAlignment(wxLAYOUT_RIGHT);
  sashwnd_panels->SetSashVisible(wxSASH_LEFT, TRUE);


  cssMapBrowserNotebook *sp = new cssMapBrowserNotebook(csi, sashwnd_panels);
  sp->LoadTabsfromPlugins();

  csRef<icssPlugin> cmd = csi->GetPlugin("cstudio.cmd.console");
  if(cmd)
  {
    csRef<icssCreateView> view = SCF_QUERY_INTERFACE(cmd, icssCreateView);
    view->Create(sashwnd_console, "", 0,0,0,0);
  }  

  
  tabwnd = new cssTabViewWnd(mainframe, csi, wxPoint(0,0), wxSize(0,0)),
  
  cssWindowManagerBase::LoadSettings();    
    
  if(!tabwnd->GetPageCount())
    tabwnd->AddPage("cstudio.view.splitview4");
  
  Layout();
  
  AppendMenuItems();      
}


void cssSingleFrame::Deactivate()
{
  cssWindowManagerBase::SaveSettings();
  
  sashwnd_panels->Destroy();
  sashwnd_console->Destroy();
  tabwnd->DeleteAllPages();
  tabwnd->Destroy();
    
  cssWindowManagerBase::Deactivate();
}


void cssSingleFrame::LoadSettings(iDocumentNode *parent)
{
  csRef<iDocumentNode> panelsnode = parent->GetNode("panels");
  if(panelsnode)
  {
    sashwnd_panels->SetDefaultSize(wxSize(panelsnode->GetAttributeValueAsInt("width"), 10));
    sashwnd_panels->Show(panelsnode->GetAttributeValueAsInt("show"));
  }

  csRef<iDocumentNode> consolenode = parent->GetNode("console");
  if(consolenode)
  {
    sashwnd_console->SetDefaultSize(wxSize(10, consolenode->GetAttributeValueAsInt("heigth")));
    sashwnd_console->Show(consolenode->GetAttributeValueAsInt("show"));
  }
    
  csRef<iDocumentNode> tabnode = parent->GetNode("tabs");
  if(tabnode)
    tabwnd->Load(tabnode);

  cssWindowManagerBase::LoadSettings(parent);    
}


void cssSingleFrame::SaveSettings(iDocumentNode *parent)
{
  cssWindowManagerBase::SaveSettings(parent);
  
  csRef<iDocumentNode> panelsnode = CreateChildNode(parent, "panels");
  panelsnode->SetAttributeAsInt("width", sashwnd_panels->GetSize().GetWidth());
  panelsnode->SetAttributeAsInt("show", sashwnd_panels->IsShown());

  csRef<iDocumentNode> consolenode = CreateChildNode(parent, "console");
  consolenode->SetAttributeAsInt("heigth", sashwnd_console->GetSize().GetHeight());
  consolenode->SetAttributeAsInt("show", sashwnd_console->IsShown());
  
  csRef<iDocumentNode> tabnode = CreateChildNode(parent, "tabs");
  tabwnd->Save(tabnode);
}


void cssSingleFrame::CreateView(const char *classid)
{
  tabwnd->AddPage(classid);
  UpdateMenu();
}


void cssSingleFrame::Layout()
{
  wxLayoutAlgorithm layout;
  layout.LayoutFrame(mainframe, tabwnd);
}


void cssSingleFrame::ShowConsole(bool show)
{ 
  sashwnd_console->Show(show);
  Layout(); 
}


void cssSingleFrame::ShowPanels(bool show)
{ 
  sashwnd_panels->Show(show);
  Layout();
}


void cssSingleFrame::UpdateMenu()
{
  menu->Enable(CSS_WINDOW_CLOSE_CURRENT, tabwnd->GetPageCount() > 1);
  menu->Enable(CSS_WINDOW_CLOSE_OTHERS, tabwnd->GetPageCount() > 1);
}


void cssSingleFrame::AppendMenuItems()
{
  cssWindowManagerBase::AppendMenuItems();

  menu->AppendSeparator();
  menu->Append(CSS_WINDOW_SET_CURRENT, "View", CreateViewPluginMenu(CSS_WINDOW_SET_CURRENT_START));
  menu->Append(CSS_WINDOW_CLOSE_CURRENT, "Close Tab");  
  menu->Append(CSS_WINDOW_CLOSE_OTHERS, "Close Other Tabs");

  menu->AppendSeparator();
  menu->Append(CSS_WINDOW_NEW_VIEW, "New Tab", CreateViewPluginMenu(CSS_WINDOW_NEW_VIEW_START));  
 
  UpdateMenu();

  menu->Check(CSS_WINDOW_SHOW_PANELS, sashwnd_panels->IsShown());
  menu->Check(CSS_WINDOW_SHOW_CONSOLE, sashwnd_console->IsShown());  
}


void cssSingleFrame::OnCloseOthers(wxCommandEvent &event)
{
  tabwnd->DeleteOtherPages(tabwnd->GetSelection());
  UpdateMenu();
}


void cssSingleFrame::OnSetCurrent(wxCommandEvent &event)
{
  int id = event.GetId() - CSS_WINDOW_SET_CURRENT_START;

  csRef<iStringArray> views = iSCF::SCF->QueryClassList("cstudio.view.");
  views->Sort();

  tabwnd->ReplacePage(tabwnd->GetSelection(), views->Get(id));
}


void cssSingleFrame::OnCloseCurrent(wxCommandEvent &event)
{  
  tabwnd->DeletePage(tabwnd->GetSelection());
  UpdateMenu();
}

  
void cssSingleFrame::OnSize(wxSizeEvent &event)
{
  Layout();
}


void cssSingleFrame::OnErase(wxEraseEvent &event)
{
}


void cssSingleFrame::OnSashDragged(wxSashEvent &event)
{
  if(event.GetDragStatus() == wxSASH_STATUS_OK)
  {
    wxRect r = event.GetDragRect();
    wxSize sz (r.GetWidth(), r.GetHeight());
    
    if(event.GetId() == CSS_SASHWINDOW_PANELS)
      sashwnd_panels->SetDefaultSize(sz);
    if(event.GetId() == CSS_SASHWINDOW_CONSOLE)
      sashwnd_console->SetDefaultSize(sz);
      
    Layout();      
  }
}
