/*
    Copyright (C) 2005 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSSSINGLEFRAME_H_INCLUDED__
#define __CSSSINGLEFRAME_H_INCLUDED__

#include "../common/csswmbase.h"
#include "csstabviewwnd.h"


struct icssCreateView;
class wxSashLayoutWindow;
struct wxSashEvent;


enum
{
  CSS_WINDOW_SET_CURRENT = CSS_WINDOW_COMMON_END + 1,
  CSS_WINDOW_SET_CURRENT_START,
  CSS_WINDOW_SET_CURRENT_END = CSS_WINDOW_SET_CURRENT_START + CSS_MAX_VIEWPLUGIN_COUNT,
  CSS_WINDOW_CLOSE_CURRENT,
  CSS_WINDOW_CLOSE_OTHERS,
  CSS_SASHWINDOW_PANELS,
  CSS_SASHWINDOW_CONSOLE
};


class cssSingleFrame : public cssWindowManagerBase
{
  wxSashLayoutWindow *sashwnd_panels, *sashwnd_console;
  cssTabViewWnd *tabwnd;

  virtual wxEvtHandler *CreateEventHandler();
  
  virtual void ShowPanels(bool show);
  virtual void ShowConsole(bool show);
  
  virtual void CreateView(const char *classid);
  
  void UpdateMenu();

  void Layout();

protected:

  void AppendMenuItems();

  void LoadSettings(iDocumentNode *parent);
  void SaveSettings(iDocumentNode *parent);

public:
  cssSingleFrame(iBase *parent);
  ~cssSingleFrame();

  // icssPlugin
  virtual const char *GetName()
    { return "cstudio.windowmanager.singleframe"; }

  // icssWindowManager
  virtual void Activate(wxFrame* mainframe);
  virtual void Deactivate();
  
  void OnSize(wxSizeEvent &event);
  void OnErase(wxEraseEvent &event);
  void OnSashDragged(wxSashEvent &event);
  
  void OnSetCurrent(wxCommandEvent &event);
  void OnCloseOthers(wxCommandEvent &event);
  void OnCloseCurrent(wxCommandEvent &event);
};



class cssSingleFrameEvtHandler : public cssWindowManagerBaseEvtHandler
{
protected:

  void OnSize(wxSizeEvent &event) { ((cssSingleFrame*)mb)->OnSize(event); }
  void OnErase(wxEraseEvent &event) { ((cssSingleFrame*)mb)->OnErase(event); }
  void OnSashDragged(wxSashEvent &event) { ((cssSingleFrame*)mb)->OnSashDragged(event); }
  
  void OnCloseCurrent(wxCommandEvent &event) { ((cssSingleFrame*)mb)->OnCloseCurrent(event); }
  void OnCloseOthers(wxCommandEvent &event) { ((cssSingleFrame*)mb)->OnCloseOthers(event); }
  
  void OnSetCurrent(wxCommandEvent &event) 
  { ((cssSingleFrame*)mb)->OnSetCurrent(event); }
  
public:
  
  cssSingleFrameEvtHandler(cssWindowManagerBase *mb) :
     cssWindowManagerBaseEvtHandler(mb)
  {
  }
  
  DECLARE_EVENT_TABLE()
};

#endif
