/*
    Copyright (C) 2005 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cssysdef.h"
#include "csutil/ref.h"
#include "iutil/document.h"
#include "app/icsstudio.h"
#include "plugins/icssplugin.h"
#include "plugins/icsscreateview.h"
#include "csstabviewwnd.h"

#include <wx/artprov.h>


cssTabViewWnd::cssTabViewWnd(wxWindow *parent, iCStudioCS *csi, wxPoint pos, wxSize size) : 
    cssTabWindow(parent, -1, pos, size, wxCLIP_CHILDREN, true)
{
  this->csi = csi;
}


cssTabViewWnd::~cssTabViewWnd()
{
}


void cssTabViewWnd::AddPage(const char *classid)
{
  csRef<icssPlugin> plg = csi->GetPlugin(classid);
  
  csRef<icssCreateView> view = SCF_QUERY_INTERFACE(plg, icssCreateView);
  wxWindow *wnd =  view->Create(this, "", 0, 0, 0, 0);

  cssTabWindow::AddPage(wnd, classid, wxArtProvider::GetBitmap(wxART_EXECUTABLE_FILE));
  views.Push(view);
}  


void cssTabViewWnd::ReplacePage(size_t idx, const char *classid)
{
  csRef<icssPlugin> plg = csi->GetPlugin(classid);
  
  csRef<icssCreateView> view = SCF_QUERY_INTERFACE(plg, icssCreateView);
  wxWindow *wnd =  view->Create(this, "", 0, 0, 0, 0);
  
  cssTabWindow::ReplacePage(idx, wnd, classid, wxArtProvider::GetBitmap(wxART_EXECUTABLE_FILE));  
  views.Put(idx, view);
}


void cssTabViewWnd::DeleteAllPages()
{
  cssTabWindow::DeleteAllPages();
  views.DeleteAll();
}


void cssTabViewWnd::DeletePage(int idx)
{
  cssTabWindow::DeletePage(idx);
  views.DeleteIndex(idx);
}


void cssTabViewWnd::DeleteOtherPages(size_t idx)
{
  cssTabWindow::DeleteOtherPages(idx);
  csRef<icssCreateView> view = views[idx];
  views.DeleteAll();
  views.Push(view);
}


bool cssTabViewWnd::Load(iDocumentNode* parent)
{
  DeleteAllPages();
  
  csRef<iDocumentNodeIterator> it = parent->GetNodes("tab");
  for(size_t i = 0; it->HasNext(); i++)
  {
    csRef<iDocumentNode> node = it->Next();
    AddPage(node->GetAttributeValue("classid"));
    views[i]->LoadXML(GetPage(i), node);
  }
  return true;
}


bool cssTabViewWnd::Save(iDocumentNode* parent)
{
  for(size_t i = 0; i < views.Length(); i++)
  {
    csRef<icssPlugin> plg = SCF_QUERY_INTERFACE(views[i], icssPlugin);
    
    csRef<iDocumentNode> node = parent->CreateNodeBefore(CS_NODE_ELEMENT);
    node->SetValue("tab");
    node->SetAttribute("classid", plg->GetName());
    
    views[i]->SaveXML(GetPage(i), node);
  }
  return true;
}
