/*
    Copyright (C) 2005 by Jan Lepper

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSSTABWINDOW_H_INCLUDED__
#define __CSSTABWINDOW_H_INCLUDED__

#include "csutil/array.h"
#include "csutil/stringarray.h"

#include <wx/wx.h>


enum
{
  CSS_TABWINDOW_BACK,
  CSS_TABWINDOW_FORWARD
};


class cssTabWindow : public wxWindow
{
  csArray<wxWindow*> pages;
  csArray<wxBitmap> bitmaps;
  csStringArray labels;
  
  wxBitmapButton *b_back, *b_forward;
  
  int width, height;
  int tab_height;
  int tabbar_y;
  int tab_y;
  int active_page;
  size_t first_tab;
  bool bottom;
  bool show_buttons;


  int GetTabWidth(int idx);
  
  void DrawTabs(wxDC  &dc);  
  void LayoutPage();
  void UpdateButtons();
  
public:
  
  cssTabWindow(wxWindow *parent, int id, wxPoint pos, wxSize size, int style,  bool bottom);
  
  size_t GetPageCount() { return pages.Length(); }
  int GetSelection() { return active_page; }
  wxWindow *GetPage(size_t idx);  
  void SetActivePage(size_t idx);  
  int GetTabAt(int x, int y);    
  
  virtual void AddPage(wxWindow* page, const char* title, const wxBitmap &bitmap);
  virtual void ReplacePage(size_t idx, wxWindow* new_page, const char* title, const wxBitmap &bitmap);
  virtual void DeleteOtherPages(size_t idx);
  virtual void DeleteAllPages();
  virtual void DeletePage(size_t idx);
  
  virtual bool Layout();  
  
  void OnSize(wxSizeEvent &event) { Layout(); }
  void OnErase(wxEraseEvent &event);
  void OnPaint(wxPaintEvent &event);
  
  void OnLeftDown(wxMouseEvent &event);
  void OnButtonBack(wxCommandEvent &event);
  void OnButtonForward(wxCommandEvent &event);

  DECLARE_EVENT_TABLE()
};

#endif
