#include "cssysdef.h"
#include "cst-config.h"

#include "cssProgressMeter.h"

#include "ivideo/graph3d.h"
#include "ivideo/txtmgr.h"
#include "ivaria/conout.h"
#include "ivaria/view.h"
#include "iengine/engine.h"
#include "iengine/sector.h"
#include "iengine/light.h"
#include "iengine/camera.h"
#include "iengine/mesh.h"
#include "iengine/movable.h"
#include "imesh/object.h"
#include "ivaria/reporter.h"
#include "csutil/cscolor.h"
#include "csgeom/path.h"
#include "cstool/csfxscr.h"

#include <wx/gauge.h>


/**************************/
SCF_IMPLEMENT_IBASE (cssProgressMeter)
  SCF_IMPLEMENTS_INTERFACE (iProgressMeter)
SCF_IMPLEMENT_IBASE_END
/*************************/

cssProgressMeter :: cssProgressMeter (int n, wxGauge *g )
	: granularity(10), total(n), current(0), gauge(g)
{
  SCF_CONSTRUCT_IBASE (0);
}

cssProgressMeter :: ~cssProgressMeter()
{
  SCF_DESTRUCT_IBASE ();
}

void cssProgressMeter :: SetProgressDescription (const char* id,
	const char* description, ...)
{
  va_list arg;
  va_start (arg, description);
  SetProgressDescriptionV (id, description, arg);
  va_end (arg);
}

void cssProgressMeter :: SetProgressDescriptionV (const char* /*id*/,
	const char* description, va_list list)
{
  vsprintf (cur_description, description, list);
}

void cssProgressMeter :: Step(unsigned int step)
{
  if (current < total)
  {
    //current++;
		current += granularity;
		gauge->SetValue(current);
	}
}

void cssProgressMeter::Restart()
{
  Reset();
}

void cssProgressMeter::Abort ()
{
  current = total;
}

void cssProgressMeter::Finalize ()
{
  current = total;
}

void cssProgressMeter::SetGranularity(int gran)
{
  if (gran >= 1 && gran <= 100)
    granularity = gran;
}
