/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSSPROGRESSMETER_H__
#define __CSSPROGRESSMETER_H__

#include <stdarg.h>
#include "csutil/scf.h"
#include "ivaria/pmeter.h"

#include <wx/gauge.h>

class cssProgressMeter : public iProgressMeter
{
private:
  int granularity;
  int total;
  int current;
  char cur_description[256];
	wxGauge *gauge;

public:
	cssProgressMeter (int total = 100, wxGauge *g = NULL );
  virtual ~cssProgressMeter ();

  SCF_DECLARE_IBASE;

	virtual void SetProgressDescription (const char*, const char*, ...);
  virtual void SetProgressDescriptionV (const char*, const char*, va_list);
  virtual void Step (unsigned int step);
  virtual void Reset () { gauge->SetValue(0); current = 0;  }
  virtual void Restart ();
  virtual void Abort ();
  virtual void Finalize ();
  virtual void SetTotal (int n) { gauge->SetRange(n); total = n; Reset(); }
  virtual int GetTotal () const { return total; }
  virtual int GetCurrent () const { return current; }
  virtual void SetGranularity (int);
  virtual int GetGranularity () const { return granularity; }
};

#endif
