/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "cst-config.h"
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif

#include <wx/gdicmn.h>
#include <wx/stattext.h>
#include <wx/artprov.h>
#include <wx/confbase.h>
#include <wx/dcscreen.h>

#include "cssysdef.h"
#include "cssmainframe.h"
#include "cssvfsfiledlg.h"
#include "cstudiocs.h"
#include "iutil/stringarray.h"
#include "iutil/objreg.h"
#include "iengine/engine.h"
#include "imap/loader.h"

#include "plugins/icsscreatetoolwindow.h"
#include "app/icsseditmodemanager.h"
#include "plugins/icsseditmode.h"
#include "plugins/icssplugin.h"
#include "plugins/icsscommandmanager.h"
#include "plugins/icssactionmanager.h"
#include "plugins/icsswindowmanager.h"


BEGIN_EVENT_TABLE (cssMainFrame, wxFrame)
  EVT_CLOSE (cssMainFrame::OnWindowClose)

  //File Menu Events
  EVT_MENU ( CSS_FILE_EXIT,  cssMainFrame::OnFileExit)
  EVT_MENU ( CSS_FILE_OPEN,  cssMainFrame::OnFileOpen)
  EVT_MENU ( CSS_FILE_SAVE,  cssMainFrame::OnFileSave)
  EVT_MENU ( CSS_FILE_SAVEAS,cssMainFrame::OnFileSaveAs)
  EVT_MENU ( CSS_EDIT_UNDO,  cssMainFrame::OnEditUndo)
  EVT_MENU ( CSS_EDIT_REDO,  cssMainFrame::OnEditRedo)

  // Window Manager Menu Events
  EVT_MENU_RANGE (CSS_WINDOWMANAGER_START, CSS_WINDOWMANAGER_END,
    cssMainFrame::OnWindowManagerRange)
  // Edit Mode Menu Events
  EVT_MENU_RANGE (CSS_EDITMODE_START, CSS_EDITMODE_END, cssMainFrame::OnEditModeRange)

  // Engine Menu Events
  EVT_MENU ( CSS_ENGINE_RELIGHT_ALL, cssMainFrame::OnRelightAll )
  EVT_MENU ( CSS_ENGINE_IMPORT_LIBRARY, cssMainFrame::OnImportLibrary )
END_EVENT_TABLE()

#include "images/cstudio.xpm"



cssMainFrame::cssMainFrame( const wxChar *title,  CStudioCS *csi, wxPoint pos, wxSize sz) :
                wxFrame
                ( (wxFrame *) NULL,
                   -1,
                   title,
                   pos,
                   sz,
                   wxDEFAULT_FRAME_STYLE
                )

{
  this->csi = csi;
  fileMenu = NULL;
  menuBar = NULL;
  statusBar = NULL;
  
  wxIcon icon (wxICON (cstudio));
  SetIcon (icon);

  wxBitmap bmFileOpen (wxArtProvider::GetBitmap (wxART_FILE_OPEN,
    wxART_TOOLBAR));

  wxMenuItem* item;
  // Create the menu
  // file menu
  fileMenu = new wxMenu();
  item = new wxMenuItem (fileMenu, CSS_FILE_OPEN, "&Open...\tCtrl-O");
  item->SetBitmap (bmFileOpen);
  fileMenu->Append (item);
  fileMenu->AppendSeparator();
  fileMenu->Append( CSS_FILE_SAVE, "&Save...\tCtrl-S" );
  fileMenu->Append( CSS_FILE_SAVEAS, "Save &As..." );
  fileMenu->AppendSeparator();
  fileMenu->Append(CSS_FILE_EXIT, "E&xit");

  // edit menu
  wxMenu *eMenu = new wxMenu();
  eMenu->Append(CSS_EDIT_UNDO, "&Undo...");
  eMenu->Append(CSS_EDIT_REDO, "&Redo...");  

  // edit mode menu
  emMenu = this->AppendEditModePluginList();

  
  // windowmanager menu
  wxMenu *wmMenu = AppendWindowManagerList();

  // engine menu
  engineMenu = new wxMenu();
  engineMenu->Append(CSS_ENGINE_RELIGHT_ALL, "Relight All");
  engineMenu->Append(CSS_ENGINE_IMPORT_LIBRARY, "Import Library");

  menuBar = new wxMenuBar;
  menuBar->Append(fileMenu, "&File");
  menuBar->Append(eMenu, "&Edit");
  menuBar->Append(emMenu, "Edit&Mode");
  menuBar->Append(engineMenu, "&Engine");
  menuBar->Append(wmMenu, "&Window Manager");

  SetMenuBar(menuBar);
  // Done creating menu

  toolBar = CreateToolBar (wxTB_FLAT | wxTB_HORIZONTAL);
  toolBar->AddTool (CSS_FILE_OPEN, _("Open File"), bmFileOpen);
  toolBar->Realize ();

  CreateStatusBar(2);
  statusBar = GetStatusBar();
  statusBar->SetStatusText( "CStudio" );

  static const int widths[SFIELD_MAX] = { -1, 150, -4 };

  statusBar->SetFieldsCount(SFIELD_MAX);
  statusBar->SetStatusWidths(SFIELD_MAX, widths);

  wxRect feildRect;
  statusBar->GetFieldRect(SFIELD_GAUGE, feildRect);

  gauge = new wxGauge(statusBar, -1, 100, feildRect.GetPosition(), feildRect.GetSize(), wxGA_SMOOTH);

  ActivateAcceleratorTable();
}

cssMainFrame::~cssMainFrame()
{
}

void cssMainFrame::OnWindowClose (wxCloseEvent& event)
{
  if(windowmanager)
  {
    csRef<icssPlugin> plg = SCF_QUERY_INTERFACE(windowmanager, icssPlugin);
    wxConfigBase::Get()->Write("windowmanager", plg->GetName());
    windowmanager->Deactivate();
  }
  windowmanager = 0;
  
  csi->shutdown();

  Destroy();
}  

void cssMainFrame::OnFileExit (wxCommandEvent & event)
{
  Close(TRUE);
}

void cssMainFrame::OnFileOpen (wxCommandEvent & event)
{
  cssVFSFileDlg dlg(this, -1, "Open Map File", wxDefaultPosition, wxDefaultSize, 
                    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER, csi, "/", "/", VFS_OPEN );
  if(dlg.ShowModal())
  {
    SetStatusText("Loading map ...");
    if(csi->LoadMap((char*)dlg.GetPathData(), (char*)dlg.GetFileNameData(), gauge ))
      SetStatusText("Map loaded successfully.");
    else
    {
      SetStatusText("Error loading map.");
      wxMessageBox("The mapfile couldn't be loaded.", "Error loading map.", wxICON_ERROR);
    }
  }
}

void cssMainFrame :: OnFileSave ( wxCommandEvent & event )
{
  //Check to see if map is already saved or not
  //If map already exists then skip showing dialog
  //and go directly to saving

  cssVFSFileDlg dlg(this, -1, "Save", wxDefaultPosition, wxDefaultSize, 
                    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER, csi, "/", "/", VFS_SAVE );
  if(dlg.ShowModal())
  {
    csi->SaveMap((char*)dlg.GetPathData(), (char*)dlg.GetFileNameData());
  }
}

void cssMainFrame :: OnFileSaveAs ( wxCommandEvent & event )
{
  cssVFSFileDlg dlg(this, -1, "Save As", wxDefaultPosition, wxDefaultSize, 
                    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER, csi, "/", "/", VFS_SAVE );
  if(dlg.ShowModal())
  {
    csi->SaveMap((char*)dlg.GetPathData(), (char*)dlg.GetFileNameData());
  }
}

void cssMainFrame::OnEditUndo(wxCommandEvent & event)
{
  csRef<icssPlugin> amplg = csi->GetPlugin("cstudio.plugin.actionmgr");
  csRef<icssActionManager> am = SCF_QUERY_INTERFACE(amplg, icssActionManager);
  am->Undo();
}

void cssMainFrame::OnEditRedo(wxCommandEvent & event)
{
  csRef<icssPlugin> amplg = csi->GetPlugin("cstudio.plugin.actionmgr");
  csRef<icssActionManager> am = SCF_QUERY_INTERFACE(amplg, icssActionManager);
  am->Redo();
}

wxMenu *cssMainFrame::AppendWindowManagerList()
{
  wxMenu * m = new wxMenu();

  csRef<iStringArray> wm = iSCF::SCF->QueryClassList("cstudio.windowmanager.");
  wm->Sort();

  size_t count = wm->Length();
  for(size_t i = 0; i < count; i++)
  {
    m->Append(CSS_WINDOWMANAGER_START + i, wm->Get(i));
  }
  return m;
}

wxMenu *cssMainFrame::AppendEditModePluginList()
{
  wxMenu * m = new wxMenu();
  csRef<iStringArray> emstr = iSCF::SCF->QueryClassList("cstudio.em.");
  emstr->Sort();

  csRef<icssPlugin> emplug = csi->GetPlugin("cstudio.plugin.emmanager");
  csRef<icssEditModeManager> emmgr = SCF_QUERY_INTERFACE(emplug, icssEditModeManager);

  size_t count = emstr->Length();
  for(size_t i = 0; i < count; i++)
  {
    const char *emclassid = emstr->Get(i);
    csRef<icssEditMode> em = emmgr->GetEM((const char*)&emclassid[strlen("cstudio.em.")]);
    m->Append(CSS_EDITMODE_START + i, em->GetDescription());
  }
  return m;
}

void cssMainFrame::OnWindowManagerRange(wxCommandEvent &event)
{
  int id = event.GetId() - CSS_WINDOWMANAGER_START;

  csRef<iStringArray> wmlist = iSCF::SCF->QueryClassList("cstudio.windowmanager.");
  wmlist->Sort();

  SetWindowManager(wmlist->Get(id));

  event.Skip();
}

void cssMainFrame::OnEditModeRange (wxCommandEvent &event)
{
  int id = event.GetId() - CSS_EDITMODE_START;
  csRef<iStringArray> emstr = iSCF::SCF->QueryClassList("cstudio.em.");
  emstr->Sort();

  const char *emclassid = emstr->Get(id);

  ActivateEditMode((const char*)&emclassid[strlen("cstudio.em.")]);
}

void cssMainFrame::ActivateEditMode(const char *name)
{
  csRef<icssPlugin> emplug = csi->GetPlugin("cstudio.plugin.emmanager");
  csRef<icssEditModeManager> emmgr = SCF_QUERY_INTERFACE(emplug, icssEditModeManager);
  csRef<icssEditMode> em = emmgr->GetEM(name);
  emmgr->SetActiveEditMode(em);	
}

void cssMainFrame::ActivateAcceleratorTable()
{
  wxAcceleratorEntry entries[1];
  entries[0].Set(wxACCEL_CTRL,  (int) 'W', CSS_WINDOW_CLOSE_CURRENT);
  wxAcceleratorTable accel(1, entries);
  SetAcceleratorTable(accel); 
}

void cssMainFrame::SetWindowManager(const char* name)
{
  if(windowmanager)
  {
    csRef<icssPlugin> plg = SCF_QUERY_INTERFACE(windowmanager, icssPlugin);
    if(strcmp(plg->GetName(), name) == 0)
      return;
    windowmanager->Deactivate();
  }

  csRef<icssPlugin> wm = csi->GetPlugin(name);
  if(wm)
  {
    windowmanager = SCF_QUERY_INTERFACE(wm, icssWindowManager);
    windowmanager->Activate(this);
  }
}

void cssMainFrame::OnRelightAll (wxCommandEvent &event)
{
  csRef<iEngine> e = CS_QUERY_REGISTRY(csi->getObjectRegistry(), iEngine);
  e->ForceRelight();
}

void cssMainFrame::OnImportLibrary (wxCommandEvent &event)
{
   cssVFSFileDlg dlg(this, -1, "Import Library", wxDefaultPosition, wxDefaultSize, 
                    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER, csi, "/", "/", VFS_OPEN );
  if(dlg.ShowModal())
  {
    csRef<iLoader> loader = CS_QUERY_REGISTRY(csi->getObjectRegistry(), iLoader);
    csString s = dlg.GetPathData();
    s.Append(dlg.GetFileNameData());
    loader->LoadLibraryFile(s.GetData());
    
  }
}

#if 0
void cssMainFrame::OnSaveLayout (wxCommandEvent &event)
{
  cssVFSFileDlg dlg(this, -1, "Save Layout as...", wxDefaultPosition, wxDefaultSize,
                      wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER, csi, "/", "/", VFS_SAVE );
  if(dlg.ShowModal())
  {
    csString s = dlg.GetPathData();
    s.Append(dlg.GetFileNameData());
    windowmanager->Save(s.GetData());
  }
}

void cssMainFrame::OnLoadLayout (wxCommandEvent &event)
{
  cssVFSFileDlg dlg(this, -1, "Load Layout...", wxDefaultPosition, wxDefaultSize,
                    wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER, csi, "/", "/", VFS_OPEN );

  if(dlg.ShowModal())
  {
    csString s = dlg.GetPathData();
    s.Append(dlg.GetFileNameData());
    windowmanager->Load(s.GetData());
  }
}
#endif
