/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef __CSSMAINFRAME_H_INCLUDED__
#define __CSSMAINFRAME_H_INCLUDED__

#include "csutil/weakref.h"
#include "csutil/ref.h"
#include <wx/frame.h>


struct icssView;
struct icssPlugin;
struct icssWindowManager;
class wxGauge;
class wxMenu;
class wxTextCtrl;
class wxButton;
class CStudioCS;


class cssMainFrame : public wxFrame
{
public:
  cssMainFrame( const wxChar *title, CStudioCS *csi, wxPoint pos, wxSize sz);
  ~cssMainFrame();

  //Menu Items
  wxStatusBar *statusBar;
  wxGauge *gauge;
  wxMenu *fileMenu;
  wxMenu *viewMenu;
  wxMenu *emMenu;
  wxMenu *engineMenu;
  wxMenuBar  *menuBar;
  wxToolBar* toolBar;

  // the window manager
  csRef<icssWindowManager> windowmanager;

  wxFrame * view3d;

  CStudioCS* csi;

  void OnWindowClose (wxCloseEvent& event);

  // File Event Methods
  void OnFileExit (wxCommandEvent & event);
  void OnFileOpen (wxCommandEvent & event);
  void OnFileSave (wxCommandEvent & event);
  void OnFileSaveAs (wxCommandEvent & event);
  
  // Edit Event Methods
  void OnEditUndo(wxCommandEvent & event);
  void OnEditRedo(wxCommandEvent & event);  

  // Window Event Methods
  void OnWindowCloseCurrent (wxCommandEvent &event);
  void OnWindowManagerRange (wxCommandEvent &event);
  
  // Edit Mode Event Methods
  void OnEditModeRange (wxCommandEvent &event);
  void ActivateEditMode(const char *name);

  // Engine Event Methods
  void OnRelightAll (wxCommandEvent &event);
  void OnImportLibrary (wxCommandEvent &event);

  wxMenu *AppendWindowManagerList();
  wxMenu *AppendEditModePluginList();

  void UpdateAllViews();
	
  void ActivateAcceleratorTable();
  
  void SetWindowManager(const char* name);

  DECLARE_EVENT_TABLE()

};

enum
{ 
  // Menu commands

  CSS_FILE_START = 0,
  CSS_FILE_EXIT  =   1,
  CSS_FILE_OPEN,
  CSS_FILE_SAVE,
  CSS_FILE_SAVEAS,
  //EDIT  start: 10
  CSS_EDIT_START = 10,
  CSS_EDIT_UNDO,
  CSS_EDIT_REDO,


  CSS_WINDOW_CLOSE_CURRENT,  
  
  CSS_WINDOWMANAGER,
  CSS_WINDOWMANAGER_START, 
  CSS_WINDOWMANAGER_END = CSS_WINDOWMANAGER_START + 10,

  //EDITMODE start
  CSS_EDITMODE,
  CSS_EDITMODE_START,
  CSS_EDITMODE_END = CSS_EDITMODE_START + 20,


  // ENGINE start
  CSS_ENGINE = 100,
  CSS_ENGINE_RELIGHT_ALL,
  CSS_ENGINE_IMPORT_LIBRARY,

  // WINDOW start
  CSS_WINDOW_START = 200,
 
};

enum
{
  //StatusBar Items
  SFIELD_NAME,
  SFIELD_BLANK,
  SFIELD_GAUGE,
  SFIELD_MAX
};

#define CSS_WINDOW_SETTINGS_PATH "/cstudio/config/cstudio_windowsettings.xml"

#endif
