/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "cst-config.h"
// For compilers that supports precompilation , includes "wx/wx.h"
#include "wx/wxprec.h"

#ifndef WX_PRECOMP
        #include "wx/wx.h"
#endif

#include <wx/confbase.h>
#include <wx/fileconf.h>
#include <wx/image.h>
#include <wx/msgdlg.h>
#include <wx/splash.h>

#include "cssysdef.h"

#include "cstudio.h"


#ifdef WIN32
#include "csutil/win32/win32.h"
#include "windows.h"
#endif

#include "iutil/vfs.h"
#include "csutil/cscolor.h"
#include "cstool/csview.h"
#include "cstool/initapp.h"
#include "csutil/syspath.h"

#include "app/icsseditmodemanager.h"
#include "plugins/icsseditmode.h"

#include "cssmainframe.h"

#include "images/splash.xpm"

IMPLEMENT_APP(CStudioApp)

iObjectRegistry *object_reg;
#if defined(WIN32)

#ifndef SW_SHOWNORMAL
#define SW_SHOWNORMAL 1
#endif

/*
  WX provides WinMain(), but not main(), which is required for console apps.
 */
int main (int argc, const char* const argv[])
{
  int retval = WinMain (GetModuleHandle (0), 0, GetCommandLineA (), 
    SW_SHOWNORMAL);

  csInitializer::DestroyApplication(object_reg);
  return retval;
}

#endif

BEGIN_EVENT_TABLE(CStudioApp, wxApp)
  EVT_IDLE(CStudioApp::OnIdle)
END_EVENT_TABLE()

/* this is executed upon startup, like 'main()' in non-wxWindows programs */
bool CStudioApp::OnInit()
{
  config = new wxFileConfig ("CStudio", _T("CrystalStudio Team"));
  wxConfigBase::Set (config);

  object_reg = csInitializer::CreateEnvironment (argc, argv);
  csi = csPtr<CStudioCS> (new CStudioCS(object_reg));

  wxSplashScreen* splash = new wxSplashScreen(wxBitmap (splash_xpm),
          wxSPLASH_CENTRE_ON_SCREEN |  wxSPLASH_TIMEOUT,
          2000, NULL, -1, wxDefaultPosition, wxDefaultSize,
          wxSIMPLE_BORDER | wxSTAY_ON_TOP);

#if wxUSE_LIBPNG
  wxImage::AddHandler( new wxPNGHandler );
#endif

#if wxUSE_XPM
  wxImage::AddHandler( new wxXPMHandler );
#endif

  wxScreenDC dc;
  wxCoord w, h;
  dc.GetSize(&w, &h);

  cssMainFrame *frame = new cssMainFrame(_T("CStudio"), csi, wxPoint(0,0), wxSize(w, h));
  frame->SetSize(0, 0, w / 2, h / 2);
  SetTopWindow(frame);

  
  csi->mainfrm = frame;

  if (!csi->Init(frame))
  {
    wxMessageDialog dlg (0, _T("Could not initialize CS"), _T("Error"), wxOK);
    dlg.ShowModal ();
    return false;
  }
  
  wxString wm;
  config->Read("windowmanager", &wm, "cstudio.windowmanager.multiframe");
  frame->SetWindowManager(wm);

  frame->Show(TRUE);
  
  //splash->Destroy();

  return true;
}

int CStudioApp::OnExit()
{
  csi->shutdown();
  csi = 0;
  wxConfigBase::Set (0);
  delete config;
  return 0;
}

#if !wxCHECK_VERSION(2,5,2)
#ifdef WIN32
bool CStudioApp::ProcessMessage(WXMSG *wxmsg)
{
  MSG *msg = (MSG *)wxmsg;
  iWin32Assistant* w32assi = csi->win32ass;
  if (w32assi && (w32assi->GetApplicationWindow() == msg->hwnd))
    return false;
  else
    return wxApp::ProcessMessage(wxmsg);
}
#endif
#endif

void CStudioApp::OnIdle(wxIdleEvent& event)
{
  wxApp::OnIdle(event);
  csi->update();
  event.RequestMore();
}
