/*
    Copyright (C) 2003 by Philipp Aumayr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSTUDIOCS_H_INCLUDED__
#define __CSTUDIOCS_H_INCLUDED__

#include <stdarg.h>

#include "csutil/scf.h"
#include "csutil/ref.h"
#include "csutil/refarr.h"
#include <wx/gauge.h>
#include "app/icsstudio.h"
#include "plugins/icssplugin.h"
#include "plugins/icssselection.h"

struct iObjectRegistry;
struct iEvent;
struct iSector;
struct iView;
struct icssTextOutput;
class cssEditModeManager;
class wxWindow;
class cssMainFrame;

/// win32 specific!!! fix me!
struct iWin32Assistant;
/// win32 specific!!! fix me!

/** \brief CStudioCS manages main interaction with CS
*
* CStudioCS has functions to initialize and shutdown
* CS related features. Especially it initializes a window
* owned by CS that will contain a view of the CS level
* (or possible the activated camera)
*/
class CStudioCS : public iCStudioCS
{
  static bool StaticEventHandler (iEvent& ev);
  csRefArray<icssPlugin> m_plugins;
  csRefArray<icssContentCallback> m_contentcb;
  csRefArray<icssActiveSectorCallback> m_activesectorcb;
  csRefArray<icssRedrawViewCallback> m_redrawviewcb;

  csRef<iMaterialWrapper> m_activemat;

  csRef<icssTextOutput> textout;

  wxFrame *view3d;
  bool do_freelook;
  bool initialized;
  
  bool redraw_views;
  
public:

  cssMainFrame *mainfrm;

  SCF_DECLARE_IBASE;

  csRef<iEngine> engine;
  csRef<iLoader> loader;
  csRef<iSaver> saver;
  csRef<iGraphics2D> g2d;
  csRef<iGraphics3D> g3d;
  csRef<iKeyboardDriver> kbd;
  csRef<iVirtualClock> vc;
  csRef<iView> view;
  csRef<iEventQueue> ev_queue;
  csRef<iVFS> vfs;
  csRef<iEngineSequenceManager> engseqmgr;
  csRef<iSector> active_sector;
  
  iObjectRegistry *object_reg;
  #ifdef WIN32
  csRef<iWin32Assistant> win32ass;
  #endif

  virtual iObjectRegistry* getObjectRegistry();
	  
  /// Active Sector
  virtual iSector* getActiveSector();
  virtual void SetActiveSector(iSector *sector);
  virtual void RegisterActiveSectorCB(icssActiveSectorCallback *cb);
  virtual void UnregisterAcitveSectorCB(icssActiveSectorCallback *cb);

  /// Active Material
  virtual iMaterialWrapper* GetActiveMaterial();
  virtual void SetActiveMaterial(iMaterialWrapper *mat);

  /// Plugin
  virtual csRef<icssPlugin> GetPlugin(const char *name);
  virtual size_t GetPluginCount();
  virtual csRef<icssPlugin> GetPlugin(size_t idx);

  /// Content Callbacks
  virtual void RegisterContentCB(icssContentCallback *cb);
  virtual void UnregisterContentCB(icssContentCallback *cb);
  
  /// Redraw View Callbacks
  virtual void RegisterRedrawViewCB(icssRedrawViewCallback *cb);
  virtual void UnregisterRedrawViewCB(icssRedrawViewCallback *cb);

  void RegisterSelection();
  void RegisterConsole();
  size_t LoadPlugins(const char *domain);
  csRef<icssPlugin> LoadPlugin(const char *name);

  
  void LoadStandardMaterials();
  //// icssContentCallback implementation
  void OnMeshListChanged();
  void OnSectorListChanged();
  void OnMaterialListChanged();
  class cssContentCallback : public icssContentCallback
  {
    SCF_DECLARE_EMBEDDED_IBASE(CStudioCS);
    virtual void OnMeshListChanged()
    { scfParent->OnMeshListChanged(); };
    virtual void OnSectorListChanged()
    { scfParent->OnSectorListChanged(); };
    virtual void OnMaterialListChanged()
    { scfParent->OnMaterialListChanged(); };
  }scficssContentCallback;
  friend class cssContentCallback;

  /// icssActiveSectorCallback implementation
  void OnActiveSectorChanged();
  class cssActiveSectorCallback : public icssActiveSectorCallback
  {
    SCF_DECLARE_EMBEDDED_IBASE(CStudioCS);
    virtual void OnActiveSectorChanged()
    { scfParent->OnActiveSectorChanged();  };
  }scficssActiveSectorCallback;
  friend class cssActiveSectorCallback;

  /** default constructor
  */
  CStudioCS(iObjectRegistry *obj_reg);
  /** virtual destructor 
  */
  virtual ~CStudioCS();

  /** \brief initializes the CS instance
  * 
  * opens a window with desired resolution
  */
  bool Init(wxWindow *parent, const char *title = "CS View", 
            int resx = 640, int resy = 480, bool useopengl = true);

  /** \brief handles events from CS
  *
  * CStudio itself can also send events to CS through this
  * \param ev event to be handled
  */
  bool HandleEvent (iEvent& ev);

  /** \brief called at beginning of frame
  * 
  * changes CS data before frame is drawn
  */
  void SetupFrame();

  /** \brief called at end of frame
  *
  * swaps front/backbuffer etc
  */
  void FinishFrame();

  bool LoadMap(char *path, char* file, wxGauge *gauge = NULL);
  bool SaveMap(char *path, char* file);
  void UnloadMap();

  virtual void Print(const char *msg);
  virtual void Printf(const char *format, ...);

  virtual wxFrame *GetView3DFrame()
  { return this->view3d; };

  bool update();
  void RedrawViews();
  int shutdown();
};


#endif
